@@ -31,6 +31,9 @@
  * </refsect2>
  */
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -57,23 +60,19 @@ enum
   PROP_EMBEDDEDFONTS
 };
 
+#define FORMATS "{ RGB, BGR, xRGB, xBGR, RGBx, BGRx, I420 }"
+
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB ";" GST_VIDEO_CAPS_BGR ";"
-        GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR ";"
-        GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx ";"
-        GST_VIDEO_CAPS_YUV ("I420"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (FORMATS))
     );
 
 static GstStaticPadTemplate video_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("video_sink",
+GST_STATIC_PAD_TEMPLATE ("video_sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB ";" GST_VIDEO_CAPS_BGR ";"
-        GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR ";"
-        GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx ";"
-        GST_VIDEO_CAPS_YUV ("I420"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (FORMATS))
     );
 
 static GstStaticPadTemplate text_sink_factory =
@@ -93,42 +92,30 @@ static void gst_ass_render_finalize (GObject * object);
 static GstStateChangeReturn gst_ass_render_change_state (GstElement * element,
     GstStateChange transition);
 
-GST_BOILERPLATE (GstAssRender, gst_ass_render, GstElement, GST_TYPE_ELEMENT);
+#define gst_ass_render_parent_class parent_class
+G_DEFINE_TYPE (GstAssRender, gst_ass_render, GST_TYPE_ELEMENT);
 
-static GstCaps *gst_ass_render_getcaps (GstPad * pad);
+static GstCaps *gst_ass_render_getcaps (GstPad * pad, GstCaps * filter);
 
 static gboolean gst_ass_render_setcaps_video (GstPad * pad, GstCaps * caps);
 static gboolean gst_ass_render_setcaps_text (GstPad * pad, GstCaps * caps);
 
-static GstFlowReturn gst_ass_render_chain_video (GstPad * pad, GstBuffer * buf);
-static GstFlowReturn gst_ass_render_chain_text (GstPad * pad, GstBuffer * buf);
+static GstFlowReturn gst_ass_render_chain_video (GstPad * pad,
+    GstObject * parent, GstBuffer * buf);
+static GstFlowReturn gst_ass_render_chain_text (GstPad * pad,
+    GstObject * parent, GstBuffer * buf);
 
-static gboolean gst_ass_render_event_video (GstPad * pad, GstEvent * event);
-static gboolean gst_ass_render_event_text (GstPad * pad, GstEvent * event);
-static gboolean gst_ass_render_event_src (GstPad * pad, GstEvent * event);
+static gboolean gst_ass_render_event_video (GstPad * pad, GstObject * parent,
+    GstEvent * event);
+static gboolean gst_ass_render_event_text (GstPad * pad, GstObject * parent,
+    GstEvent * event);
+static gboolean gst_ass_render_event_src (GstPad * pad, GstObject * parent,
+    GstEvent * event);
 
-static GstFlowReturn gst_ass_render_bufferalloc_video (GstPad * pad,
-    guint64 offset, guint size, GstCaps * caps, GstBuffer ** buffer);
-
-static gboolean gst_ass_render_query_src (GstPad * pad, GstQuery * query);
-
-static void
-gst_ass_render_base_init (gpointer gclass)
-{
-  GstElementClass *element_class = (GstElementClass *) gclass;
-
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &video_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &text_sink_factory);
-
-  gst_element_class_set_details_simple (element_class, "ASS/SSA Render",
-      "Mixer/Video/Overlay/Subtitle",
-      "Renders ASS/SSA subtitles with libass",
-      "Benjamin Schmitz <vortex@wolpzone.de>, "
-      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
-}
+static gboolean gst_ass_render_query_video (GstPad * pad, GstObject * parent,
+    GstQuery * query);
+static gboolean gst_ass_render_query_src (GstPad * pad, GstObject * parent,
+    GstQuery * query);
 
 /* initialize the plugin's class */
 static void
@@ -152,6 +139,19 @@ gst_ass_render_class_init (GstAssRenderClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_ass_render_change_state);
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&video_sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&text_sink_factory));
+
+  gst_element_class_set_details_simple (gstelement_class, "ASS/SSA Render",
+      "Mixer/Video/Overlay/Subtitle",
+      "Renders ASS/SSA subtitles with libass",
+      "Benjamin Schmitz <vortex@wolpzone.de>, "
+      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 }
 
 #if defined(LIBASS_VERSION) && LIBASS_VERSION >= 0x00907000
@@ -177,7 +177,7 @@ _libass_message_cb (gint level, const gchar * fmt, va_list args,
 #endif
 
 static void
-gst_ass_render_init (GstAssRender * render, GstAssRenderClass * gclass)
+gst_ass_render_init (GstAssRender * render)
 {
   GST_DEBUG_OBJECT (render, "init");
 
@@ -187,16 +187,6 @@ gst_ass_render_init (GstAssRender * render, GstAssRenderClass * gclass)
   render->text_sinkpad =
       gst_pad_new_from_static_template (&text_sink_factory, "text_sink");
 
-  gst_pad_set_setcaps_function (render->video_sinkpad,
-      GST_DEBUG_FUNCPTR (gst_ass_render_setcaps_video));
-  gst_pad_set_setcaps_function (render->text_sinkpad,
-      GST_DEBUG_FUNCPTR (gst_ass_render_setcaps_text));
-
-  gst_pad_set_getcaps_function (render->srcpad,
-      GST_DEBUG_FUNCPTR (gst_ass_render_getcaps));
-  gst_pad_set_getcaps_function (render->video_sinkpad,
-      GST_DEBUG_FUNCPTR (gst_ass_render_getcaps));
-
   gst_pad_set_chain_function (render->video_sinkpad,
       GST_DEBUG_FUNCPTR (gst_ass_render_chain_video));
   gst_pad_set_chain_function (render->text_sinkpad,
@@ -209,21 +199,19 @@ gst_ass_render_init (GstAssRender * render, GstAssRenderClass * gclass)
   gst_pad_set_event_function (render->srcpad,
       GST_DEBUG_FUNCPTR (gst_ass_render_event_src));
 
-  gst_pad_set_bufferalloc_function (render->video_sinkpad,
-      GST_DEBUG_FUNCPTR (gst_ass_render_bufferalloc_video));
-
   gst_pad_set_query_function (render->srcpad,
       GST_DEBUG_FUNCPTR (gst_ass_render_query_src));
+  gst_pad_set_query_function (render->video_sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ass_render_query_video));
 
   gst_element_add_pad (GST_ELEMENT (render), render->srcpad);
   gst_element_add_pad (GST_ELEMENT (render), render->video_sinkpad);
   gst_element_add_pad (GST_ELEMENT (render), render->text_sinkpad);
 
-  render->width = 0;
-  render->height = 0;
+  gst_video_info_init (&render->info);
 
-  render->subtitle_mutex = g_mutex_new ();
-  render->subtitle_cond = g_cond_new ();
+  g_mutex_init (&render->subtitle_mutex);
+  g_cond_init (&render->subtitle_cond);
 
   render->renderer_init_ok = FALSE;
   render->track_init_ok = FALSE;
@@ -233,7 +221,7 @@ gst_ass_render_init (GstAssRender * render, GstAssRenderClass * gclass)
   gst_segment_init (&render->video_segment, GST_FORMAT_TIME);
   gst_segment_init (&render->subtitle_segment, GST_FORMAT_TIME);
 
-  render->ass_mutex = g_mutex_new ();
+  g_mutex_init (&render->ass_mutex);
   render->ass_library = ass_library_init ();
 #if defined(LIBASS_VERSION) && LIBASS_VERSION >= 0x00907000
   ass_set_message_cb (render->ass_library, _libass_message_cb, render);
@@ -256,11 +244,8 @@ gst_ass_render_finalize (GObject * object)
 {
   GstAssRender *render = GST_ASS_RENDER (object);
 
-  if (render->subtitle_mutex)
-    g_mutex_free (render->subtitle_mutex);
-
-  if (render->subtitle_cond)
-    g_cond_free (render->subtitle_cond);
+  g_mutex_clear (&render->subtitle_mutex);
+  g_cond_clear (&render->subtitle_cond);
 
   if (render->ass_track) {
     ass_free_track (render->ass_track);
@@ -274,8 +259,7 @@ gst_ass_render_finalize (GObject * object)
     ass_library_done (render->ass_library);
   }
 
-  if (render->ass_mutex)
-    g_mutex_free (render->ass_mutex);
+  g_mutex_clear (&render->ass_mutex);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -292,9 +276,9 @@ gst_ass_render_set_property (GObject * object, guint prop_id,
       break;
     case PROP_EMBEDDEDFONTS:
       render->embeddedfonts = g_value_get_boolean (value);
-      g_mutex_lock (render->ass_mutex);
+      g_mutex_lock (&render->ass_mutex);
       ass_set_extract_fonts (render->ass_library, render->embeddedfonts);
-      g_mutex_unlock (render->ass_mutex);
+      g_mutex_unlock (&render->ass_mutex);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -339,13 +323,13 @@ gst_ass_render_change_state (GstElement * element, GstStateChange transition)
       break;
 
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      g_mutex_lock (render->subtitle_mutex);
+      g_mutex_lock (&render->subtitle_mutex);
       render->subtitle_flushing = TRUE;
       if (render->subtitle_pending)
         gst_buffer_unref (render->subtitle_pending);
       render->subtitle_pending = NULL;
-      g_cond_signal (render->subtitle_cond);
-      g_mutex_unlock (render->subtitle_mutex);
+      g_cond_signal (&render->subtitle_cond);
+      g_mutex_unlock (&render->subtitle_mutex);
       break;
   }
 
@@ -353,11 +337,11 @@ gst_ass_render_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      g_mutex_lock (render->ass_mutex);
+      g_mutex_lock (&render->ass_mutex);
       if (render->ass_track)
         ass_free_track (render->ass_track);
       render->ass_track = NULL;
-      g_mutex_unlock (render->ass_mutex);
+      g_mutex_unlock (&render->ass_mutex);
       render->track_init_ok = FALSE;
       render->renderer_init_ok = FALSE;
       break;
@@ -372,21 +356,34 @@ gst_ass_render_change_state (GstElement * element, GstStateChange transition)
 }
 
 static gboolean
-gst_ass_render_query_src (GstPad * pad, GstQuery * query)
+gst_ass_render_query_src (GstPad * pad, GstObject * parent, GstQuery * query)
 {
-  GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
-  gboolean ret;
+  gboolean res = FALSE;
 
-  ret = gst_pad_peer_query (render->video_sinkpad, query);
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CAPS:
+    {
+      GstCaps *filter, *caps;
 
-  gst_object_unref (render);
-  return ret;
+      gst_query_parse_caps (query, &filter);
+      caps = gst_ass_render_getcaps (pad, filter);
+      gst_query_set_caps_result (query, caps);
+      gst_caps_unref (caps);
+      res = TRUE;
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, parent, query);
+      break;
+  }
+
+  return res;
 }
 
 static gboolean
-gst_ass_render_event_src (GstPad * pad, GstEvent * event)
+gst_ass_render_event_src (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
+  GstAssRender *render = GST_ASS_RENDER (parent);
   gboolean ret = FALSE;
 
   switch (GST_EVENT_TYPE (event)) {
@@ -402,13 +399,13 @@ gst_ass_render_event_src (GstPad * pad, GstEvent * event)
         gst_pad_push_event (render->srcpad, gst_event_new_flush_start ());
 
       /* Mark subtitle as flushing, unblocks chains */
-      g_mutex_lock (render->subtitle_mutex);
+      g_mutex_lock (&render->subtitle_mutex);
       if (render->subtitle_pending)
         gst_buffer_unref (render->subtitle_pending);
       render->subtitle_pending = NULL;
       render->subtitle_flushing = TRUE;
-      g_cond_signal (render->subtitle_cond);
-      g_mutex_unlock (render->subtitle_mutex);
+      g_cond_signal (&render->subtitle_cond);
+      g_mutex_unlock (&render->subtitle_mutex);
 
       /* Seek on each sink pad */
       gst_event_ref (event);
@@ -427,38 +424,38 @@ gst_ass_render_event_src (GstPad * pad, GstEvent * event)
       break;
   }
 
-  gst_object_unref (render);
-
   return ret;
 }
 
 static GstCaps *
-gst_ass_render_getcaps (GstPad * pad)
+gst_ass_render_getcaps (GstPad * pad, GstCaps * filter)
 {
   GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
   GstPad *otherpad;
   GstCaps *caps;
+  GstCaps *templ;
 
   if (pad == render->srcpad)
     otherpad = render->video_sinkpad;
   else
     otherpad = render->srcpad;
 
+  templ = gst_pad_get_pad_template_caps (otherpad);
+
   /* we can do what the peer can */
-  caps = gst_pad_peer_get_caps (otherpad);
+  caps = gst_pad_peer_query_caps (otherpad, filter);
   if (caps) {
     GstCaps *temp;
-    const GstCaps *templ;
 
     /* filtered against our padtemplate */
-    templ = gst_pad_get_pad_template_caps (otherpad);
     temp = gst_caps_intersect (caps, templ);
     gst_caps_unref (caps);
+    gst_caps_unref (templ);
     /* this is what we can do */
     caps = temp;
   } else {
     /* no peer, our padtemplate is enough then */
-    caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+    caps = templ;
   }
 
   gst_object_unref (render);
@@ -468,19 +465,24 @@ gst_ass_render_getcaps (GstPad * pad)
 
 #define CREATE_RGB_BLIT_FUNCTION(name,bpp,R,G,B) \
 static void \
-blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
+blit_##name (GstAssRender * render, ASS_Image * ass_image, GstVideoFrame * frame) \
 { \
   guint counter = 0; \
   gint alpha, r, g, b, k; \
   const guint8 *src; \
-  guint8 *dst; \
+  guint8 *dst, *data; \
   gint x, y, w, h; \
-  gint width = render->width; \
-  gint height = render->height; \
-  gint dst_stride = GST_ROUND_UP_4 (width * bpp); \
+  gint width; \
+  gint height; \
+  gint dst_stride; \
   gint dst_skip; \
   gint src_skip; \
   \
+  width = GST_VIDEO_FRAME_WIDTH (frame); \
+  height = GST_VIDEO_FRAME_HEIGHT (frame); \
+  dst_stride = GST_VIDEO_FRAME_PLANE_STRIDE (frame, 0); \
+  data = GST_VIDEO_FRAME_PLANE_DATA (frame, 0); \
+  \
   while (ass_image) { \
     if (ass_image->dst_y > height || ass_image->dst_x > width) \
       goto next; \
@@ -491,7 +493,7 @@ blit_##name (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer) \
     g = ((ass_image->color) >> 16) & 0xff; \
     b = ((ass_image->color) >> 8) & 0xff; \
     src = ass_image->bitmap; \
-    dst = buffer->data + ass_image->dst_y * dst_stride + ass_image->dst_x * bpp; \
+    dst = data + ass_image->dst_y * dst_stride + ass_image->dst_x * bpp; \
     \
     w = MIN (ass_image->w, width - ass_image->dst_x); \
     h = MIN (ass_image->h, height - ass_image->dst_y); \
@@ -561,7 +563,7 @@ rgb_to_v (gint r, gint g, gint b)
 }
 
 static void
-blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
+blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstVideoFrame * frame)
 {
   guint counter = 0;
   gint alpha, r, g, b, k, k2;
@@ -574,25 +576,20 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
   gint w2;
   gint src_stride;
 #endif
-  gint width = render->width;
-  gint height = render->height;
-  gint y_offset, y_stride;
-  gint u_offset, u_stride;
-  gint v_offset, v_stride;
+  gint width, height;
+  guint8 *y_data, *u_data, *v_data;
+  gint y_stride, u_stride, v_stride;
 
-  y_offset =
-      gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 0, width,
-      height);
-  u_offset =
-      gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 1, width,
-      height);
-  v_offset =
-      gst_video_format_get_component_offset (GST_VIDEO_FORMAT_I420, 2, width,
-      height);
+  width = GST_VIDEO_FRAME_WIDTH (frame);
+  height = GST_VIDEO_FRAME_HEIGHT (frame);
 
-  y_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 0, width);
-  u_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 1, width);
-  v_stride = gst_video_format_get_row_stride (GST_VIDEO_FORMAT_I420, 2, width);
+  y_data = GST_VIDEO_FRAME_COMP_DATA (frame, 0);
+  u_data = GST_VIDEO_FRAME_COMP_DATA (frame, 1);
+  v_data = GST_VIDEO_FRAME_COMP_DATA (frame, 2);
+
+  y_stride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 0);
+  u_stride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 1);
+  v_stride = GST_VIDEO_FRAME_COMP_STRIDE (frame, 2);
 
   while (ass_image) {
     if (ass_image->dst_y > height || ass_image->dst_x > width)
@@ -618,19 +615,14 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
 #endif
 
     src = ass_image->bitmap;
-    dst_y =
-        buffer->data + y_offset + ass_image->dst_y * y_stride +
-        ass_image->dst_x;
-    dst_u =
-        buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
-        ass_image->dst_x / 2;
-    dst_v =
-        buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
-        ass_image->dst_x / 2;
+#if 0
+    dst_y = y_data + ass_image->dst_y * y_stride + ass_image->dst_x;
+    dst_u = u_data + (ass_image->dst_y / 2) * u_stride + ass_image->dst_x / 2;
+    dst_v = v_data + (ass_image->dst_y / 2) * v_stride + ass_image->dst_x / 2;
+#endif
 
     for (y = 0; y < h; y++) {
-      dst_y = buffer->data + y_offset + (ass_image->dst_y + y) * y_stride +
-          ass_image->dst_x;
+      dst_y = y_data + (ass_image->dst_y + y) * y_stride + ass_image->dst_x;
       for (x = 0; x < w; x++) {
         k = src[y * ass_image->w + x] * alpha / 255;
         dst_y[x] = (k * Y + (255 - k) * dst_y[x]) / 255;
@@ -639,12 +631,8 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
 
     y = 0;
     if (ass_image->dst_y & 1) {
-      dst_u =
-          buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
-          ass_image->dst_x / 2;
-      dst_v =
-          buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
-          ass_image->dst_x / 2;
+      dst_u = u_data + (ass_image->dst_y / 2) * u_stride + ass_image->dst_x / 2;
+      dst_v = v_data + (ass_image->dst_y / 2) * v_stride + ass_image->dst_x / 2;
       x = 0;
       if (ass_image->dst_x & 1) {
         k2 = src[y * ass_image->w + x] * alpha / 255;
@@ -673,11 +661,9 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
     }
 
     for (; y < h - 1; y += 2) {
-      dst_u =
-          buffer->data + u_offset + ((ass_image->dst_y + y) / 2) * u_stride +
+      dst_u = u_data + ((ass_image->dst_y + y) / 2) * u_stride +
           ass_image->dst_x / 2;
-      dst_v =
-          buffer->data + v_offset + ((ass_image->dst_y + y) / 2) * v_stride +
+      dst_v = v_data + ((ass_image->dst_y + y) / 2) * v_stride +
           ass_image->dst_x / 2;
       x = 0;
       if (ass_image->dst_x & 1) {
@@ -711,12 +697,8 @@ blit_i420 (GstAssRender * render, ASS_Image * ass_image, GstBuffer * buffer)
     }
 
     if (y < h) {
-      dst_u =
-          buffer->data + u_offset + (ass_image->dst_y / 2) * u_stride +
-          ass_image->dst_x / 2;
-      dst_v =
-          buffer->data + v_offset + (ass_image->dst_y / 2) * v_stride +
-          ass_image->dst_x / 2;
+      dst_u = u_data + (ass_image->dst_y / 2) * u_stride + ass_image->dst_x / 2;
+      dst_v = v_data + (ass_image->dst_y / 2) * v_stride + ass_image->dst_x / 2;
       x = 0;
       if (ass_image->dst_x & 1) {
         k2 = src[y * ass_image->w + x] * alpha / 255;
@@ -761,25 +743,18 @@ gst_ass_render_setcaps_video (GstPad * pad, GstCaps * caps)
   gboolean ret = FALSE;
   gint par_n = 1, par_d = 1;
   gdouble dar;
+  GstVideoInfo info;
 
-  render->width = 0;
-  render->height = 0;
+  if (!gst_video_info_from_caps (&info, caps))
+    goto invalid_caps;
 
-  if (!gst_video_format_parse_caps (caps, &render->format, &render->width,
-          &render->height) ||
-      !gst_video_parse_caps_framerate (caps, &render->fps_n, &render->fps_d)) {
-    GST_ERROR_OBJECT (render, "Can't parse caps: %" GST_PTR_FORMAT, caps);
-    ret = FALSE;
-    goto out;
-  }
-
-  gst_video_parse_caps_pixel_aspect_ratio (caps, &par_n, &par_d);
+  render->info = info;
 
   ret = gst_pad_set_caps (render->srcpad, caps);
   if (!ret)
     goto out;
 
-  switch (render->format) {
+  switch (GST_VIDEO_INFO_FORMAT (&info)) {
     case GST_VIDEO_FORMAT_RGB:
       render->blit = blit_rgb;
       break;
@@ -806,16 +781,16 @@ gst_ass_render_setcaps_video (GstPad * pad, GstCaps * caps)
       goto out;
   }
 
-  g_mutex_lock (render->ass_mutex);
-  ass_set_frame_size (render->ass_renderer, render->width, render->height);
+  g_mutex_lock (&render->ass_mutex);
+  ass_set_frame_size (render->ass_renderer, info.width, info.height);
 
-  dar = (((gdouble) par_n) * ((gdouble) render->width))
-      / (((gdouble) par_d) * ((gdouble) render->height));
+  dar = (((gdouble) par_n) * ((gdouble) info.width))
+      / (((gdouble) par_d) * ((gdouble) info.height));
 #if !defined(LIBASS_VERSION) || LIBASS_VERSION < 0x00907000
   ass_set_aspect_ratio (render->ass_renderer, dar);
 #else
   ass_set_aspect_ratio (render->ass_renderer,
-      dar, ((gdouble) render->width) / ((gdouble) render->height));
+      dar, ((gdouble) info.width) / ((gdouble) info.height));
 #endif
   ass_set_font_scale (render->ass_renderer, 1.0);
   ass_set_hinting (render->ass_renderer, ASS_HINTING_LIGHT);
@@ -829,7 +804,7 @@ gst_ass_render_setcaps_video (GstPad * pad, GstCaps * caps)
 #endif
   ass_set_margins (render->ass_renderer, 0, 0, 0, 0);
   ass_set_use_margins (render->ass_renderer, 0);
-  g_mutex_unlock (render->ass_mutex);
+  g_mutex_unlock (&render->ass_mutex);
 
   render->renderer_init_ok = TRUE;
 
@@ -839,6 +814,14 @@ out:
   gst_object_unref (render);
 
   return ret;
+
+  /* ERRORS */
+invalid_caps:
+  {
+    GST_ERROR_OBJECT (render, "Can't parse caps: %" GST_PTR_FORMAT, caps);
+    ret = FALSE;
+    goto out;
+  }
 }
 
 static gboolean
@@ -848,8 +831,7 @@ gst_ass_render_setcaps_text (GstPad * pad, GstCaps * caps)
   GstStructure *structure;
   const GValue *value;
   GstBuffer *priv;
-  gchar *codec_private;
-  guint codec_private_size;
+  GstMapInfo map;
   gboolean ret = FALSE;
 
   structure = gst_caps_get_structure (caps, 0);
@@ -859,19 +841,19 @@ gst_ass_render_setcaps_text (GstPad * pad, GstCaps * caps)
 
   value = gst_structure_get_value (structure, "codec_data");
 
-  g_mutex_lock (render->ass_mutex);
+  g_mutex_lock (&render->ass_mutex);
   if (value != NULL) {
     priv = gst_value_get_buffer (value);
     g_return_val_if_fail (priv != NULL, FALSE);
 
-    codec_private = (gchar *) GST_BUFFER_DATA (priv);
-    codec_private_size = GST_BUFFER_SIZE (priv);
+    gst_buffer_map (priv, &map, GST_MAP_READ);
 
     if (!render->ass_track)
       render->ass_track = ass_new_track (render->ass_library);
 
-    ass_process_codec_private (render->ass_track,
-        codec_private, codec_private_size);
+    ass_process_codec_private (render->ass_track, (char *) map.data, map.size);
+
+    gst_buffer_unmap (priv, &map);
 
     GST_DEBUG_OBJECT (render, "ass track created");
 
@@ -885,7 +867,7 @@ gst_ass_render_setcaps_text (GstPad * pad, GstCaps * caps)
 
     ret = TRUE;
   }
-  g_mutex_unlock (render->ass_mutex);
+  g_mutex_unlock (&render->ass_mutex);
 
   gst_object_unref (render);
 
@@ -897,8 +879,7 @@ static void
 gst_ass_render_process_text (GstAssRender * render, GstBuffer * buffer,
     GstClockTime running_time, GstClockTime duration)
 {
-  gchar *data = (gchar *) GST_BUFFER_DATA (buffer);
-  guint size = GST_BUFFER_SIZE (buffer);
+  GstMapInfo map;
   gdouble pts_start, pts_end;
 
   pts_start = running_time;
@@ -910,41 +891,26 @@ gst_ass_render_process_text (GstAssRender * render, GstBuffer * buffer,
       "Processing subtitles with running time %" GST_TIME_FORMAT
       " and duration %" GST_TIME_FORMAT, GST_TIME_ARGS (running_time),
       GST_TIME_ARGS (duration));
-  g_mutex_lock (render->ass_mutex);
-  ass_process_chunk (render->ass_track, data, size, pts_start, pts_end);
-  g_mutex_unlock (render->ass_mutex);
+
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
+
+  g_mutex_lock (&render->ass_mutex);
+  ass_process_chunk (render->ass_track, (gchar *) map.data, map.size,
+      pts_start, pts_end);
+  g_mutex_unlock (&render->ass_mutex);
+
+  gst_buffer_unmap (buffer, &map);
   gst_buffer_unref (buffer);
 }
 
 static GstFlowReturn
-gst_ass_render_bufferalloc_video (GstPad * pad, guint64 offset, guint size,
-    GstCaps * caps, GstBuffer ** buffer)
+gst_ass_render_chain_video (GstPad * pad, GstObject * parent,
+    GstBuffer * buffer)
 {
-  GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
-  GstFlowReturn ret = GST_FLOW_WRONG_STATE;
-  GstPad *allocpad;
-
-  GST_OBJECT_LOCK (render);
-  allocpad = render->srcpad ? gst_object_ref (render->srcpad) : NULL;
-  GST_OBJECT_UNLOCK (render);
-
-  if (allocpad) {
-    ret = gst_pad_alloc_buffer (allocpad, offset, size, caps, buffer);
-    gst_object_unref (allocpad);
-  }
-
-  gst_object_unref (render);
-
-  return ret;
-}
-
-static GstFlowReturn
-gst_ass_render_chain_video (GstPad * pad, GstBuffer * buffer)
-{
-  GstAssRender *render = GST_ASS_RENDER (GST_PAD_PARENT (pad));
+  GstAssRender *render = GST_ASS_RENDER (parent);
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean in_seg = FALSE;
-  gint64 start, stop, clip_start = 0, clip_stop = 0;
+  guint64 start, stop, clip_start = 0, clip_stop = 0;
   ASS_Image *ass_image;
 
   if (!GST_BUFFER_TIMESTAMP_IS_VALID (buffer)) {
@@ -977,16 +943,15 @@ gst_ass_render_chain_video (GstPad * pad, GstBuffer * buffer)
   /* if the buffer is only partially in the segment, fix up stamps */
   if (clip_start != start || (stop != -1 && clip_stop != stop)) {
     GST_DEBUG_OBJECT (render, "clipping buffer timestamp/duration to segment");
-    buffer = gst_buffer_make_metadata_writable (buffer);
+    buffer = gst_buffer_make_writable (buffer);
     GST_BUFFER_TIMESTAMP (buffer) = clip_start;
     if (stop != -1)
       GST_BUFFER_DURATION (buffer) = clip_stop - clip_start;
   }
 
-  gst_segment_set_last_stop (&render->video_segment, GST_FORMAT_TIME,
-      clip_start);
+  render->video_segment.position = clip_start;
 
-  g_mutex_lock (render->subtitle_mutex);
+  g_mutex_lock (&render->subtitle_mutex);
   if (render->subtitle_pending) {
     GstClockTime sub_running_time, vid_running_time;
     GstClockTime sub_running_time_end, vid_running_time_end;
@@ -1012,15 +977,15 @@ gst_ass_render_chain_video (GstPad * pad, GstBuffer * buffer)
           GST_TIME_FORMAT, GST_TIME_ARGS (sub_running_time_end),
           GST_TIME_ARGS (vid_running_time));
       render->subtitle_pending = NULL;
-      g_cond_signal (render->subtitle_cond);
+      g_cond_signal (&render->subtitle_cond);
     } else if (sub_running_time <= vid_running_time_end + GST_SECOND / 2) {
       gst_ass_render_process_text (render, render->subtitle_pending,
           sub_running_time, sub_running_time_end - sub_running_time);
       render->subtitle_pending = NULL;
-      g_cond_signal (render->subtitle_cond);
+      g_cond_signal (&render->subtitle_cond);
     }
   }
-  g_mutex_unlock (render->subtitle_mutex);
+  g_mutex_unlock (&render->subtitle_mutex);
 
   /* now start rendering subtitles, if all conditions are met */
   if (render->renderer_init_ok && render->track_init_ok && render->enable) {
@@ -1039,7 +1004,7 @@ gst_ass_render_chain_video (GstPad * pad, GstBuffer * buffer)
     /* libass needs timestamps in ms */
     timestamp = running_time / GST_MSECOND;
 
-    g_mutex_lock (render->ass_mutex);
+    g_mutex_lock (&render->ass_mutex);
 #ifndef GST_DISABLE_GST_DEBUG
     /* only for testing right now. could possibly be used for optimizations? */
     step = ass_step_sub (render->ass_track, timestamp, 1);
@@ -1051,11 +1016,16 @@ gst_ass_render_chain_video (GstPad * pad, GstBuffer * buffer)
     /* not sure what the last parameter to this call is for (detect_change) */
     ass_image = ass_render_frame (render->ass_renderer, render->ass_track,
         timestamp, NULL);
-    g_mutex_unlock (render->ass_mutex);
+    g_mutex_unlock (&render->ass_mutex);
 
     if (ass_image != NULL) {
+      GstVideoFrame frame;
+
       buffer = gst_buffer_make_writable (buffer);
-      render->blit (render, ass_image, buffer);
+
+      gst_video_frame_map (&frame, &render->info, buffer, GST_MAP_WRITE);
+      render->blit (render, ass_image, &frame);
+      gst_video_frame_unmap (&frame);
     } else {
       GST_LOG_OBJECT (render, "nothing to render right now");
     }
@@ -1076,19 +1046,19 @@ out_of_segment:
 }
 
 static GstFlowReturn
-gst_ass_render_chain_text (GstPad * pad, GstBuffer * buffer)
+gst_ass_render_chain_text (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
   GstFlowReturn ret = GST_FLOW_OK;
-  GstAssRender *render = GST_ASS_RENDER (GST_PAD_PARENT (pad));
+  GstAssRender *render = GST_ASS_RENDER (parent);
   GstClockTime timestamp, duration;
   GstClockTime sub_running_time, vid_running_time;
   GstClockTime sub_running_time_end;
-  gint64 cstart, cstop;
+  guint64 cstart, cstop;
   gboolean in_seg;
 
   if (render->subtitle_flushing) {
     gst_buffer_unref (buffer);
-    return GST_FLOW_WRONG_STATE;
+    return GST_FLOW_FLUSHING;
   }
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
@@ -1117,8 +1087,7 @@ gst_ass_render_chain_text (GstPad * pad, GstBuffer * buffer)
   GST_BUFFER_TIMESTAMP (buffer) = timestamp = cstart;
   GST_BUFFER_DURATION (buffer) = duration = cstop - cstart;
 
-  gst_segment_set_last_stop (&render->subtitle_segment, GST_FORMAT_TIME,
-      GST_BUFFER_TIMESTAMP (buffer));
+  render->subtitle_segment.position = GST_BUFFER_TIMESTAMP (buffer);
 
   sub_running_time =
       gst_segment_to_running_time (&render->subtitle_segment, GST_FORMAT_TIME,
@@ -1128,28 +1097,29 @@ gst_ass_render_chain_text (GstPad * pad, GstBuffer * buffer)
       timestamp + duration);
   vid_running_time =
       gst_segment_to_running_time (&render->video_segment, GST_FORMAT_TIME,
-      render->video_segment.last_stop);
+      render->video_segment.position);
 
-  if (render->fps_n && render->fps_d)
+  if (render->info.fps_n && render->info.fps_d)
     vid_running_time +=
-        gst_util_uint64_scale (GST_SECOND, render->fps_d, render->fps_n);
+        gst_util_uint64_scale (GST_SECOND, render->info.fps_d,
+        render->info.fps_n);
 
   if (sub_running_time > vid_running_time + GST_SECOND / 2) {
     g_assert (render->subtitle_pending == NULL);
-    g_mutex_lock (render->subtitle_mutex);
+    g_mutex_lock (&render->subtitle_mutex);
     if (G_UNLIKELY (render->subtitle_flushing)) {
       GST_DEBUG_OBJECT (render, "Text pad flushing");
       gst_buffer_unref (buffer);
-      g_mutex_unlock (render->subtitle_mutex);
-      return GST_FLOW_WRONG_STATE;
+      g_mutex_unlock (&render->subtitle_mutex);
+      return GST_FLOW_FLUSHING;
     }
     GST_DEBUG_OBJECT (render,
         "Too early text buffer, waiting (%" GST_TIME_FORMAT " > %"
         GST_TIME_FORMAT, GST_TIME_ARGS (sub_running_time),
         GST_TIME_ARGS (vid_running_time));
     render->subtitle_pending = buffer;
-    g_cond_wait (render->subtitle_cond, render->subtitle_mutex);
-    g_mutex_unlock (render->subtitle_mutex);
+    g_cond_wait (&render->subtitle_cond, &render->subtitle_mutex);
+    g_mutex_unlock (&render->subtitle_mutex);
   } else if (sub_running_time_end < vid_running_time) {
     GST_DEBUG_OBJECT (render,
         "Too late text buffer, dropping (%" GST_TIME_FORMAT " < %"
@@ -1174,6 +1144,7 @@ gst_ass_render_chain_text (GstPad * pad, GstBuffer * buffer)
 static void
 gst_ass_render_handle_tags (GstAssRender * render, GstTagList * taglist)
 {
+#if 0
   static const gchar *mimetypes[] = {
     "application/x-font-ttf",
     "application/x-font-otf",
@@ -1183,14 +1154,15 @@ gst_ass_render_handle_tags (GstAssRender * render, GstTagList * taglist)
     ".otf",
     ".ttf"
   };
+#endif
   guint tag_size;
-  guint index;
 
   if (!taglist)
     return;
 
   tag_size = gst_tag_list_get_tag_size (taglist, GST_TAG_ATTACHMENT);
   if (tag_size > 0 && render->embeddedfonts) {
+#if 0
     const GValue *value;
     GstBuffer *buf;
     GstCaps *caps;
@@ -1198,10 +1170,13 @@ gst_ass_render_handle_tags (GstAssRender * render, GstTagList * taglist)
     gboolean valid_mimetype, valid_extension;
     guint j;
     const gchar *filename;
+#endif
+    guint index;
 
     GST_DEBUG_OBJECT (render, "TAG event has attachments");
 
     for (index = 0; index < tag_size; index++) {
+#if 0
       value = gst_tag_list_get_value_index (taglist, GST_TAG_ATTACHMENT, index);
       buf = gst_value_get_buffer (value);
       if (!buf || !GST_BUFFER_CAPS (buf))
@@ -1235,44 +1210,49 @@ gst_ass_render_handle_tags (GstAssRender * render, GstTagList * taglist)
       }
 
       if (valid_mimetype || valid_extension) {
-        g_mutex_lock (render->ass_mutex);
+        g_mutex_lock (&render->ass_mutex);
         ass_add_font (render->ass_library, (gchar *) filename,
             (gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
         GST_DEBUG_OBJECT (render, "registered new font %s", filename);
-        g_mutex_unlock (render->ass_mutex);
+        g_mutex_unlock (&render->ass_mutex);
       }
+#endif
     }
   }
 }
 
 static gboolean
-gst_ass_render_event_video (GstPad * pad, GstEvent * event)
+gst_ass_render_event_video (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   gboolean ret = FALSE;
-  GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
+  GstAssRender *render = GST_ASS_RENDER (parent);
 
   GST_DEBUG_OBJECT (pad, "received video event %s",
       GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_CAPS:
     {
-      GstFormat format;
-      gdouble rate;
-      gint64 start, stop, time;
-      gboolean update;
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      ret = gst_ass_render_setcaps_video (pad, caps);
+      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_SEGMENT:
+    {
+      GstSegment segment;
 
       GST_DEBUG_OBJECT (render, "received new segment");
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+      gst_event_copy_segment (event, &segment);
 
-      if (format == GST_FORMAT_TIME) {
+      if (segment.format == GST_FORMAT_TIME) {
         GST_DEBUG_OBJECT (render, "VIDEO SEGMENT now: %" GST_SEGMENT_FORMAT,
             &render->video_segment);
 
-        gst_segment_set_newsegment (&render->video_segment, update, rate,
-            format, start, stop, time);
+        render->video_segment = segment;
 
         GST_DEBUG_OBJECT (render, "VIDEO SEGMENT after: %" GST_SEGMENT_FORMAT,
             &render->video_segment);
@@ -1304,39 +1284,66 @@ gst_ass_render_event_video (GstPad * pad, GstEvent * event)
       break;
   }
 
-  gst_object_unref (render);
-
   return ret;
 }
 
 static gboolean
-gst_ass_render_event_text (GstPad * pad, GstEvent * event)
+gst_ass_render_query_video (GstPad * pad, GstObject * parent, GstQuery * query)
+{
+  gboolean res = FALSE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CAPS:
+    {
+      GstCaps *filter, *caps;
+
+      gst_query_parse_caps (query, &filter);
+      caps = gst_ass_render_getcaps (pad, filter);
+      gst_query_set_caps_result (query, caps);
+      gst_caps_unref (caps);
+      res = TRUE;
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, parent, query);
+      break;
+  }
+
+  return res;
+}
+
+static gboolean
+gst_ass_render_event_text (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   gint i;
   gboolean ret = FALSE;
-  GstAssRender *render = GST_ASS_RENDER (gst_pad_get_parent (pad));
+  GstAssRender *render = GST_ASS_RENDER (parent);
 
   GST_DEBUG_OBJECT (pad, "received text event %s", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_CAPS:
     {
-      GstFormat format;
-      gdouble rate;
-      gint64 start, stop, time;
-      gboolean update;
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      ret = gst_ass_render_setcaps_text (pad, caps);
+      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_SEGMENT:
+    {
+      GstSegment segment;
 
       GST_DEBUG_OBJECT (render, "received new segment");
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+      gst_event_copy_segment (event, &segment);
 
-      if (format == GST_FORMAT_TIME) {
+      if (segment.format == GST_FORMAT_TIME) {
         GST_DEBUG_OBJECT (render, "SUBTITLE SEGMENT now: %" GST_SEGMENT_FORMAT,
             &render->subtitle_segment);
 
-        gst_segment_set_newsegment (&render->subtitle_segment, update, rate,
-            format, start, stop, time);
+        render->subtitle_segment = segment;
 
         GST_DEBUG_OBJECT (render,
             "SUBTITLE SEGMENT after: %" GST_SEGMENT_FORMAT,
@@ -1359,7 +1366,7 @@ gst_ass_render_event_text (GstPad * pad, GstEvent * event)
       break;
     case GST_EVENT_FLUSH_START:
       GST_DEBUG_OBJECT (render, "begin flushing");
-      g_mutex_lock (render->ass_mutex);
+      g_mutex_lock (&render->ass_mutex);
       if (render->ass_track) {
         /* delete any events on the ass_track */
         for (i = 0; i < render->ass_track->n_events; i++) {
@@ -1369,14 +1376,14 @@ gst_ass_render_event_text (GstPad * pad, GstEvent * event)
         render->ass_track->n_events = 0;
         GST_DEBUG_OBJECT (render, "done flushing");
       }
-      g_mutex_unlock (render->ass_mutex);
-      g_mutex_lock (render->subtitle_mutex);
+      g_mutex_unlock (&render->ass_mutex);
+      g_mutex_lock (&render->subtitle_mutex);
       if (render->subtitle_pending)
         gst_buffer_unref (render->subtitle_pending);
       render->subtitle_pending = NULL;
       render->subtitle_flushing = TRUE;
-      g_cond_signal (render->subtitle_cond);
-      g_mutex_unlock (render->subtitle_mutex);
+      g_cond_signal (&render->subtitle_cond);
+      g_mutex_unlock (&render->subtitle_mutex);
       gst_event_unref (event);
       ret = TRUE;
       break;
@@ -1404,8 +1411,6 @@ gst_ass_render_event_text (GstPad * pad, GstEvent * event)
       break;
   }
 
-  gst_object_unref (render);
-
   return ret;
 }
 
@@ -1423,6 +1428,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "assrender",
+    assrender,
     "ASS/SSA subtitle renderer",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
