@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2008 Benjamin Schmitz <vortex@wolpzone.de>
+ * Copyright (c) 2009 Sebastian Dröge <sebastian.droege@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License
@@ -16,10 +17,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#ifndef __GST_ASSRENDER_H__
-#define __GST_ASSRENDER_H__
+#ifndef __GST_ASS_RENDER_H__
+#define __GST_ASS_RENDER_H__
 
 #include <gst/gst.h>
+#include <gst/video/video.h>
 
 #include <ass/ass.h>
 #include <ass/ass_types.h>
@@ -33,37 +35,51 @@ G_BEGIN_DECLS
 #define ASS_Image ass_image_t
 #endif
 
-#define GST_TYPE_ASSRENDER (gst_assrender_get_type())
-#define GST_ASSRENDER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ASSRENDER,Gstassrender))
-#define GST_ASSRENDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ASSRENDER,GstassrenderClass))
-#define GST_IS_ASSRENDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ASSRENDER))
-#define GST_IS_ASSRENDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ASSRENDER))
-typedef struct _Gstassrender Gstassrender;
-typedef struct _GstassrenderClass GstassrenderClass;
+#define GST_TYPE_ASS_RENDER (gst_ass_render_get_type())
+#define GST_ASS_RENDER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ASS_RENDER,GstAssRender))
+#define GST_ASS_RENDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ASS_RENDER,GstAssRenderClass))
+#define GST_IS_ASS_RENDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ASS_RENDER))
+#define GST_IS_ASS_RENDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ASS_RENDER))
 
-struct _Gstassrender
+typedef struct _GstAssRender GstAssRender;
+typedef struct _GstAssRenderClass GstAssRenderClass;
+typedef void (*GstAssRenderBlitFunction) (GstAssRender *render, ASS_Image *ass_image, GstVideoFrame *frame);
+
+struct _GstAssRender
 {
   GstElement element;
 
   GstPad *video_sinkpad, *text_sinkpad, *srcpad;
 
+  /* properties */
+  gboolean enable, embeddedfonts;
+
+  /* <private> */
   GstSegment video_segment;
 
-  gint width, height;
+  GstVideoInfo info;
+  GstAssRenderBlitFunction blit;
 
+  GMutex subtitle_mutex;
+  GCond subtitle_cond;
+  GstBuffer *subtitle_pending;
+  gboolean subtitle_flushing;
+  GstSegment subtitle_segment;
+
+  GMutex ass_mutex;
   ASS_Library *ass_library;
   ASS_Renderer *ass_renderer;
   ASS_Track *ass_track;
 
-  gboolean renderer_init_ok, track_init_ok, enable, embeddedfonts;
+  gboolean renderer_init_ok, track_init_ok;
 };
 
-struct _GstassrenderClass
+struct _GstAssRenderClass
 {
   GstElementClass parent_class;
 };
 
-GType gst_assrender_get_type (void);
+GType gst_ass_render_get_type (void);
 
 G_END_DECLS
 
