@@ -57,6 +57,8 @@
 
 #include "gstcurlsink.h"
 
+#include "gst/glib-compat-private.h"
+
 /* Default values */
 #define GST_CAT_DEFAULT                gst_curl_sink_debug
 #define DEFAULT_URL                    "localhost:5555"
@@ -155,7 +157,8 @@ gst_curl_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sinktemplate));
   gst_element_class_set_details_simple (element_class,
       "Curl sink",
       "Sink/Network",
@@ -847,7 +850,7 @@ gst_curl_sink_handle_transfer (GstCurlSink * sink)
     if (!proxy_conn_established && (resp_proxy != RESPONSE_CONNECT_PROXY)
         && proxy_auth) {
       curl_easy_getinfo (sink->curl, CURLINFO_HTTP_CONNECTCODE, &resp_proxy);
-      if ((resp_proxy == RESPONSE_CONNECT_PROXY)) {
+      if (resp_proxy == RESPONSE_CONNECT_PROXY) {
         GST_LOG ("received HTTP/1.0 200 Connection Established");
         /* Workaround: redefine HTTP headers before connecting to HTTP server.
          * When talking to proxy, the Content-Length: 0 is send with the request.
