@@ -132,10 +132,9 @@ static DFBSurfacePixelFormat gst_dfbvideosink_get_format_from_caps (GstCaps *
 static void gst_dfbvideosink_update_colorbalance (GstDfbVideoSink *
     dfbvideosink);
 static void gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
-    GstDfbSurface * surface);
+    GstBuffer * surface);
 
 static GstVideoSinkClass *parent_class = NULL;
-static GstBufferClass *surface_parent_class = NULL;
 
 #ifndef GST_DISABLE_GST_DEBUG
 static const char *
@@ -188,12 +187,52 @@ gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
 }
 #endif /* GST_DISABLE_GST_DEBUG */
 
+static void
+gst_dfbsurface_dispose (GstBuffer * surface)
+{
+  GstDfbVideoSink *dfbvideosink = NULL;
+  GstMetaDfbSurface *meta;
+
+  g_return_if_fail (surface != NULL);
+
+  meta = GST_META_DFBSURFACE_GET (surface);
+
+  dfbvideosink = meta->dfbvideosink;
+  if (!dfbvideosink) {
+    GST_WARNING_OBJECT (surface, "no sink found");
+    goto beach;
+  }
+
+  /* If our geometry changed we can't reuse that image. */
+  if ((meta->width != dfbvideosink->video_width) ||
+      (meta->height != dfbvideosink->video_height) ||
+      (meta->pixel_format != dfbvideosink->pixel_format)) {
+    GST_DEBUG_OBJECT (dfbvideosink, "destroy surface %p as its size changed "
+        "%dx%d vs current %dx%d", surface, meta->width, meta->height,
+        dfbvideosink->video_width, dfbvideosink->video_height);
+    gst_dfbvideosink_surface_destroy (dfbvideosink, surface);
+  } else {
+    /* In that case we can reuse the image and add it to our image pool. */
+    GST_DEBUG_OBJECT (dfbvideosink, "recycling surface %p in pool", surface);
+    /* need to increment the refcount again to recycle */
+    gst_buffer_ref (surface);
+    g_mutex_lock (dfbvideosink->pool_lock);
+    dfbvideosink->buffer_pool = g_slist_prepend (dfbvideosink->buffer_pool,
+        surface);
+    g_mutex_unlock (dfbvideosink->pool_lock);
+  }
+
+beach:
+  return;
+}
+
 /* Creates miniobject and our internal surface */
-static GstDfbSurface *
+static GstBuffer *
 gst_dfbvideosink_surface_create (GstDfbVideoSink * dfbvideosink, GstCaps * caps,
     size_t size)
 {
-  GstDfbSurface *surface = NULL;
+  GstBuffer *surface = NULL;
+  GstMetaDfbSurface *meta = NULL;
   GstStructure *structure = NULL;
   DFBResult ret;
   DFBSurfaceDescription s_dsc;
@@ -203,25 +242,28 @@ gst_dfbvideosink_surface_create (GstDfbVideoSink * dfbvideosink, GstCaps * caps,
 
   g_return_val_if_fail (GST_IS_DFBVIDEOSINK (dfbvideosink), NULL);
 
-  surface = (GstDfbSurface *) gst_mini_object_new (GST_TYPE_DFBSURFACE);
+  surface = gst_buffer_new ();
+  GST_MINI_OBJECT_CAST (surface)->dispose =
+      (GstMiniObjectDisposeFunction) gst_dfbsurface_dispose;
+  meta = GST_META_DFBSURFACE_ADD (surface);
 
   /* Keep a ref to our sink */
-  surface->dfbvideosink = gst_object_ref (dfbvideosink);
+  meta->dfbvideosink = gst_object_ref (dfbvideosink);
   /* Surface is not locked yet */
-  surface->locked = FALSE;
+  meta->locked = FALSE;
 
   structure = gst_caps_get_structure (caps, 0);
 
-  if (!gst_structure_get_int (structure, "width", &surface->width) ||
-      !gst_structure_get_int (structure, "height", &surface->height)) {
+  if (!gst_structure_get_int (structure, "width", &meta->width) ||
+      !gst_structure_get_int (structure, "height", &meta->height)) {
     GST_WARNING_OBJECT (dfbvideosink, "failed getting geometry from caps %"
         GST_PTR_FORMAT, caps);
     goto fallback;
   }
 
   /* Pixel format from caps */
-  surface->pixel_format = gst_dfbvideosink_get_format_from_caps (caps);
-  if (surface->pixel_format == DSPF_UNKNOWN) {
+  meta->pixel_format = gst_dfbvideosink_get_format_from_caps (caps);
+  if (meta->pixel_format == DSPF_UNKNOWN) {
     goto fallback;
   }
 
@@ -236,41 +278,41 @@ gst_dfbvideosink_surface_create (GstDfbVideoSink * dfbvideosink, GstCaps * caps,
   s_dsc.flags =
       DSDESC_PIXELFORMAT | DSDESC_WIDTH | DSDESC_HEIGHT /*| DSDESC_CAPS */ ;
 
-  s_dsc.pixelformat = surface->pixel_format;
-  s_dsc.width = surface->width;
-  s_dsc.height = surface->height;
+  s_dsc.pixelformat = meta->pixel_format;
+  s_dsc.width = meta->width;
+  s_dsc.height = meta->height;
   /*s_dsc.caps = DSCAPS_VIDEOONLY; */
 
   ret = dfbvideosink->dfb->CreateSurface (dfbvideosink->dfb, &s_dsc,
-      &surface->surface);
+      &meta->surface);
   if (ret != DFB_OK) {
     GST_WARNING_OBJECT (dfbvideosink, "failed creating a DirectFB surface");
-    surface->surface = NULL;
+    meta->surface = NULL;
     goto fallback;
   }
 
   /* Clearing surface */
-  surface->surface->Clear (surface->surface, 0x00, 0x00, 0x00, 0xFF);
+  meta->surface->Clear (meta->surface, 0x00, 0x00, 0x00, 0xFF);
 
   /* Locking the surface to acquire the memory pointer */
-  surface->surface->Lock (surface->surface, DSLF_WRITE, &data, &pitch);
-  surface->locked = TRUE;
+  meta->surface->Lock (meta->surface, DSLF_WRITE, &data, &pitch);
+  meta->locked = TRUE;
   GST_BUFFER_DATA (surface) = data;
-  GST_BUFFER_SIZE (surface) = pitch * surface->height;
+  GST_BUFFER_SIZE (surface) = pitch * meta->height;
 
   /* Be carefull here. If size is different from the surface size
      (pitch * height), we can't use that surface through buffer alloc system
      or we are going to run into serious stride issues */
   if (GST_BUFFER_SIZE (surface) != size) {
     GST_WARNING_OBJECT (dfbvideosink, "DirectFB surface size (%dx%d=%d) "
-        "differs from GStreamer requested size %u", pitch, surface->height,
+        "differs from GStreamer requested size %u", pitch, meta->height,
         GST_BUFFER_SIZE (surface), (guint) size);
     goto fallback;
   }
 
   GST_DEBUG_OBJECT (dfbvideosink, "creating a %dx%d surface (%p) with %s "
-      "pixel format, line pitch %d", surface->width, surface->height, surface,
-      gst_dfbvideosink_get_format_name (surface->pixel_format), pitch);
+      "pixel format, line pitch %d", meta->width, meta->height, surface,
+      gst_dfbvideosink_get_format_name (meta->pixel_format), pitch);
 
   succeeded = TRUE;
 
@@ -280,16 +322,17 @@ fallback:
 
   /* We allocate a standard buffer ourselves to store it in our buffer pool,
      this is an optimisation for memory allocation */
-  GST_BUFFER (surface)->malloc_data = g_malloc (size);
-  GST_BUFFER_DATA (surface) = GST_BUFFER (surface)->malloc_data;
+  GST_BUFFER_MALLOCDATA (surface) = g_malloc (size);
+  GST_BUFFER_DATA (surface) = GST_BUFFER_MALLOCDATA (surface);
   GST_BUFFER_SIZE (surface) = size;
-  if (surface->surface) {
-    if (surface->locked) {
-      surface->surface->Unlock (surface->surface);
-      surface->locked = FALSE;
+
+  if (meta->surface) {
+    if (meta->locked) {
+      meta->surface->Unlock (meta->surface);
+      meta->locked = FALSE;
     }
-    surface->surface->Release (surface->surface);
-    surface->surface = NULL;
+    meta->surface->Release (meta->surface);
+    meta->surface = NULL;
   }
   GST_DEBUG_OBJECT (dfbvideosink, "allocating a buffer (%p) of %u bytes",
       surface, (guint) size);
@@ -308,28 +351,29 @@ beach:
  * destroyed so we just have to clean our internal stuff */
 static void
 gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
-    GstDfbSurface * surface)
+    GstBuffer * surface)
 {
+  GstMetaDfbSurface *meta;
+
   g_return_if_fail (GST_IS_DFBVIDEOSINK (dfbvideosink));
 
+  meta = GST_META_DFBSURFACE_GET (surface);
+
   /* Release our internal surface */
-  if (surface->surface) {
-    if (surface->locked) {
-      surface->surface->Unlock (surface->surface);
-      surface->locked = FALSE;
+  if (meta->surface) {
+    if (meta->locked) {
+      meta->surface->Unlock (meta->surface);
+      meta->locked = FALSE;
     }
-    surface->surface->Release (surface->surface);
-    surface->surface = NULL;
+    meta->surface->Release (meta->surface);
+    meta->surface = NULL;
   }
 
-  if (surface->dfbvideosink) {
+  if (meta->dfbvideosink) {
     /* Release the ref to our sink */
-    surface->dfbvideosink = NULL;
+    meta->dfbvideosink = NULL;
     gst_object_unref (dfbvideosink);
   }
-
-  GST_MINI_OBJECT_CLASS (surface_parent_class)->finalize (GST_MINI_OBJECT
-      (surface));
 }
 
 static gpointer
@@ -339,8 +383,8 @@ gst_dfbvideosink_event_thread (GstDfbVideoSink * dfbvideosink)
 
   while (dfbvideosink->running) {
     /* Wait for an event with a 50 ms timeout */
-    dfbvideosink->event_buffer->
-        WaitForEventWithTimeout (dfbvideosink->event_buffer, 0, 50);
+    dfbvideosink->event_buffer->WaitForEventWithTimeout (dfbvideosink->
+        event_buffer, 0, 50);
 
     /* Do we have an event ? */
     ret = dfbvideosink->event_buffer->HasEvent (dfbvideosink->event_buffer);
@@ -1515,6 +1559,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
   GstVideoRectangle dst, src, result;
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean mem_cpy = TRUE;
+  GstMetaDfbSurface *meta;
 
   dfbvideosink = GST_DFBVIDEOSINK (bsink);
 
@@ -1523,12 +1568,12 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     goto beach;
   }
 
-  /* Is that a buffer we allocated ourselves ? */
-  if (GST_IS_DFBSURFACE (buf)) {
-    GstDfbSurface *tmp_surface = GST_DFBSURFACE (buf);
+  meta = GST_META_DFBSURFACE_GET (buf);
 
+  /* Is that a buffer we allocated ourselves ? */
+  if (meta != NULL) {
     /* Does it have a surface ? */
-    if (tmp_surface->surface) {
+    if (meta->surface) {
       mem_cpy = FALSE;
       GST_DEBUG_OBJECT (dfbvideosink, "we have a buffer (%p) we allocated "
           "ourselves and it has a surface, no memcpy then", buf);
@@ -1613,7 +1658,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
     res = dest->Unlock (dest);
 
-    res = dest->Release (dest);
+    dest->Release (dest);
 
     if (dfbvideosink->backbuffer) {
       if (dfbvideosink->vsync) {
@@ -1624,8 +1669,6 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     }
   } else {
     /* Else we will [Stretch]Blit to our primary */
-    GstDfbSurface *surface = GST_DFBSURFACE (buf);
-
     GST_DEBUG_OBJECT (dfbvideosink, "blitting to a primary surface (vsync %d)",
         dfbvideosink->vsync);
 
@@ -1635,9 +1678,9 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     dfbvideosink->primary->GetSize (dfbvideosink->primary, &dst.w, &dst.h);
 
     /* Unlocking surface before blit */
-    if (surface->locked) {
-      surface->surface->Unlock (surface->surface);
-      surface->locked = FALSE;
+    if (meta->locked) {
+      meta->surface->Unlock (meta->surface);
+      meta->locked = FALSE;
     }
 
     gst_video_sink_center_rect (src, dst, &result, dfbvideosink->hw_scaling);
@@ -1649,14 +1692,14 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
     if (dfbvideosink->hw_scaling) {
       dfbvideosink->primary->StretchBlit (dfbvideosink->primary,
-          surface->surface, NULL, (DFBRectangle *) (void *) &result);
+          meta->surface, NULL, (DFBRectangle *) (void *) &result);
     } else {
       DFBRectangle clip;
 
       clip.x = clip.y = 0;
       clip.w = result.w;
       clip.h = result.h;
-      dfbvideosink->primary->Blit (dfbvideosink->primary, surface->surface,
+      dfbvideosink->primary->Blit (dfbvideosink->primary, meta->surface,
           &clip, result.x, result.y);
     }
 
@@ -1679,7 +1722,7 @@ gst_dfbvideosink_bufferpool_clear (GstDfbVideoSink * dfbvideosink)
 {
   g_mutex_lock (dfbvideosink->pool_lock);
   while (dfbvideosink->buffer_pool) {
-    GstDfbSurface *surface = dfbvideosink->buffer_pool->data;
+    GstBuffer *surface = dfbvideosink->buffer_pool->data;
 
     dfbvideosink->buffer_pool = g_slist_delete_link (dfbvideosink->buffer_pool,
         dfbvideosink->buffer_pool);
@@ -1696,7 +1739,7 @@ gst_dfbvideosink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
     GstCaps * caps, GstBuffer ** buf)
 {
   GstDfbVideoSink *dfbvideosink;
-  GstDfbSurface *surface = NULL;
+  GstBuffer *surface = NULL;
   GstFlowReturn ret = GST_FLOW_OK;
 
   gboolean rev_nego = FALSE;
@@ -1798,7 +1841,9 @@ alloc:
   /* Inspect our buffer pool */
   g_mutex_lock (dfbvideosink->pool_lock);
   while (dfbvideosink->buffer_pool) {
-    surface = (GstDfbSurface *) dfbvideosink->buffer_pool->data;
+    GstMetaDfbSurface *meta;
+
+    surface = (GstBuffer *) dfbvideosink->buffer_pool->data;
 
     if (surface) {
       /* Removing from the pool */
@@ -1806,10 +1851,12 @@ alloc:
           g_slist_delete_link (dfbvideosink->buffer_pool,
           dfbvideosink->buffer_pool);
 
+      meta = GST_META_DFBSURFACE_GET (surface);
+
       /* If the surface is invalid for our need, destroy */
-      if ((surface->width != width) ||
-          (surface->height != height) ||
-          (surface->pixel_format != dfbvideosink->pixel_format)) {
+      if ((meta->width != width) ||
+          (meta->height != height) ||
+          (meta->pixel_format != dfbvideosink->pixel_format)) {
         gst_dfbvideosink_surface_destroy (dfbvideosink, surface);
         surface = NULL;
       } else {
@@ -1832,100 +1879,34 @@ alloc:
   /* Now we should have a surface, set appropriate caps on it */
   if (surface) {
     if (rev_nego) {
-      gst_buffer_set_caps (GST_BUFFER (surface), desired_caps);
+      gst_buffer_set_caps (surface, desired_caps);
     } else {
-      gst_buffer_set_caps (GST_BUFFER (surface), caps);
+      gst_buffer_set_caps (surface, caps);
     }
   }
 
-  *buf = GST_BUFFER (surface);
+  *buf = surface;
 
   gst_caps_unref (desired_caps);
 
   return ret;
 }
 
-/* Our subclass of GstBuffer */
-
-static void
-gst_dfbsurface_finalize (GstDfbSurface * surface)
+/* our metadata */
+const GstMetaInfo *
+gst_meta_dfbsurface_get_info (void)
 {
-  GstDfbVideoSink *dfbvideosink = NULL;
+  static const GstMetaInfo *meta_info = NULL;
 
-  g_return_if_fail (surface != NULL);
-
-  dfbvideosink = surface->dfbvideosink;
-  if (!dfbvideosink) {
-    GST_WARNING_OBJECT (surface, "no sink found");
-    goto beach;
+  if (meta_info == NULL) {
+    meta_info = gst_meta_register ("GstMetaDfbSurface", "GstMetaDfbSurface",
+        sizeof (GstMetaDfbSurface),
+        (GstMetaInitFunction) NULL,
+        (GstMetaFreeFunction) NULL,
+        (GstMetaTransformFunction) NULL,
+        (GstMetaSerializeFunction) NULL, (GstMetaDeserializeFunction) NULL);
   }
-
-  /* If our geometry changed we can't reuse that image. */
-  if ((surface->width != dfbvideosink->video_width) ||
-      (surface->height != dfbvideosink->video_height) ||
-      (surface->pixel_format != dfbvideosink->pixel_format)) {
-    GST_DEBUG_OBJECT (dfbvideosink, "destroy surface %p as its size changed "
-        "%dx%d vs current %dx%d", surface, surface->width, surface->height,
-        dfbvideosink->video_width, dfbvideosink->video_height);
-    gst_dfbvideosink_surface_destroy (dfbvideosink, surface);
-  } else {
-    /* In that case we can reuse the image and add it to our image pool. */
-    GST_DEBUG_OBJECT (dfbvideosink, "recycling surface %p in pool", surface);
-    /* need to increment the refcount again to recycle */
-    gst_buffer_ref (GST_BUFFER (surface));
-    g_mutex_lock (dfbvideosink->pool_lock);
-    dfbvideosink->buffer_pool = g_slist_prepend (dfbvideosink->buffer_pool,
-        surface);
-    g_mutex_unlock (dfbvideosink->pool_lock);
-  }
-
-beach:
-  return;
-}
-
-static void
-gst_dfbsurface_init (GstDfbSurface * surface, gpointer g_class)
-{
-  surface->surface = NULL;
-  surface->width = 0;
-  surface->height = 0;
-  surface->pixel_format = DSPF_UNKNOWN;
-  surface->dfbvideosink = NULL;
-}
-
-static void
-gst_dfbsurface_class_init (gpointer g_class, gpointer class_data)
-{
-  GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
-
-  surface_parent_class = g_type_class_peek_parent (g_class);
-
-  mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
-      gst_dfbsurface_finalize;
-}
-
-GType
-gst_dfbsurface_get_type (void)
-{
-  static GType _gst_dfbsurface_type;
-
-  if (G_UNLIKELY (_gst_dfbsurface_type == 0)) {
-    static const GTypeInfo dfbsurface_info = {
-      sizeof (GstBufferClass),
-      NULL,
-      NULL,
-      gst_dfbsurface_class_init,
-      NULL,
-      NULL,
-      sizeof (GstDfbSurface),
-      0,
-      (GInstanceInitFunc) gst_dfbsurface_init,
-      NULL
-    };
-    _gst_dfbsurface_type = g_type_register_static (GST_TYPE_BUFFER,
-        "GstDfbSurface", &dfbsurface_info, 0);
-  }
-  return _gst_dfbsurface_type;
+  return meta_info;
 }
 
 /* Interfaces stuff */
@@ -2110,7 +2091,7 @@ gst_dfbvideosink_colorbalance_get_value (GstColorBalance * balance,
 }
 
 static void
-gst_dfbvideosink_colorbalance_init (GstColorBalanceClass * iface)
+gst_dfbvideosink_colorbalance_init (GstColorBalanceInterface * iface)
 {
   GST_COLOR_BALANCE_TYPE (iface) = GST_COLOR_BALANCE_HARDWARE;
   iface->list_channels = gst_dfbvideosink_colorbalance_list_channels;
@@ -2283,8 +2264,8 @@ gst_dfbvideosink_base_init (gpointer g_class)
       "Sink/Video",
       "A DirectFB based videosink", "Julien Moutte <julien@moutte.net>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_dfbvideosink_sink_template_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_dfbvideosink_sink_template_factory));
 }
 
 static void
@@ -2415,6 +2396,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "dfbvideosink",
+    dfbvideosink,
     "DirectFB video output plugin",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
