@@ -41,7 +41,6 @@
 
 #include <string.h>
 #include <gst/audio/audio.h>
-#include <gst/audio/multichannel.h>
 
 /* These are the correct types for these functions, as defined in the source,
  * with types changed to match glib types, since those are defined for us.
@@ -92,27 +91,17 @@ GST_DEBUG_CATEGORY_STATIC (faad_debug);
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/mpeg, " "mpegversion = (int) { 2, 4 }")
+    GST_STATIC_CAPS ("audio/mpeg, " "mpegversion = (int) 2; "
+        "audio/mpeg, mpegversion = (int) 4, stream-format = (string) { raw, adts }")
     );
 
-#define STATIC_INT_CAPS(bpp) \
-  "audio/x-raw-int, " \
-    "endianness = (int) BYTE_ORDER, " \
-    "signed = (bool) TRUE, " \
-    "width = (int) " G_STRINGIFY (bpp) ", " \
-    "depth = (int) " G_STRINGIFY (bpp) ", " \
+#define STATIC_RAW_CAPS(format) \
+  "audio/x-raw, " \
+    "format = (string) "GST_AUDIO_NE(format)", " \
+    "layout = (string) interleaved, " \
     "rate = (int) [ 8000, 96000 ], " \
     "channels = (int) [ 1, 8 ]"
 
-#if 0
-#define STATIC_FLOAT_CAPS(bpp) \
-  "audio/x-raw-float, " \
-    "endianness = (int) BYTE_ORDER, " \
-    "depth = (int) " G_STRINGIFY (bpp) ", " \
-    "rate = (int) [ 8000, 96000 ], " \
-    "channels = (int) [ 1, 8 ]"
-#endif
-
 /*
  * All except 16-bit integer are disabled until someone fixes FAAD.
  * FAAD allocates approximately 8*1024*2 bytes bytes, which is enough
@@ -122,16 +111,16 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
  */
 
 #define STATIC_CAPS \
-  STATIC_INT_CAPS (16)
+  STATIC_RAW_CAPS (S16)
 #if 0
 #define NOTUSED "; " \
-STATIC_INT_CAPS (24) \
+STATIC_RAW_CAPS (S24) \
     "; " \
-STATIC_INT_CAPS (32) \
+STATIC_RAW_CAPS (S32) \
     "; " \
-STATIC_FLOAT_CAPS (32) \
+STATIC_RAW_CAPS (F32) \
     "; " \
-STATIC_FLOAT_CAPS (64)
+STATIC_RAW_CAPS (F64)
 #endif
 
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
@@ -140,57 +129,28 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (STATIC_CAPS)
     );
 
-static void gst_faad_base_init (GstFaadClass * klass);
-static void gst_faad_class_init (GstFaadClass * klass);
-static void gst_faad_init (GstFaad * faad);
 static void gst_faad_reset (GstFaad * faad);
-static void gst_faad_finalize (GObject * object);
 
-static void clear_queued (GstFaad * faad);
+static gboolean gst_faad_start (GstAudioDecoder * dec);
+static gboolean gst_faad_stop (GstAudioDecoder * dec);
+static gboolean gst_faad_set_format (GstAudioDecoder * dec, GstCaps * caps);
+static gboolean gst_faad_parse (GstAudioDecoder * dec, GstAdapter * adapter,
+    gint * offset, gint * length);
+static GstFlowReturn gst_faad_handle_frame (GstAudioDecoder * dec,
+    GstBuffer * buffer);
+static void gst_faad_flush (GstAudioDecoder * dec, gboolean hard);
 
-static gboolean gst_faad_setcaps (GstPad * pad, GstCaps * caps);
-static gboolean gst_faad_src_event (GstPad * pad, GstEvent * event);
-static gboolean gst_faad_sink_event (GstPad * pad, GstEvent * event);
-static gboolean gst_faad_src_query (GstPad * pad, GstQuery * query);
-static GstFlowReturn gst_faad_chain (GstPad * pad, GstBuffer * buffer);
-static GstStateChangeReturn gst_faad_change_state (GstElement * element,
-    GstStateChange transition);
-static gboolean gst_faad_src_convert (GstFaad * faad, GstFormat src_format,
-    gint64 src_val, GstFormat dest_format, gint64 * dest_val);
 static gboolean gst_faad_open_decoder (GstFaad * faad);
 static void gst_faad_close_decoder (GstFaad * faad);
 
-static GstElementClass *parent_class;   /* NULL */
-
-GType
-gst_faad_get_type (void)
-{
-  static GType gst_faad_type = 0;
-
-  if (!gst_faad_type) {
-    static const GTypeInfo gst_faad_info = {
-      sizeof (GstFaadClass),
-      (GBaseInitFunc) gst_faad_base_init,
-      NULL,
-      (GClassInitFunc) gst_faad_class_init,
-      NULL,
-      NULL,
-      sizeof (GstFaad),
-      0,
-      (GInstanceInitFunc) gst_faad_init,
-    };
-
-    gst_faad_type = g_type_register_static (GST_TYPE_ELEMENT,
-        "GstFaad", &gst_faad_info, 0);
-  }
-
-  return gst_faad_type;
-}
+#define gst_faad_parent_class parent_class
+G_DEFINE_TYPE (GstFaad, gst_faad, GST_TYPE_AUDIO_DECODER);
 
 static void
-gst_faad_base_init (GstFaadClass * klass)
+gst_faad_class_init (GstFaadClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstAudioDecoderClass *base_class = GST_AUDIO_DECODER_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
@@ -202,53 +162,25 @@ gst_faad_base_init (GstFaadClass * klass)
       "Free MPEG-2/4 AAC decoder",
       "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
+  base_class->start = GST_DEBUG_FUNCPTR (gst_faad_start);
+  base_class->stop = GST_DEBUG_FUNCPTR (gst_faad_stop);
+  base_class->set_format = GST_DEBUG_FUNCPTR (gst_faad_set_format);
+  base_class->parse = GST_DEBUG_FUNCPTR (gst_faad_parse);
+  base_class->handle_frame = GST_DEBUG_FUNCPTR (gst_faad_handle_frame);
+  base_class->flush = GST_DEBUG_FUNCPTR (gst_faad_flush);
+
   GST_DEBUG_CATEGORY_INIT (faad_debug, "faad", 0, "AAC decoding");
 }
 
-static void
-gst_faad_class_init (GstFaadClass * klass)
-{
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
-  parent_class = g_type_class_peek_parent (klass);
-
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_faad_finalize);
-
-  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_faad_change_state);
-}
-
 static void
 gst_faad_init (GstFaad * faad)
 {
-  faad->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
-  gst_element_add_pad (GST_ELEMENT (faad), faad->sinkpad);
-  gst_pad_set_event_function (faad->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_faad_sink_event));
-  gst_pad_set_setcaps_function (faad->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_faad_setcaps));
-  gst_pad_set_chain_function (faad->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_faad_chain));
-
-  faad->srcpad = gst_pad_new_from_static_template (&src_template, "src");
-  gst_pad_use_fixed_caps (faad->srcpad);
-  gst_pad_set_query_function (faad->srcpad,
-      GST_DEBUG_FUNCPTR (gst_faad_src_query));
-  gst_pad_set_event_function (faad->srcpad,
-      GST_DEBUG_FUNCPTR (gst_faad_src_event));
-  gst_element_add_pad (GST_ELEMENT (faad), faad->srcpad);
-
-  faad->adapter = gst_adapter_new ();
-
   gst_faad_reset (faad);
 }
 
 static void
 gst_faad_reset_stream_state (GstFaad * faad)
 {
-  faad->sync_flush = 0;
-  gst_adapter_clear (faad->adapter);
-  clear_queued (faad);
   if (faad->handle)
     faacDecPostSeekReset (faad->handle, 0);
 }
@@ -256,44 +188,43 @@ gst_faad_reset_stream_state (GstFaad * faad)
 static void
 gst_faad_reset (GstFaad * faad)
 {
-  gst_segment_init (&faad->segment, GST_FORMAT_TIME);
   faad->samplerate = -1;
   faad->channels = -1;
   faad->init = FALSE;
   faad->packetised = FALSE;
   g_free (faad->channel_positions);
   faad->channel_positions = NULL;
-  faad->next_ts = 0;
-  faad->prev_ts = GST_CLOCK_TIME_NONE;
-  faad->bytes_in = 0;
-  faad->sum_dur_out = 0;
-  faad->error_count = 0;
+  faad->last_header = 0;
 
   gst_faad_reset_stream_state (faad);
 }
 
-
-static void
-gst_faad_finalize (GObject * object)
+static gboolean
+gst_faad_start (GstAudioDecoder * dec)
 {
-  GstFaad *faad = GST_FAAD (object);
+  GstFaad *faad = GST_FAAD (dec);
 
-  g_object_unref (faad->adapter);
+  GST_DEBUG_OBJECT (dec, "start");
+  gst_faad_reset (faad);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  /* call upon legacy upstream byte support (e.g. seeking) */
+  gst_audio_decoder_set_estimate_rate (dec, TRUE);
+  /* never mind a few errors */
+  gst_audio_decoder_set_max_errors (dec, 10);
+
+  return TRUE;
 }
 
-static void
-gst_faad_send_tags (GstFaad * faad)
+static gboolean
+gst_faad_stop (GstAudioDecoder * dec)
 {
-  GstTagList *tags;
+  GstFaad *faad = GST_FAAD (dec);
 
-  tags = gst_tag_list_new ();
+  GST_DEBUG_OBJECT (dec, "stop");
+  gst_faad_reset (faad);
+  gst_faad_close_decoder (faad);
 
-  gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE,
-      GST_TAG_AUDIO_CODEC, "MPEG-4 AAC audio", NULL);
-
-  gst_element_found_tags (GST_ELEMENT (faad), tags);
+  return TRUE;
 }
 
 static gint
@@ -326,12 +257,18 @@ aac_rate_idx (gint rate)
 }
 
 static gboolean
-gst_faad_setcaps (GstPad * pad, GstCaps * caps)
+gst_faad_set_format (GstAudioDecoder * dec, GstCaps * caps)
 {
-  GstFaad *faad = GST_FAAD (gst_pad_get_parent (pad));
+  GstFaad *faad = GST_FAAD (dec);
   GstStructure *str = gst_caps_get_structure (caps, 0);
   GstBuffer *buf;
   const GValue *value;
+  GstMapInfo map;
+  guint8 *cdata;
+  gsize csize;
+
+  /* clean up current decoder, rather than trying to reconfigure */
+  gst_faad_close_decoder (faad);
 
   /* Assume raw stream */
   faad->packetised = FALSE;
@@ -343,17 +280,16 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
     guint32 samplerate;
 #endif
     guint8 channels;
-    guint8 *cdata;
-    guint csize;
 
     /* We have codec data, means packetised stream */
     faad->packetised = TRUE;
-    buf = gst_value_get_buffer (value);
 
+    buf = gst_value_get_buffer (value);
     g_return_val_if_fail (buf != NULL, FALSE);
 
-    cdata = GST_BUFFER_DATA (buf);
-    csize = GST_BUFFER_SIZE (buf);
+    gst_buffer_map (buf, &map, GST_MAP_READ);
+    cdata = map.data;
+    csize = map.size;
 
     if (csize < 2)
       goto wrong_length;
@@ -364,6 +300,8 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
         (((cdata[0] & 0x07) << 1) | ((cdata[1] & 0x80) >> 7)),
         ((cdata[1] & 0x78) >> 3));
 
+    if (!gst_faad_open_decoder (faad))
+      goto open_failed;
     /* someone forgot that char can be unsigned when writing the API */
     if ((gint8) faacDecInit2 (faad->handle, cdata, csize, &samplerate,
             &channels) < 0)
@@ -385,12 +323,11 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
     faad->channels = 0;
 
     faad->init = TRUE;
-    gst_faad_send_tags (faad);
-
-    gst_adapter_clear (faad->adapter);
+    gst_buffer_unmap (buf, &map);
   } else if ((value = gst_structure_get_value (str, "framed")) &&
       g_value_get_boolean (value) == TRUE) {
     faad->packetised = TRUE;
+    faad->init = FALSE;
     GST_DEBUG_OBJECT (faad, "we have packetized audio");
   } else {
     faad->init = FALSE;
@@ -417,7 +354,6 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
     }
   }
 
-  gst_object_unref (faad);
   return TRUE;
 
   /* ERRORS */
@@ -425,37 +361,45 @@ wrong_length:
   {
     GST_DEBUG_OBJECT (faad, "codec_data less than 2 bytes long");
     gst_object_unref (faad);
+    gst_buffer_unmap (buf, &map);
+    return FALSE;
+  }
+open_failed:
+  {
+    GST_DEBUG_OBJECT (faad, "failed to create decoder");
+    gst_object_unref (faad);
+    gst_buffer_unmap (buf, &map);
     return FALSE;
   }
 init_failed:
   {
     GST_DEBUG_OBJECT (faad, "faacDecInit2() failed");
     gst_object_unref (faad);
+    gst_buffer_unmap (buf, &map);
     return FALSE;
   }
 }
 
-static GstAudioChannelPosition *
-gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
-    gboolean * channel_map_failed)
+static gboolean
+gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos,
+    GstAudioChannelPosition * pos, guint num)
 {
-  GstAudioChannelPosition *pos;
   guint n;
   gboolean unknown_channel = FALSE;
 
-  *channel_map_failed = FALSE;
-
   /* special handling for the common cases for mono and stereo */
   if (num == 1 && fpos[0] == FRONT_CHANNEL_CENTER) {
     GST_DEBUG_OBJECT (faad, "mono common case; won't set channel positions");
-    return NULL;
+    pos[0] = GST_AUDIO_CHANNEL_POSITION_MONO;
+    return TRUE;
   } else if (num == 2 && fpos[0] == FRONT_CHANNEL_LEFT
       && fpos[1] == FRONT_CHANNEL_RIGHT) {
     GST_DEBUG_OBJECT (faad, "stereo common case; won't set channel positions");
-    return NULL;
+    pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+    pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+    return TRUE;
   }
 
-  pos = g_new (GstAudioChannelPosition, num);
   for (n = 0; n < num; n++) {
     GST_DEBUG_OBJECT (faad, "faad channel %d as %d", n, fpos[n]);
     switch (fpos[n]) {
@@ -468,7 +412,7 @@ gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
       case FRONT_CHANNEL_CENTER:
         /* argh, mono = center */
         if (num == 1)
-          pos[n] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
+          pos[n] = GST_AUDIO_CHANNEL_POSITION_MONO;
         else
           pos[n] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
         break;
@@ -488,7 +432,7 @@ gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
         pos[n] = GST_AUDIO_CHANNEL_POSITION_REAR_CENTER;
         break;
       case LFE_CHANNEL:
-        pos[n] = GST_AUDIO_CHANNEL_POSITION_LFE;
+        pos[n] = GST_AUDIO_CHANNEL_POSITION_LFE1;
         break;
       default:
         GST_DEBUG_OBJECT (faad, "unknown channel %d at %d", fpos[n], n);
@@ -497,413 +441,98 @@ gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
     }
   }
   if (unknown_channel) {
-    g_free (pos);
-    pos = NULL;
     switch (num) {
       case 1:{
         GST_DEBUG_OBJECT (faad,
             "FAAD reports unknown 1 channel mapping. Forcing to mono");
+        pos[0] = GST_AUDIO_CHANNEL_POSITION_MONO;
         break;
       }
       case 2:{
         GST_DEBUG_OBJECT (faad,
             "FAAD reports unknown 2 channel mapping. Forcing to stereo");
+        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
         break;
       }
       default:{
         GST_WARNING_OBJECT (faad,
             "Unsupported FAAD channel position 0x%x encountered", fpos[n]);
-        *channel_map_failed = TRUE;
+        return FALSE;
         break;
       }
     }
   }
 
-  return pos;
-}
-
-static void
-clear_queued (GstFaad * faad)
-{
-  g_list_foreach (faad->queued, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (faad->queued);
-  faad->queued = NULL;
-}
-
-static GstFlowReturn
-flush_queued (GstFaad * faad)
-{
-  GstFlowReturn ret = GST_FLOW_OK;
-
-  while (faad->queued) {
-    GstBuffer *buf = GST_BUFFER_CAST (faad->queued->data);
-
-    GST_LOG_OBJECT (faad, "pushing buffer %p, timestamp %"
-        GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT, buf,
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
-
-    /* iterate ouput queue an push downstream */
-    ret = gst_pad_push (faad->srcpad, buf);
-
-    faad->queued = g_list_delete_link (faad->queued, faad->queued);
-  }
-  return ret;
-}
-
-static GstFlowReturn
-gst_faad_drain (GstFaad * faad)
-{
-  GstFlowReturn ret = GST_FLOW_OK;
-
-  if (faad->segment.rate < 0.0) {
-    /* if we have some queued frames for reverse playback, flush
-     * them now */
-    ret = flush_queued (faad);
-  } else {
-    /* squeeze any possible remaining frames that are pending sync */
-    gst_faad_chain (faad->sinkpad, NULL);
-  }
-  return ret;
-}
-
-static gboolean
-gst_faad_do_raw_seek (GstFaad * faad, GstEvent * event)
-{
-  GstSeekFlags flags;
-  GstSeekType start_type, end_type;
-  GstFormat format;
-  gdouble rate;
-  gint64 start, start_time;
-
-  gst_event_parse_seek (event, &rate, &format, &flags, &start_type,
-      &start_time, &end_type, NULL);
-
-  if (rate != 1.0 ||
-      format != GST_FORMAT_TIME ||
-      start_type != GST_SEEK_TYPE_SET || end_type != GST_SEEK_TYPE_NONE) {
-    return FALSE;
-  }
-
-  if (!gst_faad_src_convert (faad, GST_FORMAT_TIME, start_time,
-          GST_FORMAT_BYTES, &start)) {
-    return FALSE;
-  }
-
-  event = gst_event_new_seek (1.0, GST_FORMAT_BYTES, flags,
-      GST_SEEK_TYPE_SET, start, GST_SEEK_TYPE_NONE, -1);
-
-  GST_DEBUG_OBJECT (faad, "seeking to %" GST_TIME_FORMAT " at byte offset %"
-      G_GINT64_FORMAT, GST_TIME_ARGS (start_time), start);
-
-  return gst_pad_push_event (faad->sinkpad, event);
-}
-
-static gboolean
-gst_faad_src_event (GstPad * pad, GstEvent * event)
-{
-  GstFaad *faad;
-  gboolean res;
-
-  faad = GST_FAAD (gst_pad_get_parent (pad));
-
-  GST_LOG_OBJECT (faad, "Handling %s event", GST_EVENT_TYPE_NAME (event));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_SEEK:{
-      /* try upstream first, there might be a demuxer */
-      gst_event_ref (event);
-      if (!(res = gst_pad_push_event (faad->sinkpad, event))) {
-        res = gst_faad_do_raw_seek (faad, event);
-      }
-      gst_event_unref (event);
-      break;
-    }
-    default:
-      res = gst_pad_push_event (faad->sinkpad, event);
-      break;
-  }
-
-  gst_object_unref (faad);
-  return res;
-}
-
-static gboolean
-gst_faad_sink_event (GstPad * pad, GstEvent * event)
-{
-  GstFaad *faad;
-  gboolean res = TRUE;
-
-  faad = GST_FAAD (gst_pad_get_parent (pad));
-
-  GST_LOG_OBJECT (faad, "Handling %s event", GST_EVENT_TYPE_NAME (event));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_faad_reset_stream_state (faad);
-      res = gst_pad_push_event (faad->srcpad, event);
-      break;
-    case GST_EVENT_EOS:
-      gst_faad_drain (faad);
-      gst_faad_reset_stream_state (faad);
-      res = gst_pad_push_event (faad->srcpad, event);
-      break;
-    case GST_EVENT_NEWSEGMENT:
-    {
-      GstFormat fmt;
-      gboolean is_update;
-      gint64 start, end, base;
-      gdouble rate;
-
-      gst_event_parse_new_segment (event, &is_update, &rate, &fmt, &start,
-          &end, &base);
-
-      /* drain queued buffers before we activate the new segment */
-      gst_faad_drain (faad);
-
-      if (fmt == GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (faad,
-            "Got NEWSEGMENT event in GST_FORMAT_TIME, passing on (%"
-            GST_TIME_FORMAT " - %" GST_TIME_FORMAT ")", GST_TIME_ARGS (start),
-            GST_TIME_ARGS (end));
-        gst_segment_set_newsegment (&faad->segment, is_update, rate, fmt, start,
-            end, base);
-      } else if (fmt == GST_FORMAT_BYTES) {
-        gint64 new_start = 0;
-        gint64 new_end = -1;
-
-        GST_DEBUG_OBJECT (faad, "Got NEWSEGMENT event in GST_FORMAT_BYTES (%"
-            G_GUINT64_FORMAT " - %" G_GUINT64_FORMAT ")", start, end);
-
-        if (gst_faad_src_convert (faad, GST_FORMAT_BYTES, start,
-                GST_FORMAT_TIME, &new_start)) {
-          if (end != -1) {
-            gst_faad_src_convert (faad, GST_FORMAT_BYTES, end,
-                GST_FORMAT_TIME, &new_end);
-          }
-        } else {
-          GST_DEBUG_OBJECT (faad,
-              "no average bitrate yet, sending newsegment with start at 0");
-        }
-        gst_event_unref (event);
-
-        event = gst_event_new_new_segment (is_update, rate,
-            GST_FORMAT_TIME, new_start, new_end, new_start);
-
-        gst_segment_set_newsegment (&faad->segment, is_update, rate,
-            GST_FORMAT_TIME, new_start, new_end, new_start);
-
-        GST_DEBUG_OBJECT (faad,
-            "Sending new NEWSEGMENT event, time %" GST_TIME_FORMAT
-            " - %" GST_TIME_FORMAT, GST_TIME_ARGS (new_start),
-            GST_TIME_ARGS (new_end));
-
-        faad->next_ts = new_start;
-        faad->prev_ts = GST_CLOCK_TIME_NONE;
-      }
-
-      res = gst_pad_push_event (faad->srcpad, event);
-      break;
-    }
-    default:
-      res = gst_pad_event_default (pad, event);
-      break;
-  }
-
-  gst_object_unref (faad);
-  return res;
-}
-
-static gboolean
-gst_faad_src_convert (GstFaad * faad, GstFormat src_format, gint64 src_val,
-    GstFormat dest_format, gint64 * dest_val)
-{
-  guint64 bytes_in, time_out, val;
-
-  if (src_format == dest_format) {
-    if (dest_val)
-      *dest_val = src_val;
-    return TRUE;
-  }
-
-  GST_OBJECT_LOCK (faad);
-  bytes_in = faad->bytes_in;
-  time_out = faad->sum_dur_out;
-  GST_OBJECT_UNLOCK (faad);
-
-  if (bytes_in == 0 || time_out == 0)
-    return FALSE;
-
-  /* convert based on the average bitrate so far */
-  if (src_format == GST_FORMAT_BYTES && dest_format == GST_FORMAT_TIME) {
-    val = gst_util_uint64_scale (src_val, time_out, bytes_in);
-  } else if (src_format == GST_FORMAT_TIME && dest_format == GST_FORMAT_BYTES) {
-    val = gst_util_uint64_scale (src_val, bytes_in, time_out);
-  } else {
-    return FALSE;
-  }
-
-  if (dest_val)
-    *dest_val = (gint64) val;
-
   return TRUE;
 }
 
-static gboolean
-gst_faad_src_query (GstPad * pad, GstQuery * query)
-{
-  gboolean res = FALSE;
-  GstFaad *faad;
-  GstPad *peer = NULL;
-
-  faad = GST_FAAD (gst_pad_get_parent (pad));
-
-  GST_LOG_OBJECT (faad, "processing %s query", GST_QUERY_TYPE_NAME (query));
-
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_DURATION:{
-      GstFormat format;
-      gint64 len_bytes, duration;
-
-      /* try upstream first, in case there's a demuxer */
-      if ((res = gst_pad_query_default (pad, query)))
-        break;
-
-      gst_query_parse_duration (query, &format, NULL);
-      if (format != GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (faad, "query failed: can't handle format %s",
-            gst_format_get_name (format));
-        break;
-      }
-
-      peer = gst_pad_get_peer (faad->sinkpad);
-      if (peer == NULL)
-        break;
-
-      format = GST_FORMAT_BYTES;
-      if (!gst_pad_query_duration (peer, &format, &len_bytes)) {
-        GST_DEBUG_OBJECT (faad, "query failed: failed to get upstream length");
-        break;
-      }
-
-      res = gst_faad_src_convert (faad, GST_FORMAT_BYTES, len_bytes,
-          GST_FORMAT_TIME, &duration);
-
-      if (res) {
-        gst_query_set_duration (query, GST_FORMAT_TIME, duration);
-
-        GST_LOG_OBJECT (faad, "duration estimate: %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (duration));
-      }
-      break;
-    }
-    case GST_QUERY_POSITION:{
-      GstFormat format;
-      gint64 pos_bytes, pos;
-
-      /* try upstream first, in case there's a demuxer */
-      if ((res = gst_pad_query_default (pad, query)))
-        break;
-
-      gst_query_parse_position (query, &format, NULL);
-      if (format != GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (faad, "query failed: can't handle format %s",
-            gst_format_get_name (format));
-        break;
-      }
-
-      peer = gst_pad_get_peer (faad->sinkpad);
-      if (peer == NULL)
-        break;
-
-      format = GST_FORMAT_BYTES;
-      if (!gst_pad_query_position (peer, &format, &pos_bytes)) {
-        GST_OBJECT_LOCK (faad);
-        pos = faad->next_ts;
-        GST_OBJECT_UNLOCK (faad);
-        res = TRUE;
-      } else {
-        res = gst_faad_src_convert (faad, GST_FORMAT_BYTES, pos_bytes,
-            GST_FORMAT_TIME, &pos);
-      }
-
-      if (res) {
-        gst_query_set_position (query, GST_FORMAT_TIME, pos);
-      }
-      break;
-    }
-    default:
-      res = gst_pad_query_default (pad, query);
-      break;
-  }
-
-  if (peer)
-    gst_object_unref (peer);
-
-  gst_object_unref (faad);
-  return res;
-}
-
-
 static gboolean
 gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
 {
-  GstAudioChannelPosition *pos;
   gboolean ret;
-  gboolean channel_map_failed;
-  GstCaps *caps;
   gboolean fmt_change = FALSE;
+  GstAudioInfo ainfo;
+  gint i;
 
   /* see if we need to renegotiate */
   if (info->samplerate != faad->samplerate ||
       info->channels != faad->channels || !faad->channel_positions) {
     fmt_change = TRUE;
   } else {
-    gint i;
-
     for (i = 0; i < info->channels; i++) {
-      if (info->channel_position[i] != faad->channel_positions[i])
+      if (info->channel_position[i] != faad->channel_positions[i]) {
         fmt_change = TRUE;
+        break;
+      }
     }
   }
 
-  if (G_LIKELY (!fmt_change))
+  if (G_LIKELY (gst_pad_has_current_caps (GST_AUDIO_DECODER_SRC_PAD (faad))
+          && !fmt_change))
     return TRUE;
 
+
   /* store new negotiation information */
   faad->samplerate = info->samplerate;
   faad->channels = info->channels;
   g_free (faad->channel_positions);
   faad->channel_positions = g_memdup (info->channel_position, faad->channels);
 
-  caps = gst_caps_new_simple ("audio/x-raw-int",
-      "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "signed", G_TYPE_BOOLEAN, TRUE,
-      "width", G_TYPE_INT, 16,
-      "depth", G_TYPE_INT, 16,
-      "rate", G_TYPE_INT, faad->samplerate,
-      "channels", G_TYPE_INT, faad->channels, NULL);
-
+  /* FIXME: Use the GstAudioInfo of GstAudioDecoder for all of this */
+  gst_audio_info_init (&ainfo);
+  gst_audio_info_set_format (&ainfo, GST_AUDIO_FORMAT_S16, faad->samplerate,
+      faad->channels, NULL);
   faad->bps = 16 / 8;
 
-  channel_map_failed = FALSE;
-  pos =
-      gst_faad_chanpos_to_gst (faad, faad->channel_positions, faad->channels,
-      &channel_map_failed);
-  if (channel_map_failed) {
+  if (!gst_faad_chanpos_to_gst (faad, faad->channel_positions,
+          faad->aac_positions, faad->channels)) {
     GST_DEBUG_OBJECT (faad, "Could not map channel positions");
-    gst_caps_unref (caps);
     return FALSE;
   }
-  if (pos) {
-    gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
-    g_free (pos);
+  memcpy (ainfo.position, faad->aac_positions,
+      faad->channels * sizeof (GstAudioChannelPosition));
+  gst_audio_channel_positions_to_valid_order (ainfo.position, faad->channels);
+  memcpy (faad->gst_positions, ainfo.position,
+      faad->channels * sizeof (GstAudioChannelPosition));
+  /* Unset UNPOSITIONED flag */
+  if (ainfo.position[0] != GST_AUDIO_CHANNEL_POSITION_NONE)
+    ainfo.flags &= ~GST_AUDIO_FLAG_UNPOSITIONED;
+
+  /* get the remap table */
+  memset (faad->reorder_map, 0, sizeof (faad->reorder_map));
+  faad->need_reorder = FALSE;
+  if (gst_audio_get_channel_reorder_map (faad->channels, faad->aac_positions,
+          faad->gst_positions, faad->reorder_map)) {
+    for (i = 0; i < faad->channels; i++) {
+      GST_DEBUG_OBJECT (faad, "remap %d -> %d", i, faad->reorder_map[i]);
+      if (faad->reorder_map[i] != i) {
+        faad->need_reorder = TRUE;
+      }
+    }
   }
 
-  GST_DEBUG_OBJECT (faad, "New output caps: %" GST_PTR_FORMAT, caps);
-
-  ret = gst_pad_set_caps (faad->srcpad, caps);
-  gst_caps_unref (caps);
+  ret = gst_audio_decoder_set_output_format (GST_AUDIO_DECODER (faad), &ainfo);
 
   return ret;
 }
@@ -917,12 +546,13 @@ gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
  * gst/typefind/) for ADTS because 12 bits isn't very reliable.
  */
 static gboolean
-gst_faad_sync (GstFaad * faad, guint8 * data, guint size, gboolean next,
-    guint * off)
+gst_faad_sync (GstFaad * faad, const guint8 * data, guint size, gboolean next,
+    gint * off, gint * length)
 {
   guint n = 0;
   gint snc;
   gboolean ret = FALSE;
+  guint len = 0;
 
   GST_LOG_OBJECT (faad, "Finding syncpoint");
 
@@ -935,8 +565,6 @@ gst_faad_sync (GstFaad * faad, guint8 * data, guint size, gboolean next,
     if ((snc & 0xfff6) == 0xfff0) {
       /* we have an ADTS syncpoint. Parse length and find
        * next syncpoint. */
-      guint len;
-
       GST_LOG_OBJECT (faad,
           "Found one ADTS syncpoint at offset 0x%x, tracing next...", n);
 
@@ -979,10 +607,14 @@ gst_faad_sync (GstFaad * faad, guint8 * data, guint size, gboolean next,
   }
 
 exit:
+
   *off = n;
 
-  if (!ret)
+  if (ret) {
+    *length = len;
+  } else {
     GST_LOG_OBJECT (faad, "Found no syncpoint");
+  }
 
   return ret;
 }
@@ -1005,73 +637,60 @@ looks_like_valid_header (guint8 * input_data, guint input_size)
   return FALSE;
 }
 
-#define FAAD_MAX_ERROR  10
-#define FAAD_MAX_SYNC   10 * 8 * 1024
+static GstFlowReturn
+gst_faad_parse (GstAudioDecoder * dec, GstAdapter * adapter,
+    gint * offset, gint * length)
+{
+  GstFaad *faad;
+  const guint8 *data;
+  guint size;
+  gboolean sync, eos;
+
+  faad = GST_FAAD (dec);
+
+  size = gst_adapter_available (adapter);
+  g_return_val_if_fail (size > 0, GST_FLOW_ERROR);
+
+  gst_audio_decoder_get_parse_state (dec, &sync, &eos);
+
+  if (faad->packetised) {
+    *offset = 0;
+    *length = size;
+    return GST_FLOW_OK;
+  } else {
+    gboolean ret;
+
+    data = gst_adapter_map (adapter, size);
+    ret = gst_faad_sync (faad, data, size, !eos, offset, length);
+    gst_adapter_unmap (adapter);
+
+    return (ret ? GST_FLOW_OK : GST_FLOW_EOS);
+  }
+}
 
 static GstFlowReturn
-gst_faad_chain (GstPad * pad, GstBuffer * buffer)
+gst_faad_handle_frame (GstAudioDecoder * dec, GstBuffer * buffer)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
-  guint input_size;
-  guint available;
-  guchar *input_data;
   GstFaad *faad;
+  GstFlowReturn ret = GST_FLOW_OK;
+  GstMapInfo map;
+  gsize input_size;
+  guchar *input_data;
   GstBuffer *outbuf;
   faacDecFrameInfo info;
   void *out;
-  gboolean run_loop = TRUE;
-  guint sync_off;
-  GstClockTime ts;
-  gboolean next;
 
-  faad = GST_FAAD (gst_pad_get_parent (pad));
+  faad = GST_FAAD (dec);
 
-  if (G_LIKELY (buffer)) {
-    GST_LOG_OBJECT (faad, "buffer of size %d with ts: %" GST_TIME_FORMAT
-        ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buffer),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
+  /* no fancy draining */
+  if (G_UNLIKELY (!buffer))
+    return GST_FLOW_OK;
 
-    if (GST_BUFFER_IS_DISCONT (buffer)) {
-      gst_faad_drain (faad);
-      gst_faad_reset_stream_state (faad);
-      faad->discont = TRUE;
-    }
-
-    gst_adapter_push (faad->adapter, buffer);
-    buffer = NULL;
-    next = TRUE;
-  } else {
-    next = FALSE;
-  }
-
-  ts = gst_adapter_prev_timestamp (faad->adapter, NULL);
-  if (GST_CLOCK_TIME_IS_VALID (ts) && (ts != faad->prev_ts))
-    faad->prev_ts = faad->next_ts = ts;
-
-  available = gst_adapter_available (faad->adapter);
-  input_size = available;
-
-  if (G_UNLIKELY (!available))
-    goto out;
-
-  input_data = (guchar *) gst_adapter_peek (faad->adapter, available);
-
-  if (!faad->packetised) {
-    if (!gst_faad_sync (faad, input_data, input_size, next, &sync_off)) {
-      faad->sync_flush += sync_off;
-      input_size -= sync_off;
-      if (faad->sync_flush > FAAD_MAX_SYNC)
-        goto parse_failed;
-      else
-        goto out;
-    } else {
-      faad->sync_flush = 0;
-      input_data += sync_off;
-      input_size -= sync_off;
-    }
-  }
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
+  input_data = map.data;
+  input_size = map.size;
 
+init:
   /* init if not already done during capsnego */
   if (!faad->init) {
 #if FAAD2_MINOR_VERSION >= 7
@@ -1082,6 +701,8 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
     guint8 ch;
 
     GST_DEBUG_OBJECT (faad, "initialising ...");
+    if (!gst_faad_open_decoder (faad))
+      goto open_failed;
     /* We check if the first data looks like it might plausibly contain
      * appropriate initialisation info... if not, we use our fake_codec_data
      */
@@ -1101,7 +722,6 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
     }
 
     faad->init = TRUE;
-    gst_faad_send_tags (faad);
 
     /* make sure we create new caps below */
     faad->samplerate = 0;
@@ -1109,51 +729,38 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
   }
 
   /* decode cycle */
-  info.bytesconsumed = input_size;
   info.error = 0;
 
-  while ((input_size > 0) && run_loop) {
+  do {
+    GstMapInfo map;
 
-    if (faad->packetised) {
-      /* Only one packet per buffer, no matter how much is really consumed */
-      run_loop = FALSE;
-    } else {
-      if (input_size < FAAD_MIN_STREAMSIZE || info.bytesconsumed <= 0) {
-        break;
+    if (!faad->packetised) {
+      /* faad only really parses ADTS header at Init time, not when decoding,
+       * so monitor for changes and kick faad when needed */
+      if (GST_READ_UINT32_BE (input_data) >> 4 != faad->last_header >> 4) {
+        GST_DEBUG_OBJECT (faad, "ADTS header changed, forcing Init");
+        faad->last_header = GST_READ_UINT32_BE (input_data);
+        /* kick hard */
+        gst_faad_close_decoder (faad);
+        faad->init = FALSE;
+        goto init;
       }
     }
 
     out = faacDecDecode (faad->handle, &info, input_data, input_size);
 
     if (info.error > 0) {
-      /* mark discont for the next buffer */
-      faad->discont = TRUE;
-      /* flush a bit, arranges for resync next time */
-      input_size--;
-      faad->error_count++;
-      /* do not bail out at once, but know when to stop */
-      if (faad->error_count > FAAD_MAX_ERROR)
-        goto decode_failed;
-      else {
-        GST_WARNING_OBJECT (faad, "decoding error: %s",
-            faacDecGetErrorMessage (info.error));
-        goto out;
-      }
+      /* give up on frame and bail out */
+      gst_audio_decoder_finish_frame (dec, NULL, 1);
+      goto decode_failed;
     }
 
-    /* ok again */
-    faad->error_count = 0;
-
     GST_LOG_OBJECT (faad, "%d bytes consumed, %d samples decoded",
         (guint) info.bytesconsumed, (guint) info.samples);
 
-    if (info.bytesconsumed > input_size)
-      info.bytesconsumed = input_size;
-
-    input_size -= info.bytesconsumed;
-    input_data += info.bytesconsumed;
-
     if (out && info.samples > 0) {
+      guint channels, samples;
+
       if (!gst_faad_update_caps (faad, &info))
         goto negotiation_failed;
 
@@ -1161,69 +768,48 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
       if (info.samples > G_MAXUINT / faad->bps)
         goto sample_overflow;
 
-      /* play decoded data */
-      if (info.samples > 0) {
-        guint bufsize = info.samples * faad->bps;
-        guint num_samples = info.samples / faad->channels;
+      channels = faad->channels;
+      /* note: info.samples is total samples, not per channel */
+      samples = info.samples / channels;
 
-        /* note: info.samples is total samples, not per channel */
-        ret =
-            gst_pad_alloc_buffer_and_set_caps (faad->srcpad, 0, bufsize,
-            GST_PAD_CAPS (faad->srcpad), &outbuf);
-        if (ret != GST_FLOW_OK)
-          goto out;
+      /* FIXME, add bufferpool and allocator support to the base class */
+      outbuf = gst_buffer_new_allocate (NULL, info.samples * faad->bps, NULL);
 
-        memcpy (GST_BUFFER_DATA (outbuf), out, GST_BUFFER_SIZE (outbuf));
-        GST_BUFFER_OFFSET (outbuf) =
-            GST_CLOCK_TIME_TO_FRAMES (faad->next_ts, faad->samplerate);
-        GST_BUFFER_TIMESTAMP (outbuf) = faad->next_ts;
-        GST_BUFFER_DURATION (outbuf) =
-            GST_FRAMES_TO_CLOCK_TIME (num_samples, faad->samplerate);
+      gst_buffer_map (outbuf, &map, GST_MAP_READWRITE);
+      if (faad->need_reorder) {
+        gint16 *dest, *src, i, j;
 
-        GST_OBJECT_LOCK (faad);
-        faad->next_ts += GST_BUFFER_DURATION (outbuf);
-        faad->sum_dur_out += GST_BUFFER_DURATION (outbuf);
-        faad->bytes_in += info.bytesconsumed;
-        GST_OBJECT_UNLOCK (faad);
+        dest = (gint16 *) map.data;
+        src = (gint16 *) out;
 
-        if ((outbuf = gst_audio_buffer_clip (outbuf, &faad->segment,
-                    faad->samplerate, faad->bps * faad->channels))) {
-          GST_LOG_OBJECT (faad,
-              "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
-              GST_BUFFER_OFFSET (outbuf),
-              GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
-
-          if (faad->discont) {
-            GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-            faad->discont = FALSE;
+        for (i = 0; i < samples; i++) {
+          for (j = 0; j < channels; j++) {
+            dest[faad->reorder_map[j]] = *src++;
           }
-
-          if (faad->segment.rate > 0.0) {
-            ret = gst_pad_push (faad->srcpad, outbuf);
-          } else {
-            /* reverse playback, queue frame till later when we get a discont. */
-            GST_LOG_OBJECT (faad, "queued frame");
-            faad->queued = g_list_prepend (faad->queued, outbuf);
-            ret = GST_FLOW_OK;
-          }
-          if (ret != GST_FLOW_OK)
-            goto out;
+          dest += channels;
         }
+      } else {
+        memcpy (map.data, out, map.size);
       }
+      gst_buffer_unmap (outbuf, &map);
+
+      ret = gst_audio_decoder_finish_frame (dec, outbuf, 1);
     }
-  }
+  } while (FALSE);
 
 out:
-  /* in raw case: (pretend) all consumed */
-  if (faad->packetised)
-    input_size = 0;
-  gst_adapter_flush (faad->adapter, available - input_size);
-
-  gst_object_unref (faad);
+  gst_buffer_unmap (buffer, &map);
 
   return ret;
 
 /* ERRORS */
+open_failed:
+  {
+    GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
+        ("Failed to open decoder"));
+    ret = GST_FLOW_ERROR;
+    goto out;
+  }
 init_failed:
   {
     GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
@@ -1240,9 +826,10 @@ init2_failed:
   }
 decode_failed:
   {
-    GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
-        ("decoding error: %s", faacDecGetErrorMessage (info.error)));
-    ret = GST_FLOW_ERROR;
+    GST_AUDIO_DECODER_ERROR (faad, 1, STREAM, DECODE, (NULL),
+        ("decoding error: %s", faacDecGetErrorMessage (info.error)), ret);
+    if (ret == GST_FLOW_OK)
+      gst_audio_decoder_finish_frame (dec, NULL, 1);
     goto out;
   }
 negotiation_failed:
@@ -1259,13 +846,12 @@ sample_overflow:
     ret = GST_FLOW_ERROR;
     goto out;
   }
-parse_failed:
-  {
-    GST_ELEMENT_ERROR (faad, STREAM, DECODE, (NULL),
-        ("failed to parse non-packetized stream"));
-    ret = GST_FLOW_ERROR;
-    goto out;
-  }
+}
+
+static void
+gst_faad_flush (GstAudioDecoder * dec, gboolean hard)
+{
+  gst_faad_reset_stream_state (GST_FAAD (dec));
 }
 
 static gboolean
@@ -1302,40 +888,6 @@ gst_faad_close_decoder (GstFaad * faad)
   }
 }
 
-static GstStateChangeReturn
-gst_faad_change_state (GstElement * element, GstStateChange transition)
-{
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
-  GstFaad *faad = GST_FAAD (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      if (!gst_faad_open_decoder (faad))
-        return GST_STATE_CHANGE_FAILURE;
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    default:
-      break;
-  }
-
-  if (GST_ELEMENT_CLASS (parent_class)->change_state)
-    ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_faad_reset (faad);
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_faad_close_decoder (faad);
-      break;
-    default:
-      break;
-  }
-
-  return ret;
-}
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -1344,6 +896,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "faad",
+    faad,
     "Free AAC Decoder (FAAD)",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
