@@ -142,24 +142,10 @@ static GstStateChangeReturn gst_kate_enc_change_state (GstElement * element,
 static gboolean gst_kate_enc_sink_event (GstPad * pad, GstEvent * event);
 static const GstQueryType *gst_kate_enc_source_query_type (GstPad * pad);
 static gboolean gst_kate_enc_source_query (GstPad * pad, GstQuery * query);
-static void gst_kate_enc_add_interfaces (GType kateenc_type);
 
-GST_BOILERPLATE_FULL (GstKateEnc, gst_kate_enc, GstElement,
-    GST_TYPE_ELEMENT, gst_kate_enc_add_interfaces);
-
-static void
-gst_kate_enc_base_init (gpointer gclass)
-{
-
-  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
-
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
-  gst_element_class_set_details_simple (element_class, "Kate stream encoder",
-      "Codec/Encoder/Subtitle",
-      "Encodes Kate streams from text or subpictures",
-      "Vincent Penquerc'h <ogg.k.ogg.k@googlemail.com>");
-}
+#define gst_kate_enc_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstKateEnc, gst_kate_enc, GST_TYPE_ELEMENT,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
 
 /* initialize the plugin's class */
 static void
@@ -171,8 +157,6 @@ gst_kate_enc_class_init (GstKateEncClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_kate_enc_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_kate_enc_get_property);
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_kate_enc_dispose);
@@ -226,15 +210,16 @@ gst_kate_enc_class_init (GstKateEncClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_kate_enc_change_state);
-}
 
-static void
-gst_kate_enc_add_interfaces (GType kateenc_type)
-{
-  static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
 
-  g_type_add_interface_static (kateenc_type, GST_TYPE_TAG_SETTER,
-      &tag_setter_info);
+  gst_element_class_set_details_simple (gstelement_class, "Kate stream encoder",
+      "Codec/Encoder/Subtitle",
+      "Encodes Kate streams from text or subpictures",
+      "Vincent Penquerc'h <ogg.k.ogg.k@googlemail.com>");
 }
 
 /* initialize the new element
@@ -243,7 +228,7 @@ gst_kate_enc_add_interfaces (GType kateenc_type)
  * initialize structure
  */
 static void
-gst_kate_enc_init (GstKateEnc * ke, GstKateEncClass * gclass)
+gst_kate_enc_init (GstKateEnc * ke)
 {
   GST_DEBUG_OBJECT (ke, "gst_kate_enc_init");
 
@@ -252,8 +237,6 @@ gst_kate_enc_init (GstKateEnc * ke, GstKateEncClass * gclass)
       GST_DEBUG_FUNCPTR (gst_kate_enc_chain));
   gst_pad_set_event_function (ke->sinkpad,
       GST_DEBUG_FUNCPTR (gst_kate_enc_sink_event));
-  gst_pad_set_setcaps_function (ke->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_kate_enc_setcaps));
   gst_element_add_pad (GST_ELEMENT (ke), ke->sinkpad);
 
   ke->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
@@ -408,14 +391,14 @@ gst_kate_enc_create_buffer (GstKateEnc * ke, kate_packet * kp,
   g_return_val_if_fail (kp != NULL, NULL);
   g_return_val_if_fail (kp->data != NULL, NULL);
 
-  buffer = gst_buffer_try_new_and_alloc (kp->nbytes);
+  buffer = gst_buffer_new_allocate (NULL, kp->nbytes, NULL);
   if (G_UNLIKELY (!buffer)) {
     GST_WARNING_OBJECT (ke, "Failed to allocate buffer for %u bytes",
         (guint) kp->nbytes);
     return NULL;
   }
 
-  memcpy (GST_BUFFER_DATA (buffer), kp->data, kp->nbytes);
+  gst_buffer_fill (buffer, 0, kp->data, kp->nbytes);
 
   /* same system as other Ogg codecs, as per ext/ogg/README:
      OFFSET_END is the granulepos
@@ -426,10 +409,6 @@ gst_kate_enc_create_buffer (GstKateEnc * ke, kate_packet * kp,
   GST_BUFFER_TIMESTAMP (buffer) = timestamp;
   GST_BUFFER_DURATION (buffer) = duration;
 
-  /* data packets are each on their own page */
-//  if (!header)
-//    GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
-
   return buffer;
 }
 
@@ -445,9 +424,6 @@ gst_kate_enc_push_buffer (GstKateEnc * ke, GstBuffer * buffer)
         GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer);
   }
 
-  /* Hack to flush each packet on its own page - taken off the CMML encoder element */
-  GST_BUFFER_DURATION (buffer) = G_MAXINT64;
-
   flow = gst_pad_push (ke->srcpad, buffer);
   if (G_UNLIKELY (flow != GST_FLOW_OK)) {
     GST_WARNING_OBJECT (ke->srcpad, "push flow: %s", gst_flow_get_name (flow));
@@ -625,7 +601,8 @@ gst_kate_enc_send_headers (GstKateEnc * ke)
       break;
     } else {
       GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-          ("kate_encode_headers: %d", ret));
+          ("Failed encoding headers: %s",
+              gst_kate_util_get_error_message (ret)));
       rflow = GST_FLOW_ERROR;
       break;
     }
@@ -642,16 +619,6 @@ gst_kate_enc_send_headers (GstKateEnc * ke)
     if (caps) {
       GST_DEBUG_OBJECT (ke, "here are the caps: %" GST_PTR_FORMAT, caps);
       gst_pad_set_caps (ke->srcpad, caps);
-
-      GST_LOG_OBJECT (ke, "setting caps on headers");
-      item = headers;
-      while (item) {
-        GstBuffer *buffer = item->data;
-        GST_LOG_OBJECT (ke, "settings caps on header %p", buffer);
-        gst_buffer_set_caps (buffer, caps);
-        item = item->next;
-      }
-
       gst_caps_unref (caps);
 
       GST_LOG_OBJECT (ke, "pushing headers");
@@ -683,7 +650,8 @@ gst_kate_enc_flush_headers (GstKateEnc * ke)
       ke->headers_sent = TRUE;
       GST_INFO_OBJECT (ke, "headers flushed");
     } else {
-      GST_WARNING_OBJECT (ke, "Failed to flush headers: %d", rflow);
+      GST_WARNING_OBJECT (ke, "Failed to flush headers: %s",
+          gst_flow_get_name (rflow));
     }
   }
   return rflow;
@@ -721,7 +689,8 @@ gst_kate_enc_generate_keepalive (GstKateEnc * ke, GstClockTime timestamp)
   GST_DEBUG_OBJECT (ke, "keepalive at %f", t);
   ret = kate_encode_keepalive (&ke->k, t, &kp);
   if (ret < 0) {
-    GST_WARNING_OBJECT (ke, "Failed to encode keepalive packet: %d", ret);
+    GST_WARNING_OBJECT (ke, "Failed to encode keepalive packet: %s",
+        gst_kate_util_get_error_message (ret));
   } else {
     kate_int64_t granpos = kate_encode_get_granule (&ke->k);
     GST_LOG_OBJECT (ke, "Keepalive packet encoded");
@@ -751,7 +720,8 @@ gst_kate_enc_flush_waiting (GstKateEnc * ke, GstClockTime now)
     ret = kate_encode_text (&ke->k, t0, t1, "", 0, &kp);
     if (G_UNLIKELY (ret < 0)) {
       GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-          ("kate_encode_text: %d", ret));
+          ("Failed to encode text packet: %s",
+              gst_kate_util_get_error_message (ret)));
       rflow = GST_FLOW_ERROR;
     } else {
       rflow =
@@ -762,7 +732,8 @@ gst_kate_enc_flush_waiting (GstKateEnc * ke, GstClockTime now)
     if (rflow == GST_FLOW_OK) {
       GST_DEBUG_OBJECT (ke, "delayed SPU packet flushed");
     } else {
-      GST_WARNING_OBJECT (ke, "Failed to flush delayed SPU packet: %d", rflow);
+      GST_WARNING_OBJECT (ke, "Failed to flush delayed SPU packet: %s",
+          gst_flow_get_name (rflow));
     }
 
     /* forget it even if we couldn't flush it */
@@ -867,24 +838,26 @@ gst_kate_enc_chain_spu (GstKateEnc * ke, GstBuffer * buf)
 #endif
     GST_DEBUG_OBJECT (ke, "Encoding %ux%u SPU: (%u bytes) from %f to %f",
         (guint) kbitmap->width, (guint) kbitmap->height,
-        GST_BUFFER_SIZE (buf), t0, t1);
+        gst_buffer_get_size (buf), t0, t1);
 
     ret = kate_encode_set_region (&ke->k, kregion);
     if (G_UNLIKELY (ret < 0)) {
       GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-          ("kate_encode_set_region: %d", ret));
+          ("Failed to set region: %s", gst_kate_util_get_error_message (ret)));
       rflow = GST_FLOW_ERROR;
     } else {
       ret = kate_encode_set_palette (&ke->k, kpalette);
       if (G_UNLIKELY (ret < 0)) {
         GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-            ("kate_encode_set_palette: %d", ret));
+            ("Failed to set palette: %s",
+                gst_kate_util_get_error_message (ret)));
         rflow = GST_FLOW_ERROR;
       } else {
         ret = kate_encode_set_bitmap (&ke->k, kbitmap);
         if (G_UNLIKELY (ret < 0)) {
           GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-              ("kate_encode_set_bitmap: %d", ret));
+              ("Failed to set bitmap: %s",
+                  gst_kate_util_get_error_message (ret)));
           rflow = GST_FLOW_ERROR;
         } else {
           /* Some SPUs have no hide time - so I'm going to delay the encoding of the packet
@@ -906,7 +879,8 @@ gst_kate_enc_chain_spu (GstKateEnc * ke, GstBuffer * buf)
             ret = kate_encode_text (&ke->k, t0, t1, "", 0, &kp);
             if (G_UNLIKELY (ret < 0)) {
               GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-                  ("Failed to encode empty text for SPU buffer: %d", ret));
+                  ("Failed to encode empty text for SPU buffer: %s",
+                      gst_kate_util_get_error_message (ret)));
               rflow = GST_FLOW_ERROR;
             } else {
               rflow =
@@ -948,32 +922,36 @@ gst_kate_enc_chain_text (GstKateEnc * ke, GstBuffer * buf,
 
   if (G_UNLIKELY (ret < 0)) {
     GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-        ("kate_encode_set_markup_type: %d", ret));
+        ("Failed to set markup type: %s",
+            gst_kate_util_get_error_message (ret)));
     rflow = GST_FLOW_ERROR;
   } else {
-    const char *text = (const char *) GST_BUFFER_DATA (buf);
-    if (text) {
-      size_t text_len = GST_BUFFER_SIZE (buf);
-      kate_float t0 = start / (double) GST_SECOND;
-      kate_float t1 = stop / (double) GST_SECOND;
-      GST_LOG_OBJECT (ke, "Encoding text: %*.*s (%u bytes) from %f to %f",
-          (int) text_len, (int) text_len, GST_BUFFER_DATA (buf),
-          GST_BUFFER_SIZE (buf), t0, t1);
-      ret = kate_encode_text (&ke->k, t0, t1, text, text_len, &kp);
-      if (G_UNLIKELY (ret < 0)) {
-        GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-            ("Failed to encode text: %d", ret));
-        rflow = GST_FLOW_ERROR;
-      } else {
-        rflow =
-            gst_kate_enc_chain_push_packet (ke, &kp, start, stop - start + 1);
-      }
-    } else {
-      /* FIXME: this should not be an error, we should ignore it and move on */
-      GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
-          ("no text in text packet"));
-      rflow = GST_FLOW_ERROR;
+    const char *text;
+    size_t text_len;
+    gboolean need_unmap = TRUE;
+    kate_float t0 = start / (double) GST_SECOND;
+    kate_float t1 = stop / (double) GST_SECOND;
+
+    text = gst_buffer_map (buf, &text_len, NULL, GST_MAP_READ);
+    if (text == NULL) {
+      text = "";
+      text_len = 0;
+      need_unmap = FALSE;
     }
+
+    GST_LOG_OBJECT (ke, "Encoding text: %*.*s (%u bytes) from %f to %f",
+        (int) text_len, (int) text_len, GST_BUFFER_DATA (buf),
+        GST_BUFFER_SIZE (buf), t0, t1);
+    ret = kate_encode_text (&ke->k, t0, t1, text, text_len, &kp);
+    if (G_UNLIKELY (ret < 0)) {
+      GST_ELEMENT_ERROR (ke, STREAM, ENCODE, (NULL),
+          ("Failed to encode text: %s", gst_kate_util_get_error_message (ret)));
+      rflow = GST_FLOW_ERROR;
+    } else {
+      rflow = gst_kate_enc_chain_push_packet (ke, &kp, start, stop - start + 1);
+    }
+    if (need_unmap)
+      gst_buffer_unmap (buf, text, text_len);
   }
 
   return rflow;
@@ -990,10 +968,10 @@ gst_kate_enc_chain (GstPad * pad, GstBuffer * buf)
   GstCaps *caps;
   const gchar *mime_type = NULL;
 
-  GST_DEBUG_OBJECT (ke, "got packet, %u bytes", GST_BUFFER_SIZE (buf));
+  GST_DEBUG_OBJECT (ke, "got packet, %u bytes", gst_buffer_get_size (buf));
 
   /* get the type of the data we're being sent */
-  caps = GST_PAD_CAPS (pad);
+  caps = gst_pad_get_current_caps (pad);
   if (G_UNLIKELY (caps == NULL)) {
     GST_WARNING_OBJECT (ke, "No input caps set");
     rflow = GST_FLOW_NOT_NEGOTIATED;
@@ -1025,6 +1003,7 @@ gst_kate_enc_chain (GstPad * pad, GstBuffer * buf)
     } else {
       GST_WARNING_OBJECT (ke, "Packet has no MIME type, ignored");
     }
+    gst_caps_unref (caps);
   }
 
   gst_buffer_unref (buf);
@@ -1047,27 +1026,29 @@ gst_kate_enc_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      ke->tags = gst_tag_list_new ();
+      ke->tags = gst_tag_list_new_empty ();
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_DEBUG_OBJECT (ke, "READY -> PAUSED, initializing kate state");
       ret = kate_info_init (&ke->ki);
       if (ret < 0) {
-        GST_WARNING_OBJECT (ke, "failed to initialize kate info structure: %d",
-            ret);
+        GST_WARNING_OBJECT (ke, "failed to initialize kate info structure: %s",
+            gst_kate_util_get_error_message (ret));
         break;
       }
       if (ke->language) {
         ret = kate_info_set_language (&ke->ki, ke->language);
         if (ret < 0) {
-          GST_WARNING_OBJECT (ke, "failed to set stream language: %d", ret);
+          GST_WARNING_OBJECT (ke, "failed to set stream language: %s",
+              gst_kate_util_get_error_message (ret));
           break;
         }
       }
       if (ke->category) {
         ret = kate_info_set_category (&ke->ki, ke->category);
         if (ret < 0) {
-          GST_WARNING_OBJECT (ke, "failed to set stream category: %d", ret);
+          GST_WARNING_OBJECT (ke, "failed to set stream category: %s",
+              gst_kate_util_get_error_message (ret));
           break;
         }
       }
@@ -1075,18 +1056,21 @@ gst_kate_enc_change_state (GstElement * element, GstStateChange transition)
           kate_info_set_original_canvas_size (&ke->ki,
           ke->original_canvas_width, ke->original_canvas_height);
       if (ret < 0) {
-        GST_WARNING_OBJECT (ke, "failed to set original canvas size: %d", ret);
+        GST_WARNING_OBJECT (ke, "failed to set original canvas size: %s",
+            gst_kate_util_get_error_message (ret));
         break;
       }
       ret = kate_comment_init (&ke->kc);
       if (ret < 0) {
         GST_WARNING_OBJECT (ke,
-            "failed to initialize kate comment structure: %d", ret);
+            "failed to initialize kate comment structure: %s",
+            gst_kate_util_get_error_message (ret));
         break;
       }
       ret = kate_encode_init (&ke->k, &ke->ki);
       if (ret < 0) {
-        GST_WARNING_OBJECT (ke, "failed to initialize kate state: %d", ret);
+        GST_WARNING_OBJECT (ke, "failed to initialize kate state: %s",
+            gst_kate_util_get_error_message (ret));
         break;
       }
       ke->headers_sent = FALSE;
@@ -1259,38 +1243,46 @@ static gboolean
 gst_kate_enc_sink_event (GstPad * pad, GstEvent * event)
 {
   GstKateEnc *ke = GST_KATE_ENC (gst_pad_get_parent (pad));
-  GstStructure *structure;
+  const GstStructure *structure;
   gboolean ret;
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      ret = gst_kate_enc_setcaps (pad, caps);
+      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_SEGMENT:
       GST_LOG_OBJECT (ke, "Got newsegment event");
       if (ke->initialized) {
         GST_LOG_OBJECT (ke, "ensuring all headers are in");
         if (gst_kate_enc_flush_headers (ke) != GST_FLOW_OK) {
           GST_WARNING_OBJECT (ke, "Failed to flush headers");
         } else {
-          GstFormat format;
-          gint64 timestamp;
+          GstSegment seg;
 
-          gst_event_parse_new_segment (event, NULL, NULL, &format, &timestamp,
-              NULL, NULL);
-          if (format != GST_FORMAT_TIME || !GST_CLOCK_TIME_IS_VALID (timestamp)) {
+          gst_event_copy_segment (event, &seg);
+          if (seg.format != GST_FORMAT_TIME
+              || !GST_CLOCK_TIME_IS_VALID (seg.start)) {
             GST_WARNING_OBJECT (ke,
                 "No time in newsegment event %p, format %d, timestamp %"
-                G_GINT64_FORMAT, event, (int) format, timestamp);
+                G_GINT64_FORMAT, event, (int) seg.format, seg.start);
             /* to be safe, we'd need to generate a keepalive anyway, but we'd have to guess at the timestamp to use; a
                good guess would be the last known timestamp plus the keepalive time, but if we then get a packet with a
                timestamp less than this, it would fail to encode, which would be Bad. If we don't encode a keepalive, we
                run the risk of stalling the pipeline and hanging, which is Very Bad. Oh dear. We can't exit(-1), can we ? */
           } else {
-            float t = timestamp / (double) GST_SECOND;
+            float t = seg.start / (double) GST_SECOND;
 
             if (ke->delayed_spu
                 && t - ke->delayed_start / (double) GST_SECOND >=
                 ke->default_spu_duration) {
               if (G_UNLIKELY (gst_kate_enc_flush_waiting (ke,
-                          timestamp) != GST_FLOW_OK)) {
+                          seg.start) != GST_FLOW_OK)) {
                 GST_WARNING_OBJECT (ke, "Failed to encode delayed packet");
                 /* continue with new segment handling anyway */
               }
@@ -1305,7 +1297,7 @@ gst_kate_enc_sink_event (GstPad * pad, GstEvent * event)
               /* we only generate a keepalive if there is no SPU waiting, as it would
                  mean out of sequence start times - and granulepos */
               if (!ke->delayed_spu) {
-                gst_kate_enc_generate_keepalive (ke, timestamp);
+                gst_kate_enc_generate_keepalive (ke, seg.start);
               }
             }
           }
@@ -1317,7 +1309,7 @@ gst_kate_enc_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_CUSTOM_DOWNSTREAM:
       GST_LOG_OBJECT (ke, "Got custom downstream event");
       /* adapted from the dvdsubdec element */
-      structure = event->structure;
+      structure = gst_event_get_structure (event);
       if (structure != NULL
           && gst_structure_has_name (structure, "application/x-gst-dvd")) {
         if (ke->initialized) {
@@ -1390,7 +1382,8 @@ gst_kate_enc_sink_event (GstPad * pad, GstEvent * event)
 
           ret = kate_encode_finish (&ke->k, -1, &kp);
           if (ret < 0) {
-            GST_WARNING_OBJECT (ke, "Failed to encode EOS packet: %d", ret);
+            GST_WARNING_OBJECT (ke, "Failed to encode EOS packet: %s",
+                gst_kate_util_get_error_message (ret));
           } else {
             kate_int64_t granpos = kate_encode_get_granule (&ke->k);
             GST_LOG_OBJECT (ke, "EOS packet encoded");
