@@ -83,7 +83,8 @@ static GstStaticPadTemplate gst_kate_parse_src_factory =
     GST_STATIC_CAPS ("subtitle/x-kate; application/x-kate")
     );
 
-GST_BOILERPLATE (GstKateParse, gst_kate_parse, GstElement, GST_TYPE_ELEMENT);
+#define gst_kate_parse_parent_class parent_class
+G_DEFINE_TYPE (GstKateParse, gst_kate_parse, GST_TYPE_ELEMENT);
 
 static GstFlowReturn gst_kate_parse_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn gst_kate_parse_change_state (GstElement * element,
@@ -98,21 +99,6 @@ static gboolean gst_kate_parse_convert (GstPad * pad,
 static GstFlowReturn gst_kate_parse_parse_packet (GstKateParse * parse,
     GstBuffer * buf);
 
-static void
-gst_kate_parse_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_kate_parse_src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_kate_parse_sink_factory);
-  gst_element_class_set_details_simple (element_class, "Kate stream parser",
-      "Codec/Parser/Subtitle",
-      "parse raw kate streams",
-      "Vincent Penquerc'h <ogg.k.ogg.k at googlemail dot com>");
-}
-
 static void
 gst_kate_parse_class_init (GstKateParseClass * klass)
 {
@@ -120,11 +106,21 @@ gst_kate_parse_class_init (GstKateParseClass * klass)
 
   gstelement_class->change_state = gst_kate_parse_change_state;
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_kate_parse_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_kate_parse_sink_factory));
+
+  gst_element_class_set_details_simple (gstelement_class, "Kate stream parser",
+      "Codec/Parser/Subtitle",
+      "parse raw kate streams",
+      "Vincent Penquerc'h <ogg.k.ogg.k at googlemail dot com>");
+
   klass->parse_packet = GST_DEBUG_FUNCPTR (gst_kate_parse_parse_packet);
 }
 
 static void
-gst_kate_parse_init (GstKateParse * parse, GstKateParseClass * g_class)
+gst_kate_parse_init (GstKateParse * parse)
 {
   parse->sinkpad =
       gst_pad_new_from_static_template (&gst_kate_parse_sink_factory, "sink");
@@ -166,7 +162,7 @@ gst_kate_parse_push_headers (GstKateParse * parse)
   /* get the headers into the caps, passing them to kate as we go */
   caps =
       gst_kate_util_set_header_on_caps (&parse->element,
-      gst_pad_get_negotiated_caps (parse->sinkpad), parse->streamheader);
+      gst_pad_get_current_caps (parse->sinkpad), parse->streamheader);
 
   if (G_UNLIKELY (!caps)) {
     GST_ELEMENT_ERROR (parse, STREAM, DECODE, (NULL),
@@ -184,14 +180,19 @@ gst_kate_parse_push_headers (GstKateParse * parse)
 
   headers = parse->streamheader;
   while (headers) {
+    guint8 *data;
+    gsize size;
+
     outbuf = GST_BUFFER_CAST (headers->data);
-    kate_packet_wrap (&packet, GST_BUFFER_SIZE (outbuf),
-        GST_BUFFER_DATA (outbuf));
+
+    data = gst_buffer_map (outbuf, &size, NULL, GST_MAP_READ);
+    kate_packet_wrap (&packet, size, data);
     ret = kate_decode_headerin (&parse->ki, &parse->kc, &packet);
     if (G_UNLIKELY (ret < 0)) {
       GST_WARNING_OBJECT (parse, "Failed to decode header: %s",
           gst_kate_util_get_error_message (ret));
     }
+    gst_buffer_unmap (outbuf, data, size);
     /* takes ownership of outbuf, which was previously in parse->streamheader */
     outbuf_list = g_list_append (outbuf_list, outbuf);
     headers = headers->next;
@@ -204,7 +205,6 @@ gst_kate_parse_push_headers (GstKateParse * parse)
   headers = outbuf_list;
   while (headers) {
     outbuf = GST_BUFFER_CAST (headers->data);
-    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
     gst_pad_push (parse->srcpad, outbuf);
     headers = headers->next;
   }
@@ -256,8 +256,6 @@ gst_kate_parse_push_buffer (GstKateParse * parse, GstBuffer * buf,
   GST_BUFFER_OFFSET_END (buf) = granulepos;
   GST_BUFFER_TIMESTAMP (buf) = GST_BUFFER_OFFSET (buf);
 
-  gst_buffer_set_caps (buf, GST_PAD_CAPS (parse->srcpad));
-
   return gst_pad_push (parse->srcpad, buf);
 }
 
@@ -319,7 +317,7 @@ gst_kate_parse_queue_buffer (GstKateParse * parse, GstBuffer * buf)
   GstFlowReturn ret = GST_FLOW_OK;
   gint64 granpos;
 
-  buf = gst_buffer_make_metadata_writable (buf);
+  buf = gst_buffer_make_writable (buf);
 
   /* oggdemux stores the granule pos in the offset end */
   granpos = GST_BUFFER_OFFSET_END (buf);
@@ -348,17 +346,20 @@ static GstFlowReturn
 gst_kate_parse_parse_packet (GstKateParse * parse, GstBuffer * buf)
 {
   GstFlowReturn ret = GST_FLOW_OK;
+  guint8 header[1];
+  gsize size;
 
   g_assert (parse);
 
   parse->packetno++;
 
-  GST_LOG_OBJECT (parse, "Got packet %02x, %u bytes",
-      GST_BUFFER_SIZE (buf) ? GST_BUFFER_DATA (buf)[0] : -1,
-      GST_BUFFER_SIZE (buf));
+  size = gst_buffer_extract (buf, 0, header, 1);
 
-  if (GST_BUFFER_SIZE (buf) > 0 && GST_BUFFER_DATA (buf)[0] & 0x80) {
-    GST_DEBUG_OBJECT (parse, "Found header %02x", GST_BUFFER_DATA (buf)[0]);
+  GST_LOG_OBJECT (parse, "Got packet %02x, %u bytes",
+      size ? header[0] : -1, gst_buffer_get_size (buf));
+
+  if (size > 0 && header[0] & 0x80) {
+    GST_DEBUG_OBJECT (parse, "Found header %02x", header[0]);
     /* if 0x80 is set, it's streamheader,
      * so put it on the streamheader list and return */
     parse->streamheader = g_list_append (parse->streamheader, buf);
@@ -388,7 +389,7 @@ gst_kate_parse_chain (GstPad * pad, GstBuffer * buffer)
 
   g_assert (klass->parse_packet != NULL);
 
-  if (G_UNLIKELY (GST_PAD_CAPS (pad) == NULL))
+  if (G_UNLIKELY (!gst_pad_has_current_caps (pad)))
     return GST_FLOW_NOT_NEGOTIATED;
 
   return klass->parse_packet (parse, buffer);
@@ -603,7 +604,7 @@ gst_kate_parse_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  ret = parent_class->change_state (element, transition);
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
