@@ -32,21 +32,15 @@
 
 enum
 {
-  ARG_0,
-  ARG_LOCATION,
-  ARG_CONNECTION_SPEED
+  PROP_0,
+  PROP_LOCATION,
+  PROP_CONNECTION_SPEED
 };
 
 
 GST_DEBUG_CATEGORY_STATIC (mmssrc_debug);
 #define GST_CAT_DEFAULT mmssrc_debug
 
-static const GstElementDetails plugin_details =
-GST_ELEMENT_DETAILS ("MMS streaming source",
-    "Source/Network",
-    "Receive data streamed via MSFT Multi Media Server protocol",
-    "Maciej Katafiasz <mathrick@users.sourceforge.net>");
-
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -73,60 +67,48 @@ static gboolean gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment);
 
 static GstFlowReturn gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf);
 
-static void
-gst_mms_urihandler_init (GType mms_type)
-{
-  static const GInterfaceInfo urihandler_info = {
-    gst_mms_uri_handler_init,
-    NULL,
-    NULL
-  };
+static gboolean gst_mms_uri_set_uri (GstURIHandler * handler,
+    const gchar * uri, GError ** error);
 
-  g_type_add_interface_static (mms_type, GST_TYPE_URI_HANDLER,
-      &urihandler_info);
-}
-
-GST_BOILERPLATE_FULL (GstMMS, gst_mms, GstPushSrc, GST_TYPE_PUSH_SRC,
-    gst_mms_urihandler_init);
-
-static void
-gst_mms_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_set_details (element_class, &plugin_details);
-
-  GST_DEBUG_CATEGORY_INIT (mmssrc_debug, "mmssrc", 0, "MMS Source Element");
-}
+#define gst_mms_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstMMS, gst_mms, GST_TYPE_PUSH_SRC,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER, gst_mms_uri_handler_init));
 
 /* initialize the plugin's class */
 static void
 gst_mms_class_init (GstMMSClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstBaseSrcClass *gstbasesrc_class;
-  GstPushSrcClass *gstpushsrc_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstbasesrc_class = (GstBaseSrcClass *) klass;
-  gstpushsrc_class = (GstPushSrcClass *) klass;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
+  GstBaseSrcClass *gstbasesrc_class = (GstBaseSrcClass *) klass;
+  GstPushSrcClass *gstpushsrc_class = (GstPushSrcClass *) klass;
 
   gobject_class->set_property = gst_mms_set_property;
   gobject_class->get_property = gst_mms_get_property;
   gobject_class->finalize = gst_mms_finalize;
 
-  g_object_class_install_property (gobject_class, ARG_LOCATION,
+  g_object_class_install_property (gobject_class, PROP_LOCATION,
       g_param_spec_string ("location", "location",
           "Host URL to connect to. Accepted are mms://, mmsu://, mmst:// URL types",
-          NULL, G_PARAM_READWRITE));
+          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, ARG_CONNECTION_SPEED,
-      g_param_spec_uint ("connection-speed", "Connection Speed",
+  /* Note: connection-speed is intentionaly limited to G_MAXINT as libmms
+   * uses int for it */
+  g_object_class_install_property (gobject_class, PROP_CONNECTION_SPEED,
+      g_param_spec_uint64 ("connection-speed", "Connection Speed",
           "Network connection speed in kbps (0 = unknown)",
-          0, G_MAXINT / 1000, DEFAULT_CONNECTION_SPEED, G_PARAM_READWRITE));
-  /* Note: connection-speed is intentionaly limited to G_MAXINT as libmms use int for it */
+          0, G_MAXINT / 1000, DEFAULT_CONNECTION_SPEED,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "MMS streaming source", "Source/Network",
+      "Receive data streamed via MSFT Multi Media Server protocol",
+      "Maciej Katafiasz <mathrick@users.sourceforge.net>");
+
+  GST_DEBUG_CATEGORY_INIT (mmssrc_debug, "mmssrc", 0, "MMS Source Element");
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_mms_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_mms_stop);
@@ -147,7 +129,7 @@ gst_mms_class_init (GstMMSClass * klass)
  * initialize structure
  */
 static void
-gst_mms_init (GstMMS * mmssrc, GstMMSClass * g_class)
+gst_mms_init (GstMMS * mmssrc)
 {
   mmssrc->uri_name = NULL;
   mmssrc->current_connection_uri_name = NULL;
@@ -177,9 +159,7 @@ gst_mms_finalize (GObject * gobject)
     mmssrc->uri_name = NULL;
   }
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    G_OBJECT_CLASS (parent_class)->finalize (gobject);
-
+  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 /* FIXME operating in TIME rather than BYTES could remove this altogether
@@ -269,7 +249,7 @@ gst_mms_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
      of the segment. */
 
   gst_segment_init (segment, seek_format);
-  gst_segment_set_seek (segment, rate, seek_format, flags, cur_type, cur,
+  gst_segment_do_seek (segment, rate, seek_format, flags, cur_type, cur,
       stop_type, stop, NULL);
 
   return TRUE;
@@ -278,7 +258,7 @@ gst_mms_prepare_seek_segment (GstBaseSrc * src, GstEvent * event,
 static gboolean
 gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
 {
-  mms_off_t start;
+  gint64 start;
   GstMMS *mmssrc = GST_MMS (src);
 
   if (segment->format == GST_FORMAT_TIME) {
@@ -288,8 +268,8 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
       return FALSE;
     }
     start = mmsx_get_current_pos (mmssrc->connection);
-    GST_LOG_OBJECT (mmssrc, "sought to %f sec, offset after seek: %lld\n",
-        (double) segment->start / GST_SECOND, start);
+    GST_INFO_OBJECT (mmssrc, "sought to %" GST_TIME_FORMAT ", offset after "
+        "seek: %" G_GINT64_FORMAT, GST_TIME_ARGS (segment->start), start);
   } else if (segment->format == GST_FORMAT_BYTES) {
     start = mmsx_seek (NULL, mmssrc->connection, segment->start, SEEK_SET);
     /* mmsx_seek will close and reopen the connection when seeking with the
@@ -298,16 +278,15 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
       GST_DEBUG_OBJECT (mmssrc, "connection broken during seek");
       return FALSE;
     }
-    GST_DEBUG_OBJECT (mmssrc,
-        "sought to: %" G_GINT64_FORMAT " bytes, result: %lld", segment->start,
-        start);
+    GST_INFO_OBJECT (mmssrc, "sought to: %" G_GINT64_FORMAT " bytes, "
+        "result: %" G_GINT64_FORMAT, segment->start, start);
   } else {
-    GST_DEBUG_OBJECT (mmssrc, "unsupported seek segment format: %d",
-        (int) segment->format);
+    GST_DEBUG_OBJECT (mmssrc, "unsupported seek segment format: %s",
+        GST_STR_NULL (gst_format_get_name (segment->format)));
     return FALSE;
   }
   gst_segment_init (segment, GST_FORMAT_BYTES);
-  gst_segment_set_seek (segment, segment->rate, GST_FORMAT_BYTES,
+  gst_segment_do_seek (segment, segment->rate, GST_FORMAT_BYTES,
       segment->flags, GST_SEEK_TYPE_SET, start, GST_SEEK_TYPE_NONE,
       segment->stop, NULL);
   return TRUE;
@@ -322,7 +301,7 @@ gst_mms_do_seek (GstBaseSrc * src, GstSegment * segment)
 static GstFlowReturn
 gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
-  GstMMS *mmssrc;
+  GstMMS *mmssrc = GST_MMS (psrc);
   guint8 *data;
   guint blocksize;
   gint result;
@@ -330,13 +309,11 @@ gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   *buf = NULL;
 
-  mmssrc = GST_MMS (psrc);
-
   offset = mmsx_get_current_pos (mmssrc->connection);
 
   /* Check if a seek perhaps has wrecked our connection */
   if (offset == -1) {
-    GST_DEBUG_OBJECT (mmssrc,
+    GST_ERROR_OBJECT (mmssrc,
         "connection broken (probably an error during mmsx_seek_time during a convert query) returning FLOW_ERROR");
     return GST_FLOW_ERROR;
   }
@@ -347,33 +324,32 @@ gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
   else
     blocksize = mmsx_get_asf_packet_len (mmssrc->connection);
 
-  *buf = gst_buffer_new_and_alloc (blocksize);
+  data = g_try_malloc (blocksize);
+  if (!data) {
+    GST_ERROR_OBJECT (mmssrc, "Failed to allocate %u bytes", blocksize);
+    return GST_FLOW_ERROR;
+  }
 
-  data = GST_BUFFER_DATA (*buf);
-  GST_BUFFER_SIZE (*buf) = 0;
   GST_LOG_OBJECT (mmssrc, "reading %d bytes", blocksize);
   result = mmsx_read (NULL, mmssrc->connection, (char *) data, blocksize);
-
   /* EOS? */
   if (result == 0)
     goto eos;
 
+  *buf = gst_buffer_new_wrapped (data, result);
   GST_BUFFER_OFFSET (*buf) = offset;
-  GST_BUFFER_SIZE (*buf) = result;
 
   GST_LOG_OBJECT (mmssrc, "Returning buffer with offset %" G_GINT64_FORMAT
-      " and size %u", GST_BUFFER_OFFSET (*buf), GST_BUFFER_SIZE (*buf));
-
-  gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (mmssrc)));
+      " and size %u", offset, result);
 
   return GST_FLOW_OK;
 
 eos:
   {
     GST_DEBUG_OBJECT (mmssrc, "EOS");
-    gst_buffer_unref (*buf);
+    g_free (data);
     *buf = NULL;
-    return GST_FLOW_UNEXPECTED;
+    return GST_FLOW_EOS;
   }
 }
 
@@ -402,11 +378,9 @@ gst_mms_get_size (GstBaseSrc * src, guint64 * size)
 static gboolean
 gst_mms_start (GstBaseSrc * bsrc)
 {
-  GstMMS *mms;
+  GstMMS *mms = GST_MMS (bsrc);
   guint bandwidth_avail;
 
-  mms = GST_MMS (bsrc);
-
   if (!mms->uri_name || *mms->uri_name == '\0')
     goto no_uri;
 
@@ -445,14 +419,23 @@ gst_mms_start (GstBaseSrc * bsrc)
 
     GST_ERROR_OBJECT (mms,
         "Could not connect to this stream, redirecting to rtsp");
-    location = gst_uri_get_location (mms->uri_name);
-    url = g_strdup_printf ("rtsp://%s", location);
-    g_free (location);
+    location = strstr (mms->uri_name, "://");
+    if (location == NULL || *location == '\0' || *(location + 3) == '\0')
+      goto no_uri;
+    url = g_strdup_printf ("rtsp://%s", location + 3);
 
     gst_element_post_message (GST_ELEMENT_CAST (mms),
         gst_message_new_element (GST_OBJECT_CAST (mms),
             gst_structure_new ("redirect", "new-location", G_TYPE_STRING, url,
                 NULL)));
+
+    /* post an error message as well, so that applications that don't handle
+     * redirect messages get to see a proper error message */
+    GST_ELEMENT_ERROR (mms, RESOURCE, OPEN_READ,
+        ("Could not connect to streaming server."),
+        ("A redirect message was posted on the bus and should have been "
+            "handled by the application."));
+
     return FALSE;
   }
 
@@ -467,9 +450,8 @@ no_uri:
 static gboolean
 gst_mms_stop (GstBaseSrc * bsrc)
 {
-  GstMMS *mms;
+  GstMMS *mms = GST_MMS (bsrc);
 
-  mms = GST_MMS (bsrc);
   if (mms->connection != NULL) {
     /* Check if the connection is still pristine, that is if no more then
        just the mmslib cached asf header has been read. If it is still pristine
@@ -490,45 +472,38 @@ static void
 gst_mms_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstMMS *mmssrc;
+  GstMMS *mmssrc = GST_MMS (object);
 
-  mmssrc = GST_MMS (object);
-
-  GST_OBJECT_LOCK (mmssrc);
   switch (prop_id) {
-    case ARG_LOCATION:
-      if (mmssrc->uri_name) {
-        g_free (mmssrc->uri_name);
-        mmssrc->uri_name = NULL;
-      }
-      mmssrc->uri_name = g_value_dup_string (value);
+    case PROP_LOCATION:
+      gst_mms_uri_set_uri (GST_URI_HANDLER (mmssrc),
+          g_value_get_string (value), NULL);
       break;
-    case ARG_CONNECTION_SPEED:
-      mmssrc->connection_speed = g_value_get_uint (value) * 1000;
+    case PROP_CONNECTION_SPEED:
+      GST_OBJECT_LOCK (mmssrc);
+      mmssrc->connection_speed = g_value_get_uint64 (value) * 1000;
+      GST_OBJECT_UNLOCK (mmssrc);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
-  GST_OBJECT_UNLOCK (mmssrc);
 }
 
 static void
 gst_mms_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GstMMS *mmssrc;
-
-  mmssrc = GST_MMS (object);
+  GstMMS *mmssrc = GST_MMS (object);
 
   GST_OBJECT_LOCK (mmssrc);
   switch (prop_id) {
-    case ARG_LOCATION:
+    case PROP_LOCATION:
       if (mmssrc->uri_name)
         g_value_set_string (value, mmssrc->uri_name);
       break;
-    case ARG_CONNECTION_SPEED:
-      g_value_set_uint (value, mmssrc->connection_speed / 1000);
+    case PROP_CONNECTION_SPEED:
+      g_value_set_uint64 (value, mmssrc->connection_speed / 1000);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -549,40 +524,95 @@ plugin_init (GstPlugin * plugin)
 }
 
 static GstURIType
-gst_mms_uri_get_type (void)
+gst_mms_uri_get_type (GType type)
 {
   return GST_URI_SRC;
 }
 
-static gchar **
-gst_mms_uri_get_protocols (void)
+static const gchar *const *
+gst_mms_uri_get_protocols (GType type)
 {
-  static gchar *protocols[] = { "mms", "mmsh", "mmst", "mmsu", NULL };
+  static const gchar *protocols[] = { "mms", "mmsh", "mmst", "mmsu", NULL };
 
   return protocols;
 }
 
-static const gchar *
+static gchar *
 gst_mms_uri_get_uri (GstURIHandler * handler)
 {
   GstMMS *src = GST_MMS (handler);
 
-  return src->uri_name;
+  /* FIXME: make thread-safe */
+  return g_strdup (src->uri_name);
 }
 
-static gboolean
-gst_mms_uri_set_uri (GstURIHandler * handler, const gchar * uri)
+static gchar *
+gst_mms_src_make_valid_uri (const gchar * uri)
 {
   gchar *protocol;
-  GstMMS *src = GST_MMS (handler);
+  const gchar *colon, *tmp;
+  gsize len;
+
+  if (!uri || !gst_uri_is_valid (uri))
+    return NULL;
 
   protocol = gst_uri_get_protocol (uri);
-  if ((strcmp (protocol, "mms") != 0) && (strcmp (protocol, "mmsh") != 0)) {
+
+  if ((strcmp (protocol, "mms") != 0) && (strcmp (protocol, "mmsh") != 0) &&
+      (strcmp (protocol, "mmst") != 0) && (strcmp (protocol, "mmsu") != 0)) {
     g_free (protocol);
     return FALSE;
   }
   g_free (protocol);
-  g_object_set (src, "location", uri, NULL);
+
+  colon = strstr (uri, "://");
+  if (!colon)
+    return NULL;
+
+  tmp = colon + 3;
+  len = strlen (tmp);
+  if (len == 0)
+    return NULL;
+
+  /* libmms segfaults if there's no hostname or
+   * no / after the hostname
+   */
+  colon = strstr (tmp, "/");
+  if (colon == tmp)
+    return NULL;
+
+  if (strstr (tmp, "/") == NULL) {
+    gchar *ret;
+
+    len = strlen (uri);
+    ret = g_malloc0 (len + 2);
+    memcpy (ret, uri, len);
+    ret[len] = '/';
+    return ret;
+  } else {
+    return g_strdup (uri);
+  }
+}
+
+static gboolean
+gst_mms_uri_set_uri (GstURIHandler * handler, const gchar * uri,
+    GError ** error)
+{
+  GstMMS *src = GST_MMS (handler);
+  gchar *fixed_uri;
+
+  fixed_uri = gst_mms_src_make_valid_uri (uri);
+  if (!fixed_uri && uri) {
+    g_set_error (error, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
+        "Invalid MMS URI");
+    return FALSE;
+  }
+
+  GST_OBJECT_LOCK (src);
+  if (src->uri_name)
+    g_free (src->uri_name);
+  src->uri_name = fixed_uri;
+  GST_OBJECT_UNLOCK (src);
 
   return TRUE;
 }
@@ -603,6 +633,6 @@ gst_mms_uri_handler_init (gpointer g_iface, gpointer iface_data)
  * so keep the name plugin_desc, or you cannot get your plug-in registered */
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "mms",
+    mms,
     "Microsoft Multi Media Server streaming protocol support",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
