@@ -43,6 +43,17 @@
 #include "config.h"
 #endif
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
+/* Required to not get an undefined warning
+ * https://bugzilla.gnome.org/show_bug.cgi?id=613795
+ */
+#ifndef WORDS_BIGENDIAN
+#define WORDS_BIGENDIAN 0
+#endif
+
 #include <stdafx.h>
 #include <libmodplug/sndfile.h>
 
@@ -84,23 +95,16 @@ enum
 #define DEFAULT_OVERSAMP         TRUE
 #define DEFAULT_NOISE_REDUCTION  TRUE
 
+#define FORMATS "{ "GST_AUDIO_NE (S32)", "GST_AUDIO_NE (S16)", U8 }"
+
 static GstStaticPadTemplate modplug_src_template_factory =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int,"
-        " endianness = (int) BYTE_ORDER,"
-        " signed = (boolean) true,"
-        " width = (int) 16,"
-        " depth = (int) 16,"
+    GST_STATIC_CAPS ("audio/x-raw,"
+        " format = (string) " FORMATS ", "
+        " layout = (string) interleaved, "
         " rate = (int) { 8000, 11025, 22050, 44100 },"
-        " channels = (int) 2; "
-        "audio/x-raw-int,"
-        " endianness = (int) BYTE_ORDER,"
-        " signed = (boolean) false,"
-        " width = (int) 8,"
-        " depth = (int) 8,"
-        " rate = (int) { 8000, 11025, 22050, 44100 }, "
         " channels = (int) [ 1, 2 ]"));
 
 static GstStaticPadTemplate modplug_sink_template_factory =
@@ -116,36 +120,20 @@ static void gst_modplug_set_property (GObject * object,
 static void gst_modplug_get_property (GObject * object,
     guint id, GValue * value, GParamSpec * pspec);
 
-static void gst_modplug_fixate (GstPad * pad, GstCaps * caps);
-static const GstQueryType *gst_modplug_get_query_types (GstPad * pad);
-static gboolean gst_modplug_src_event (GstPad * pad, GstEvent * event);
-static gboolean gst_modplug_src_query (GstPad * pad, GstQuery * query);
+static gboolean gst_modplug_src_event (GstPad * pad, GstObject * parent,
+    GstEvent * event);
+static gboolean gst_modplug_src_query (GstPad * pad, GstObject * parent,
+    GstQuery * query);
 static GstStateChangeReturn gst_modplug_change_state (GstElement * element,
     GstStateChange transition);
 
-static gboolean gst_modplug_sinkpad_activate (GstPad * pad);
-static gboolean gst_modplug_sinkpad_activate_pull (GstPad * pad,
-    gboolean active);
+static gboolean gst_modplug_sinkpad_activate (GstPad * pad, GstObject * parent);
+static gboolean gst_modplug_sinkpad_activate_mode (GstPad * pad,
+    GstObject * parent, GstPadMode mode, gboolean active);
 static void gst_modplug_loop (GstModPlug * element);
 
-GST_BOILERPLATE (GstModPlug, gst_modplug, GstElement, GST_TYPE_ELEMENT);
-
-static void
-gst_modplug_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&modplug_sink_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&modplug_src_template_factory));
-
-  gst_element_class_set_details_simple (element_class, "ModPlug",
-      "Codec/Decoder/Audio", "Module decoder based on modplug engine",
-      "Jeremy SIMON <jsimon13@yahoo.fr>");
-
-  GST_DEBUG_CATEGORY_INIT (modplug_debug, "modplug", 0, "ModPlug element");
-}
+#define parent_class gst_modplug_parent_class
+G_DEFINE_TYPE (GstModPlug, gst_modplug, GST_TYPE_ELEMENT);
 
 static void
 gst_modplug_class_init (GstModPlugClass * klass)
@@ -162,78 +150,95 @@ gst_modplug_class_init (GstModPlugClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SONGNAME,
       g_param_spec_string ("songname", "Songname", "The song name",
-          NULL, G_PARAM_READABLE));
+          NULL, (GParamFlags) (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB,
       g_param_spec_boolean ("reverb", "reverb", "Reverb",
-          DEFAULT_REVERB, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_REVERB,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB_DEPTH,
       g_param_spec_int ("reverb-depth", "reverb depth", "Reverb depth",
-          0, 100, DEFAULT_REVERB_DEPTH, (GParamFlags) G_PARAM_READWRITE));
+          0, 100, DEFAULT_REVERB_DEPTH,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_REVERB_DELAY,
       g_param_spec_int ("reverb-delay", "reverb delay", "Reverb delay",
-          0, 200, DEFAULT_REVERB_DELAY, (GParamFlags) G_PARAM_READWRITE));
+          0, 200, DEFAULT_REVERB_DELAY,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS,
       g_param_spec_boolean ("megabass", "megabass", "Megabass",
-          DEFAULT_MEGABASS, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_MEGABASS,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS_AMOUNT,
       g_param_spec_int ("megabass-amount", "megabass amount", "Megabass amount",
-          0, 100, DEFAULT_MEGABASS_AMOUNT, (GParamFlags) G_PARAM_READWRITE));
+          0, 100, DEFAULT_MEGABASS_AMOUNT,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MEGABASS_RANGE,
       g_param_spec_int ("megabass-range", "megabass range", "Megabass range",
-          0, 100, DEFAULT_MEGABASS_RANGE, (GParamFlags) G_PARAM_READWRITE));
+          0, 100, DEFAULT_MEGABASS_RANGE,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND,
       g_param_spec_boolean ("surround", "surround", "Surround",
-          DEFAULT_SURROUND, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_SURROUND,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND_DEPTH,
       g_param_spec_int ("surround-depth", "surround depth", "Surround depth",
-          0, 100, DEFAULT_SURROUND_DEPTH, (GParamFlags) G_PARAM_READWRITE));
+          0, 100, DEFAULT_SURROUND_DEPTH,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SURROUND_DELAY,
       g_param_spec_int ("surround-delay", "surround delay", "Surround delay",
-          0, 40, DEFAULT_SURROUND_DELAY, (GParamFlags) G_PARAM_READWRITE));
+          0, 40, DEFAULT_SURROUND_DELAY,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_OVERSAMP,
       g_param_spec_boolean ("oversamp", "oversamp", "oversamp",
-          DEFAULT_OVERSAMP, (GParamFlags) G_PARAM_READWRITE));
+          DEFAULT_OVERSAMP,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NOISE_REDUCTION,
       g_param_spec_boolean ("noise-reduction", "noise reduction",
           "noise reduction", DEFAULT_NOISE_REDUCTION,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   gstelement_class->change_state = gst_modplug_change_state;
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&modplug_sink_template_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&modplug_src_template_factory));
+
+  gst_element_class_set_details_simple (gstelement_class, "ModPlug",
+      "Codec/Decoder/Audio", "Module decoder based on modplug engine",
+      "Jeremy SIMON <jsimon13@yahoo.fr>");
+
+  GST_DEBUG_CATEGORY_INIT (modplug_debug, "modplug", 0, "ModPlug element");
 }
 
 static void
-gst_modplug_init (GstModPlug * modplug, GstModPlugClass * klass)
+gst_modplug_init (GstModPlug * modplug)
 {
   /* create the sink and src pads */
   modplug->sinkpad =
       gst_pad_new_from_static_template (&modplug_sink_template_factory, "sink");
   gst_pad_set_activate_function (modplug->sinkpad,
       GST_DEBUG_FUNCPTR (gst_modplug_sinkpad_activate));
-  gst_pad_set_activatepull_function (modplug->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_modplug_sinkpad_activate_pull));
+  gst_pad_set_activatemode_function (modplug->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_modplug_sinkpad_activate_mode));
   gst_element_add_pad (GST_ELEMENT (modplug), modplug->sinkpad);
 
   modplug->srcpad =
       gst_pad_new_from_static_template (&modplug_src_template_factory, "src");
-  gst_pad_set_fixatecaps_function (modplug->srcpad,
-      GST_DEBUG_FUNCPTR (gst_modplug_fixate));
   gst_pad_set_event_function (modplug->srcpad,
       GST_DEBUG_FUNCPTR (gst_modplug_src_event));
   gst_pad_set_query_function (modplug->srcpad,
       GST_DEBUG_FUNCPTR (gst_modplug_src_query));
-  gst_pad_set_query_type_function (modplug->srcpad,
-      GST_DEBUG_FUNCPTR (gst_modplug_get_query_types));
   gst_element_add_pad (GST_ELEMENT (modplug), modplug->srcpad);
 
   modplug->reverb = DEFAULT_REVERB;
@@ -248,7 +253,7 @@ gst_modplug_init (GstModPlug * modplug, GstModPlugClass * klass)
   modplug->oversamp = DEFAULT_OVERSAMP;
   modplug->noise_reduction = DEFAULT_NOISE_REDUCTION;
 
-  modplug->_16bit = TRUE;
+  modplug->bits = 16;
   modplug->channel = 2;
   modplug->frequency = 44100;
 }
@@ -267,35 +272,22 @@ gst_modplug_dispose (GObject * object)
   }
 }
 
-static const GstQueryType *
-gst_modplug_get_query_types (GstPad * pad)
-{
-  static const GstQueryType gst_modplug_src_query_types[] = {
-    GST_QUERY_DURATION,
-    GST_QUERY_POSITION,
-    (GstQueryType) 0
-  };
-
-  return gst_modplug_src_query_types;
-}
-
-
 static gboolean
-gst_modplug_src_query (GstPad * pad, GstQuery * query)
+gst_modplug_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
   GstModPlug *modplug;
   gboolean res = FALSE;
 
-  modplug = GST_MODPLUG (gst_pad_get_parent (pad));
-
-  if (!modplug->mSoundFile)
-    goto done;
+  modplug = GST_MODPLUG (parent);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
     {
       GstFormat format;
 
+      if (!modplug->mSoundFile)
+        goto done;
+
       gst_query_parse_duration (query, &format, NULL);
       if (format == GST_FORMAT_TIME) {
         gst_query_set_duration (query, format, modplug->song_length);
@@ -307,6 +299,9 @@ gst_modplug_src_query (GstPad * pad, GstQuery * query)
     {
       GstFormat format;
 
+      if (!modplug->mSoundFile)
+        goto done;
+
       gst_query_parse_position (query, &format, NULL);
       if (format == GST_FORMAT_TIME) {
         gint64 pos;
@@ -319,172 +314,201 @@ gst_modplug_src_query (GstPad * pad, GstQuery * query)
     }
       break;
     default:
-      res = gst_pad_query_default (pad, query);
+      res = gst_pad_query_default (pad, parent, query);
       break;
   }
 
 done:
-  gst_object_unref (modplug);
-
   return res;
 }
 
 static gboolean
-gst_modplug_src_event (GstPad * pad, GstEvent * event)
+gst_modplug_do_seek (GstModPlug * modplug, GstEvent * event)
+{
+  gdouble rate;
+  GstFormat format;
+  GstSeekFlags flags;
+  GstSeekType cur_type, stop_type;
+  gboolean flush;
+  gint64 cur, stop;
+  GstSegment seg;
+/* FIXME timestamp is set but not used */
+#if 0
+  guint64 timestamp;
+#endif
+
+  if (modplug->frequency == 0)
+    goto no_song;
+
+#if 0
+  timestamp = gst_util_uint64_scale_int (modplug->offset, GST_SECOND,
+      modplug->frequency);
+#endif
+
+  gst_event_parse_seek (event, &rate, &format, &flags,
+      &cur_type, &cur, &stop_type, &stop);
+
+  if (format != GST_FORMAT_TIME)
+    goto no_time;
+
+  /* FIXME: we should be using GstSegment for all this */
+  if (cur_type != GST_SEEK_TYPE_SET || stop_type != GST_SEEK_TYPE_NONE)
+    goto not_supported;
+
+  if (stop_type == GST_SEEK_TYPE_NONE)
+    stop = GST_CLOCK_TIME_NONE;
+
+  cur = CLAMP (cur, 0, modplug->song_length);
+
+  GST_DEBUG_OBJECT (modplug, "seek to %" GST_TIME_FORMAT,
+      GST_TIME_ARGS ((guint64) cur));
+
+  modplug->seek_at = cur;
+
+  flush = ((flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
+
+  if (flush) {
+    gst_pad_push_event (modplug->srcpad, gst_event_new_flush_start ());
+  } else {
+    gst_pad_stop_task (modplug->sinkpad);
+  }
+
+  GST_PAD_STREAM_LOCK (modplug->sinkpad);
+
+  if (flags & GST_SEEK_FLAG_SEGMENT) {
+    gst_element_post_message (GST_ELEMENT (modplug),
+        gst_message_new_segment_start (GST_OBJECT (modplug), format, cur));
+  }
+  if (stop == -1 && modplug->song_length > 0)
+    stop = modplug->song_length;
+
+  if (flush) {
+    gst_pad_push_event (modplug->srcpad, gst_event_new_flush_stop (TRUE));
+  }
+
+  GST_LOG_OBJECT (modplug, "sending newsegment from %" GST_TIME_FORMAT "-%"
+      GST_TIME_FORMAT ", pos=%" GST_TIME_FORMAT,
+      GST_TIME_ARGS ((guint64) cur), GST_TIME_ARGS ((guint64) stop),
+      GST_TIME_ARGS ((guint64) cur));
+
+  gst_segment_init (&seg, GST_FORMAT_TIME);
+  seg.rate = rate;
+  seg.start = cur;
+  seg.stop = stop;
+  seg.time = cur;
+  gst_pad_push_event (modplug->srcpad, gst_event_new_segment (&seg));
+
+  modplug->offset =
+      gst_util_uint64_scale_int (cur, modplug->frequency, GST_SECOND);
+
+  gst_pad_start_task (modplug->sinkpad,
+      (GstTaskFunction) gst_modplug_loop, modplug);
+
+  GST_PAD_STREAM_UNLOCK (modplug->sinkpad);
+
+  return TRUE;
+
+  /* ERROR */
+no_song:
+  {
+    GST_DEBUG_OBJECT (modplug, "no song loaded yet");
+    return FALSE;
+  }
+no_time:
+  {
+    GST_DEBUG_OBJECT (modplug, "seeking is only supported in TIME format");
+    return FALSE;
+  }
+not_supported:
+  {
+    GST_DEBUG_OBJECT (modplug, "unsupported seek type");
+    return FALSE;
+  }
+}
+
+static gboolean
+gst_modplug_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstModPlug *modplug;
   gboolean res = FALSE;
 
-  modplug = GST_MODPLUG (gst_pad_get_parent (pad));
+  modplug = GST_MODPLUG (parent);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-    {
-      gdouble rate;
-      GstFormat format;
-      GstSeekFlags flags;
-      GstSeekType cur_type, stop_type;
-      gboolean flush;
-      gint64 cur, stop;
-      guint64 timestamp;
-
-      if (modplug->frequency == 0) {
-        GST_DEBUG_OBJECT (modplug, "no song loaded yet");
-        break;
-      }
-
-      timestamp = gst_util_uint64_scale_int (modplug->offset, GST_SECOND,
-          modplug->frequency);
-
-      gst_event_parse_seek (event, &rate, &format, &flags,
-          &cur_type, &cur, &stop_type, &stop);
-
-      if (format != GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (modplug, "seeking is only supported in TIME format");
-        gst_event_unref (event);
-        break;
-      }
-
-      /* FIXME: we should be using GstSegment for all this */
-      if (cur_type != GST_SEEK_TYPE_SET || stop_type != GST_SEEK_TYPE_NONE) {
-        GST_DEBUG_OBJECT (modplug, "unsupported seek type");
-        gst_event_unref (event);
-        break;
-      }
-
-      if (stop_type == GST_SEEK_TYPE_NONE)
-        stop = GST_CLOCK_TIME_NONE;
-
-      cur = CLAMP (cur, 0, modplug->song_length);
-
-      GST_DEBUG_OBJECT (modplug, "seek to %" GST_TIME_FORMAT,
-          GST_TIME_ARGS ((guint64) cur));
-
-      modplug->seek_at = cur;
-
-      flush = ((flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
-
-      if (flush) {
-        gst_pad_push_event (modplug->srcpad, gst_event_new_flush_start ());
-      } else {
-        gst_pad_stop_task (modplug->sinkpad);
-      }
-
-      GST_PAD_STREAM_LOCK (modplug->sinkpad);
-
-      if (flags & GST_SEEK_FLAG_SEGMENT) {
-        gst_element_post_message (GST_ELEMENT (modplug),
-            gst_message_new_segment_start (GST_OBJECT (modplug), format, cur));
-      }
-      if (stop == -1 && modplug->song_length > 0)
-        stop = modplug->song_length;
-
-      if (flush) {
-        gst_pad_push_event (modplug->srcpad, gst_event_new_flush_stop ());
-      }
-
-      GST_LOG_OBJECT (modplug, "sending newsegment from %" GST_TIME_FORMAT "-%"
-          GST_TIME_FORMAT ", pos=%" GST_TIME_FORMAT,
-          GST_TIME_ARGS ((guint64) cur), GST_TIME_ARGS ((guint64) stop),
-          GST_TIME_ARGS ((guint64) cur));
-
-      gst_pad_push_event (modplug->srcpad,
-          gst_event_new_new_segment (FALSE, rate,
-              GST_FORMAT_TIME, cur, stop, cur));
-
-      modplug->offset =
-          gst_util_uint64_scale_int (cur, modplug->frequency, GST_SECOND);
-
-      gst_pad_start_task (modplug->sinkpad,
-          (GstTaskFunction) gst_modplug_loop, modplug);
-
-      GST_PAD_STREAM_UNLOCK (modplug->sinkpad);
-      res = TRUE;
+      res = gst_modplug_do_seek (modplug, event);
       break;
-    }
     default:
-      res = gst_pad_event_default (pad, event);
+      res = gst_pad_event_default (pad, parent, event);
       break;
   }
-
-  gst_object_unref (modplug);
   return res;
 }
 
-static void
-gst_modplug_fixate (GstPad * pad, GstCaps * caps)
-{
-  GstStructure *structure;
-
-  structure = gst_caps_get_structure (caps, 0);
-  if (gst_structure_fixate_field_nearest_int (structure, "rate", 44100))
-    return;
-  if (gst_structure_fixate_field_nearest_int (structure, "channels", 2))
-    return;
-}
-
 static gboolean
 gst_modplug_load_song (GstModPlug * modplug)
 {
-  GstCaps *newcaps, *othercaps;
+  GstCaps *newcaps;
   GstStructure *structure;
-  gint depth;
+  GstMapInfo map;
+  const gchar * format;
 
   GST_DEBUG_OBJECT (modplug, "Setting caps");
 
   /* negotiate srcpad caps */
-  if ((othercaps = gst_pad_get_allowed_caps (modplug->srcpad))) {
-    newcaps = gst_caps_copy_nth (othercaps, 0);
-    gst_caps_unref (othercaps);
-  } else {
-    GST_WARNING ("no allowed caps on srcpad, no peer linked");
-    /* FIXME: this can be done in a better way */
-    newcaps =
-        gst_caps_copy_nth (gst_pad_get_pad_template_caps (modplug->srcpad), 0);
+  if ((newcaps = gst_pad_get_allowed_caps (modplug->srcpad)) == NULL) {
+    newcaps = gst_pad_get_pad_template_caps (modplug->srcpad);
   }
-  gst_pad_fixate_caps (modplug->srcpad, newcaps);
-  gst_pad_set_caps (modplug->srcpad, newcaps);
+  newcaps = gst_caps_make_writable (newcaps);
+
+  GST_DEBUG_OBJECT (modplug, "allowed caps %"GST_PTR_FORMAT, newcaps);
+
+  structure = gst_caps_get_structure (newcaps, 0);
+
+  if (!gst_structure_fixate_field_string (structure, "format", GST_AUDIO_NE (S16)))
+    GST_WARNING_OBJECT (modplug, "Failed to fixate format to S16NE");
+  if (!gst_structure_fixate_field_nearest_int (structure, "rate", 44100))
+    GST_WARNING_OBJECT (modplug, "Failed to fixate rate to 44100");
+  if (!gst_structure_fixate_field_nearest_int (structure, "channels", 2))
+    GST_WARNING_OBJECT (modplug, "Failed to fixate number of channels to stereo");
+
+  GST_DEBUG_OBJECT (modplug, "normalized caps %"GST_PTR_FORMAT, newcaps);
+
+  newcaps = gst_caps_fixate (newcaps);
+
+  GST_DEBUG_OBJECT (modplug, "fixated caps %"GST_PTR_FORMAT, newcaps);
 
   /* set up modplug to output the negotiated format */
   structure = gst_caps_get_structure (newcaps, 0);
-  gst_structure_get_int (structure, "depth", &depth);
-  modplug->_16bit = (depth == 16);
+  format = gst_structure_get_string (structure, "format");
+
+  if (g_str_equal (format, GST_AUDIO_NE (S32)))
+    modplug->bits = 32;
+  else if (g_str_equal (format, GST_AUDIO_NE (S16)))
+    modplug->bits = 16;
+  else
+    modplug->bits = 8;
+
   gst_structure_get_int (structure, "channels", &modplug->channel);
   gst_structure_get_int (structure, "rate", &modplug->frequency);
+  
+  GST_DEBUG_OBJECT (modplug, 
+      "Audio settings: %d bits, %d channel(s), %d Hz sampling rate",
+      modplug->bits, modplug->channel, modplug->frequency);
+
+  gst_pad_set_caps (modplug->srcpad, newcaps);
+  gst_caps_unref (newcaps);
 
   modplug->read_samples = 1152;
-  modplug->read_bytes = modplug->read_samples * modplug->channel * depth / 8;
+  modplug->read_bytes =
+      modplug->read_samples * modplug->channel * modplug->bits / 8;
 
   GST_DEBUG_OBJECT (modplug, "Loading song");
 
   modplug->mSoundFile = new CSoundFile;
 
-  if (modplug->_16bit)
-    modplug->mSoundFile->SetWaveConfig (modplug->frequency, 16,
-        modplug->channel);
-  else
-    modplug->mSoundFile->SetWaveConfig (modplug->frequency, 8,
-        modplug->channel);
+  modplug->mSoundFile->SetWaveConfig (modplug->frequency, modplug->bits,
+      modplug->channel);
 
   modplug->mSoundFile->SetWaveConfigEx (modplug->surround, !modplug->oversamp,
       modplug->reverb, true, modplug->megabass, modplug->noise_reduction, true);
@@ -502,12 +526,11 @@ gst_modplug_load_song (GstModPlug * modplug)
     modplug->mSoundFile->SetReverbParameters (modplug->reverb_depth,
         modplug->reverb_delay);
 
-  if (!modplug->mSoundFile->Create (GST_BUFFER_DATA (modplug->buffer),
-          modplug->song_size)) {
-    GST_ELEMENT_ERROR (modplug, STREAM, DECODE, (NULL),
-        ("Unable to load song"));
-    return FALSE;
-  }
+
+  gst_buffer_map (modplug->buffer, &map, GST_MAP_READ);
+  if (!modplug->mSoundFile->Create (map.data, modplug->song_size))
+    goto load_error;
+  gst_buffer_unmap (modplug->buffer, &map);
 
   modplug->song_length = modplug->mSoundFile->GetSongTime () * GST_SECOND;
   modplug->seek_at = -1;
@@ -516,34 +539,75 @@ gst_modplug_load_song (GstModPlug * modplug)
       GST_TIME_ARGS ((guint64) modplug->song_length));
 
   return TRUE;
-}
 
-static gboolean
-gst_modplug_sinkpad_activate (GstPad * pad)
-{
-  if (!gst_pad_check_pull_range (pad))
+  /* ERRORS */
+load_error:
+  {
+    gst_buffer_unmap (modplug->buffer, &map);
+    GST_ELEMENT_ERROR (modplug, STREAM, DECODE, (NULL),
+        ("Unable to load song"));
     return FALSE;
-
-  return gst_pad_activate_pull (pad, TRUE);
+  }
 }
 
 static gboolean
-gst_modplug_sinkpad_activate_pull (GstPad * pad, gboolean active)
+gst_modplug_sinkpad_activate (GstPad * sinkpad, GstObject * parent)
 {
-  GstModPlug *modplug = GST_MODPLUG (GST_OBJECT_PARENT (pad));
+  GstQuery *query;
+  gboolean pull_mode;
 
-  if (active) {
-    return gst_pad_start_task (pad, (GstTaskFunction) gst_modplug_loop,
-        modplug);
-  } else {
-    return gst_pad_stop_task (pad);
+  query = gst_query_new_scheduling ();
+
+  if (!gst_pad_peer_query (sinkpad, query)) {
+    gst_query_unref (query);
+    goto activate_push;
   }
+
+  pull_mode = gst_query_has_scheduling_mode (query, GST_PAD_MODE_PULL);
+  gst_query_unref (query);
+
+  if (!pull_mode)
+    goto activate_push;
+
+  GST_DEBUG_OBJECT (sinkpad, "activating pull");
+  return gst_pad_activate_mode (sinkpad, GST_PAD_MODE_PULL, TRUE);
+
+activate_push:
+  {
+    GST_DEBUG_OBJECT (sinkpad, "activating push");
+    return gst_pad_activate_mode (sinkpad, GST_PAD_MODE_PUSH, TRUE);
+  }
+}
+
+static gboolean
+gst_modplug_sinkpad_activate_mode (GstPad * pad, GstObject * parent,
+    GstPadMode mode, gboolean active)
+{
+  GstModPlug *modplug = GST_MODPLUG (parent);
+  gboolean res;
+
+  switch (mode) {
+    case GST_PAD_MODE_PUSH:
+      res = TRUE;
+      break;
+    case GST_PAD_MODE_PULL:
+      if (active) {
+        res = gst_pad_start_task (pad, (GstTaskFunction) gst_modplug_loop,
+            modplug);
+      } else {
+        res = gst_pad_stop_task (pad);
+      }
+      break;
+    default:
+      res = FALSE;
+      break;
+  }
+  return res;
 }
 
 static gboolean
 gst_modplug_get_upstream_size (GstModPlug * modplug, gint64 * length)
 {
-  GstFormat format = GST_FORMAT_BYTES;
   gboolean res = FALSE;
   GstPad *peer;
 
@@ -551,7 +615,7 @@ gst_modplug_get_upstream_size (GstModPlug * modplug, gint64 * length)
   if (peer == NULL)
     return FALSE;
 
-  if (gst_pad_query_duration (peer, &format, length) && *length >= 0) {
+  if (gst_pad_query_duration (peer, GST_FORMAT_BYTES, length) && *length >= 0) {
     res = TRUE;
   }
 
@@ -564,6 +628,7 @@ gst_modplug_loop (GstModPlug * modplug)
 {
   GstFlowReturn flow;
   GstBuffer *out = NULL;
+  GstMapInfo map;
 
   g_assert (GST_IS_MODPLUG (modplug));
 
@@ -600,18 +665,20 @@ gst_modplug_loop (GstModPlug * modplug)
     }
 
     /* GST_LOG_OBJECT (modplug, "Read %u bytes", GST_BUFFER_SIZE (buffer)); */
-    g_memmove (GST_BUFFER_DATA (modplug->buffer) + modplug->offset,
-        GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+    gst_buffer_map (buffer, &map, GST_MAP_READ);
+    gst_buffer_fill (modplug->buffer, modplug->offset, map.data, map.size);
+    gst_buffer_unmap (buffer, &map);
     gst_buffer_unref (buffer);
 
     modplug->offset += read_size;
 
     /* actually load it */
     if (modplug->offset == modplug->song_size) {
-      GstEvent *newsegment;
       GstTagList *tags;
       gboolean ok;
-      gchar comment[16384];
+      #define COMMENT_SIZE 16384
+      gchar comment[COMMENT_SIZE];
+      GstSegment seg;
 
       ok = gst_modplug_load_song (modplug);
       gst_buffer_unref (modplug->buffer);
@@ -622,24 +689,24 @@ gst_modplug_loop (GstModPlug * modplug)
         goto pause;
       }
 
-      newsegment = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
-          0, modplug->song_length, 0);
-      gst_pad_push_event (modplug->srcpad, newsegment);
+      gst_segment_init (&seg, GST_FORMAT_TIME);
+      seg.stop = modplug->song_length;
+      gst_pad_push_event (modplug->srcpad, gst_event_new_segment (&seg));
 
       /* get and send metadata */
-      tags = gst_tag_list_new ();
+      tags = gst_tag_list_new_empty ();
       gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
           GST_TAG_TITLE, modplug->mSoundFile->GetTitle (),
           GST_TAG_BEATS_PER_MINUTE,
           (gdouble) modplug->mSoundFile->GetMusicTempo (), NULL);
 
-      if (modplug->mSoundFile->GetSongComments ((gchar *) & comment, 16384, 32)) {
+      if (modplug->mSoundFile->GetSongComments ((gchar *) & comment,
+          COMMENT_SIZE, 32)) {
+        comment[COMMENT_SIZE - 1] = '\0';
         gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
             GST_TAG_COMMENT, comment, NULL);
       }
-
-
-      gst_element_found_tags (GST_ELEMENT (modplug), tags);
+      gst_pad_push_event (modplug->srcpad, gst_event_new_tag (tags));
     } else {
       /* not fully loaded yet */
       return;
@@ -653,7 +720,7 @@ gst_modplug_loop (GstModPlug * modplug)
     gfloat temp;
 
     temp = (gfloat) modplug->song_length / modplug->seek_at;
-    seek_to_pos = (int) (modplug->mSoundFile->GetMaxPosition () / temp);
+    seek_to_pos = (gint) (modplug->mSoundFile->GetMaxPosition () / temp);
 
     GST_DEBUG_OBJECT (modplug, "Seeking to row %d", seek_to_pos);
 
@@ -662,19 +729,18 @@ gst_modplug_loop (GstModPlug * modplug)
   }
 
   /* read and output a buffer */
-  flow = gst_pad_alloc_buffer_and_set_caps (modplug->srcpad,
-      GST_BUFFER_OFFSET_NONE, modplug->read_bytes,
-      GST_PAD_CAPS (modplug->srcpad), &out);
+  GST_LOG_OBJECT (modplug, "Read %d bytes", (gint)modplug->read_bytes);
+  /* libmodplug 0.8.7 trashes memory */
+  out = gst_buffer_new_allocate (NULL, modplug->read_bytes * 2, NULL);
 
-  if (flow != GST_FLOW_OK) {
-    GST_LOG_OBJECT (modplug, "pad alloc flow: %s", gst_flow_get_name (flow));
-    goto pause;
-  }
-
-  if (!modplug->mSoundFile->Read (GST_BUFFER_DATA (out), modplug->read_bytes))
+  gst_buffer_map (out, &map, GST_MAP_WRITE);
+  if (!modplug->mSoundFile->Read (map.data, modplug->read_bytes)) {
+    gst_buffer_unmap (out, &map);
     goto eos;
+  }
+  gst_buffer_unmap (out, &map);
+  gst_buffer_resize (out, 0, modplug->read_bytes);
 
-  GST_BUFFER_SIZE (out) = modplug->read_bytes;
   GST_BUFFER_DURATION (out) =
       gst_util_uint64_scale_int (modplug->read_samples, GST_SECOND,
       modplug->frequency);
@@ -740,6 +806,7 @@ gst_modplug_change_state (GstElement * element, GstStateChange transition)
       }
       if (modplug->mSoundFile) {
         modplug->mSoundFile->Destroy ();
+        delete modplug->mSoundFile;
         modplug->mSoundFile = NULL;
       }
       break;
@@ -850,6 +917,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "modplug",
+    modplug,
     ".MOD audio decoding",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
