@@ -121,12 +121,13 @@ gst_nas_sink_class_init (GstNasSinkClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_MUTE,
       g_param_spec_boolean ("mute", "mute", "Whether to mute playback",
-          DEFAULT_MUTE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          DEFAULT_MUTE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, ARG_HOST,
       g_param_spec_string ("host", "host",
           "host running the NAS daemon (name of X/Terminal, default is "
           "$AUDIOSERVER or $DISPLAY)", DEFAULT_HOST,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_nas_sink_getcaps);
 
@@ -631,6 +632,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "nas",
+    nas,
     "NAS (Network Audio System) support for GStreamer",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
