@@ -48,6 +48,7 @@
 
 #include <gst/gsttagsetter.h>
 #include <gst/audio/audio.h>
+#include <gst/glib-compat-private.h>
 #include "gstopusheader.h"
 #include "gstopuscommon.h"
 #include "gstopusenc.h"
@@ -113,14 +114,15 @@ gst_opus_enc_frame_size_get_type (void)
   return id;
 }
 
+#define FORMAT_STR GST_AUDIO_NE(S16)
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int, "
-        "rate = (int) { 48000, 24000, 16000, 12000, 8000 }, "
-        "channels = (int) [ 1, 8 ], "
-        "endianness = (int) BYTE_ORDER, "
-        "signed = (boolean) TRUE, " "width = (int) 16, " "depth = (int) 16")
+    GST_STATIC_CAPS ("audio/x-raw, "
+        "format = (string) " FORMAT_STR ", "
+        "layout = (string) interleaved, "
+        "rate = (int) { 8000, 12000, 16000, 24000, 48000 }, "
+        "channels = (int) [ 1, 2 ] ")
     );
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
@@ -161,7 +163,8 @@ static void gst_opus_enc_finalize (GObject * object);
 
 static gboolean gst_opus_enc_sink_event (GstAudioEncoder * benc,
     GstEvent * event);
-static GstCaps *gst_opus_enc_sink_getcaps (GstAudioEncoder * benc);
+static GstCaps *gst_opus_enc_sink_getcaps (GstAudioEncoder * benc,
+    GstCaps * filter);
 static gboolean gst_opus_enc_setup (GstOpusEnc * enc);
 
 static void gst_opus_enc_get_property (GObject * object, guint prop_id,
@@ -179,57 +182,39 @@ static gint64 gst_opus_enc_get_latency (GstOpusEnc * enc);
 
 static GstFlowReturn gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buffer);
 
-static void
-gst_opus_enc_setup_interfaces (GType opusenc_type)
-{
-  static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
-  const GInterfaceInfo preset_interface_info = {
-    NULL,                       /* interface_init */
-    NULL,                       /* interface_finalize */
-    NULL                        /* interface_data */
-  };
-
-  g_type_add_interface_static (opusenc_type, GST_TYPE_TAG_SETTER,
-      &tag_setter_info);
-  g_type_add_interface_static (opusenc_type, GST_TYPE_PRESET,
-      &preset_interface_info);
-
-  GST_DEBUG_CATEGORY_INIT (opusenc_debug, "opusenc", 0, "Opus encoder");
-}
-
-GST_BOILERPLATE_FULL (GstOpusEnc, gst_opus_enc, GstAudioEncoder,
-    GST_TYPE_AUDIO_ENCODER, gst_opus_enc_setup_interfaces);
-
-static void
-gst_opus_enc_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
-  gst_element_class_set_details_simple (element_class, "Opus audio encoder",
-      "Codec/Encoder/Audio",
-      "Encodes audio in Opus format",
-      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
-}
+#define gst_opus_enc_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstOpusEnc, gst_opus_enc, GST_TYPE_AUDIO_ENCODER,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL);
+    G_IMPLEMENT_INTERFACE (GST_TYPE_PRESET, NULL));
 
 static void
 gst_opus_enc_class_init (GstOpusEncClass * klass)
 {
   GObjectClass *gobject_class;
   GstAudioEncoderClass *base_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
   base_class = (GstAudioEncoderClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   gobject_class->set_property = gst_opus_enc_set_property;
   gobject_class->get_property = gst_opus_enc_get_property;
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_set_details_simple (gstelement_class, "Opus audio encoder",
+      "Codec/Encoder/Audio",
+      "Encodes audio in Opus format",
+      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
+
   base_class->start = GST_DEBUG_FUNCPTR (gst_opus_enc_start);
   base_class->stop = GST_DEBUG_FUNCPTR (gst_opus_enc_stop);
   base_class->set_format = GST_DEBUG_FUNCPTR (gst_opus_enc_set_format);
   base_class->handle_frame = GST_DEBUG_FUNCPTR (gst_opus_enc_handle_frame);
-  base_class->event = GST_DEBUG_FUNCPTR (gst_opus_enc_sink_event);
+  base_class->sink_event = GST_DEBUG_FUNCPTR (gst_opus_enc_sink_event);
   base_class->getcaps = GST_DEBUG_FUNCPTR (gst_opus_enc_sink_getcaps);
 
   g_object_class_install_property (gobject_class, PROP_AUDIO,
@@ -291,6 +276,8 @@ gst_opus_enc_class_init (GstOpusEncClass * klass)
           GST_PARAM_MUTABLE_PLAYING));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_opus_enc_finalize);
+
+  GST_DEBUG_CATEGORY_INIT (opusenc_debug, "opusenc", 0, "Opus encoder");
 }
 
 static void
@@ -306,7 +293,7 @@ gst_opus_enc_finalize (GObject * object)
 }
 
 static void
-gst_opus_enc_init (GstOpusEnc * enc, GstOpusEncClass * klass)
+gst_opus_enc_init (GstOpusEnc * enc)
 {
   GstAudioEncoder *benc = GST_AUDIO_ENCODER (enc);
 
@@ -340,7 +327,7 @@ gst_opus_enc_start (GstAudioEncoder * benc)
   GstOpusEnc *enc = GST_OPUS_ENC (benc);
 
   GST_DEBUG_OBJECT (enc, "start");
-  enc->tags = gst_tag_list_new ();
+  enc->tags = gst_tag_list_new_empty ();
   enc->header_sent = FALSE;
 
   return TRUE;
@@ -381,11 +368,9 @@ gst_opus_enc_setup_base_class (GstOpusEnc * enc, GstAudioEncoder * benc)
 {
   gst_audio_encoder_set_latency (benc,
       gst_opus_enc_get_latency (enc), gst_opus_enc_get_latency (enc));
-  gst_audio_encoder_set_frame_samples_min (benc,
-      enc->frame_samples * enc->n_channels * 2);
-  gst_audio_encoder_set_frame_samples_max (benc,
-      enc->frame_samples * enc->n_channels * 2);
-  gst_audio_encoder_set_frame_max (benc, 0);
+  gst_audio_encoder_set_frame_samples_min (benc, enc->frame_samples);
+  gst_audio_encoder_set_frame_samples_max (benc, enc->frame_samples);
+  gst_audio_encoder_set_frame_max (benc, 1);
 }
 
 static gint
@@ -717,15 +702,16 @@ gst_opus_enc_sink_event (GstAudioEncoder * benc, GstEvent * event)
       gst_tag_setter_merge_tags (setter, list, mode);
       break;
     }
+
     default:
       break;
   }
 
-  return FALSE;
+  return GST_AUDIO_ENCODER_CLASS (parent_class)->sink_event (benc, event);
 }
 
 static GstCaps *
-gst_opus_enc_sink_getcaps (GstAudioEncoder * benc)
+gst_opus_enc_sink_getcaps (GstAudioEncoder * benc, GstCaps * filter)
 {
   GstOpusEnc *enc;
   GstCaps *caps;
@@ -738,7 +724,7 @@ gst_opus_enc_sink_getcaps (GstAudioEncoder * benc)
 
   GST_DEBUG_OBJECT (enc, "sink getcaps");
 
-  peercaps = gst_pad_peer_get_caps (GST_AUDIO_ENCODER_SRC_PAD (benc));
+  peercaps = gst_pad_peer_query_caps (GST_AUDIO_ENCODER_SRC_PAD (benc), NULL);
   if (!peercaps) {
     GST_DEBUG_OBJECT (benc, "No peercaps, returning template sink caps");
     return
@@ -782,6 +768,13 @@ gst_opus_enc_sink_getcaps (GstAudioEncoder * benc)
     g_value_unset (&range);
   }
 
+  if (filter) {
+    GstCaps *tmp = gst_caps_intersect_full (caps, filter,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (caps);
+    caps = tmp;
+  }
+
   GST_DEBUG_OBJECT (enc, "Returning caps: %" GST_PTR_FORMAT, caps);
   return caps;
 }
@@ -789,24 +782,28 @@ gst_opus_enc_sink_getcaps (GstAudioEncoder * benc)
 static GstFlowReturn
 gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buf)
 {
-  guint8 *bdata, *data, *mdata = NULL;
+  guint8 *bdata = NULL, *data, *mdata = NULL;
   gsize bsize, size;
-  gsize bytes;
+  gsize bytes = enc->frame_samples * enc->n_channels * 2;
   gint ret = GST_FLOW_OK;
+  GstMapInfo map;
+  GstMapInfo omap;
+  gint outsize;
+  GstBuffer *outbuf;
 
   g_mutex_lock (enc->property_lock);
 
-  bytes = enc->frame_samples * enc->n_channels * 2;
   if (G_LIKELY (buf)) {
-    bdata = GST_BUFFER_DATA (buf);
-    bsize = GST_BUFFER_SIZE (buf);
+    gst_buffer_map (buf, &map, GST_MAP_READ);
+    bdata = map.data;
+    bsize = map.size;
+
     if (G_UNLIKELY (bsize % bytes)) {
       GST_DEBUG_OBJECT (enc, "draining; adding silence samples");
 
       size = ((bsize / bytes) + 1) * bytes;
       mdata = g_malloc0 (size);
       memcpy (mdata, bdata, bsize);
-      bdata = NULL;
       data = mdata;
     } else {
       data = bdata;
@@ -817,54 +814,49 @@ gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buf)
     goto done;
   }
 
+  g_assert (size == bytes);
 
-  while (size) {
-    gint outsize;
-    GstBuffer *outbuf;
+  outbuf = gst_buffer_new_and_alloc (enc->max_payload_size * enc->n_channels);
+  if (!outbuf)
+    goto done;
 
-    ret = gst_pad_alloc_buffer_and_set_caps (GST_AUDIO_ENCODER_SRC_PAD (enc),
-        GST_BUFFER_OFFSET_NONE, enc->max_payload_size * enc->n_channels,
-        GST_PAD_CAPS (GST_AUDIO_ENCODER_SRC_PAD (enc)), &outbuf);
+  GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)",
+      enc->frame_samples, (int) bytes);
 
-    if (GST_FLOW_OK != ret)
-      goto done;
+  gst_buffer_map (outbuf, &omap, GST_MAP_WRITE);
 
-    GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)",
-        enc->frame_samples, (int) bytes);
+  GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)",
+      enc->frame_samples, (int) bytes);
 
-    outsize =
-        opus_multistream_encode (enc->state, (const gint16 *) data,
-        enc->frame_samples, GST_BUFFER_DATA (outbuf),
-        enc->max_payload_size * enc->n_channels);
+  outsize =
+      opus_multistream_encode (enc->state, (const gint16 *) data,
+      enc->frame_samples, omap.data, enc->max_payload_size * enc->n_channels);
 
-    if (outsize < 0) {
-      GST_ERROR_OBJECT (enc, "Encoding failed: %d", outsize);
-      ret = GST_FLOW_ERROR;
-      goto done;
-    } else if (outsize > enc->max_payload_size) {
-      GST_WARNING_OBJECT (enc,
-          "Encoded size %d is higher than max payload size (%d bytes)",
-          outsize, enc->max_payload_size);
-      ret = GST_FLOW_ERROR;
-      goto done;
-    }
+  gst_buffer_unmap (outbuf, &omap);
 
-    GST_DEBUG_OBJECT (enc, "Output packet is %u bytes", outsize);
-    GST_BUFFER_SIZE (outbuf) = outsize;
-
-    ret =
-        gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (enc), outbuf,
-        enc->frame_samples);
-
-    if ((GST_FLOW_OK != ret) && (GST_FLOW_NOT_LINKED != ret))
-      goto done;
-
-    data += bytes;
-    size -= bytes;
+  if (outsize < 0) {
+    GST_ERROR_OBJECT (enc, "Encoding failed: %d", outsize);
+    ret = GST_FLOW_ERROR;
+    goto done;
+  } else if (outsize > enc->max_payload_size) {
+    GST_WARNING_OBJECT (enc,
+        "Encoded size %d is higher than max payload size (%d bytes)",
+        outsize, enc->max_payload_size);
+    ret = GST_FLOW_ERROR;
+    goto done;
   }
 
+  GST_DEBUG_OBJECT (enc, "Output packet is %u bytes", outsize);
+  gst_buffer_set_size (outbuf, outsize);
+
+  ret =
+      gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (enc), outbuf,
+      enc->frame_samples);
+
 done:
 
+  if (bdata)
+    gst_buffer_unmap (buf, &map);
   g_mutex_unlock (enc->property_lock);
 
   if (mdata)
@@ -898,14 +890,14 @@ gst_opus_enc_handle_frame (GstAudioEncoder * benc, GstBuffer * buf)
     /* negotiate with these caps */
     GST_DEBUG_OBJECT (enc, "here are the caps: %" GST_PTR_FORMAT, caps);
 
-    gst_pad_set_caps (GST_AUDIO_ENCODER_SRC_PAD (enc), caps);
+    gst_audio_encoder_set_output_format (benc, caps);
     gst_caps_unref (caps);
 
     enc->header_sent = TRUE;
   }
 
   GST_DEBUG_OBJECT (enc, "received buffer %p of %u bytes", buf,
-      buf ? GST_BUFFER_SIZE (buf) : 0);
+      buf ? gst_buffer_get_size (buf) : 0);
 
   ret = gst_opus_enc_encode (enc, buf);
 
