@@ -55,7 +55,7 @@ struct _GstSchroDec
 
   SchroDecoder *decoder;
 
-  GstBuffer *seq_header_buffer;
+  gboolean seq_header_buffer_seen;
 };
 
 struct _GstSchroDecClass
@@ -78,12 +78,9 @@ enum
 };
 
 static void gst_schro_dec_finalize (GObject * object);
-static void gst_schro_dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_schro_dec_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
 
-static gboolean gst_schro_dec_sink_query (GstPad * pad, GstQuery * query);
+static gboolean gst_schro_dec_sink_query (GstPad * pad, GstSchroDec * dec,
+    GstQuery * query);
 
 static gboolean gst_schro_dec_start (GstBaseVideoDecoder * dec);
 static gboolean gst_schro_dec_stop (GstBaseVideoDecoder * dec);
@@ -91,7 +88,7 @@ static gboolean gst_schro_dec_reset (GstBaseVideoDecoder * dec);
 static GstFlowReturn gst_schro_dec_parse_data (GstBaseVideoDecoder *
     base_video_decoder, gboolean at_eos);
 static GstFlowReturn gst_schro_dec_handle_frame (GstBaseVideoDecoder * decoder,
-    GstVideoFrame * frame);
+    GstVideoFrameState * frame);
 static gboolean gst_schro_dec_finish (GstBaseVideoDecoder * base_video_decoder);
 static void gst_schrodec_send_tags (GstSchroDec * schro_dec);
 
@@ -106,17 +103,24 @@ static GstStaticPadTemplate gst_schro_dec_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("{ I420, YUY2, AYUV }"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (GST_SCHRO_YUV_LIST))
     );
 
-GST_BOILERPLATE (GstSchroDec, gst_schro_dec, GstBaseVideoDecoder,
-    GST_TYPE_BASE_VIDEO_DECODER);
+#define gst_schro_dec_parent_class parent_class
+G_DEFINE_TYPE (GstSchroDec, gst_schro_dec, GST_TYPE_BASE_VIDEO_DECODER);
 
 static void
-gst_schro_dec_base_init (gpointer g_class)
+gst_schro_dec_class_init (GstSchroDecClass * klass)
 {
+  GObjectClass *gobject_class;
+  GstElementClass *element_class;
+  GstBaseVideoDecoderClass *base_video_decoder_class;
 
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  gobject_class = G_OBJECT_CLASS (klass);
+  element_class = GST_ELEMENT_CLASS (klass);
+  base_video_decoder_class = GST_BASE_VIDEO_DECODER_CLASS (klass);
+
+  gobject_class->finalize = gst_schro_dec_finalize;
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_schro_dec_src_template));
@@ -126,20 +130,6 @@ gst_schro_dec_base_init (gpointer g_class)
   gst_element_class_set_details_simple (element_class, "Dirac Decoder",
       "Codec/Decoder/Video",
       "Decode Dirac streams", "David Schleef <ds@schleef.org>");
-}
-
-static void
-gst_schro_dec_class_init (GstSchroDecClass * klass)
-{
-  GObjectClass *gobject_class;
-  GstBaseVideoDecoderClass *base_video_decoder_class;
-
-  gobject_class = G_OBJECT_CLASS (klass);
-  base_video_decoder_class = GST_BASE_VIDEO_DECODER_CLASS (klass);
-
-  gobject_class->set_property = gst_schro_dec_set_property;
-  gobject_class->get_property = gst_schro_dec_get_property;
-  gobject_class->finalize = gst_schro_dec_finalize;
 
   base_video_decoder_class->start = GST_DEBUG_FUNCPTR (gst_schro_dec_start);
   base_video_decoder_class->stop = GST_DEBUG_FUNCPTR (gst_schro_dec_stop);
@@ -155,12 +145,12 @@ gst_schro_dec_class_init (GstSchroDecClass * klass)
 }
 
 static void
-gst_schro_dec_init (GstSchroDec * schro_dec, GstSchroDecClass * klass)
+gst_schro_dec_init (GstSchroDec * schro_dec)
 {
   GST_DEBUG ("gst_schro_dec_init");
 
   gst_pad_set_query_function (GST_BASE_VIDEO_CODEC_SINK_PAD (schro_dec),
-      gst_schro_dec_sink_query);
+      (GstPadQueryFunction) gst_schro_dec_sink_query);
 
   schro_dec->decoder = schro_decoder_new ();
 }
@@ -172,21 +162,16 @@ static gint64
 granulepos_to_frame (gint64 granulepos)
 {
   guint64 pt;
-  int dist_h;
-  int dist_l;
-  int dist;
-  int delay;
-  guint64 dt;
 
   if (granulepos == -1)
     return -1;
 
   pt = ((granulepos >> 22) + (granulepos & OGG_DIRAC_GRANULE_LOW_MASK)) >> 9;
-  dist_h = (granulepos >> 22) & 0xff;
-  dist_l = granulepos & 0xff;
-  dist = (dist_h << 8) | dist_l;
-  delay = (granulepos >> 9) & 0x1fff;
-  dt = pt - delay;
+  /* dist_h = (granulepos >> 22) & 0xff;
+   * dist_l = granulepos & 0xff;
+   * dist = (dist_h << 8) | dist_l;
+   * delay = (granulepos >> 9) & 0x1fff;
+   * dt = pt - delay; */
 
   return pt >> 1;
 }
@@ -228,13 +213,10 @@ gst_schro_dec_sink_convert (GstPad * pad,
 }
 
 static gboolean
-gst_schro_dec_sink_query (GstPad * pad, GstQuery * query)
+gst_schro_dec_sink_query (GstPad * pad, GstSchroDec * dec, GstQuery * query)
 {
-  GstSchroDec *dec;
   gboolean res = FALSE;
 
-  dec = GST_SCHRO_DEC (gst_pad_get_parent (pad));
-
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONVERT:
     {
@@ -250,13 +232,12 @@ gst_schro_dec_sink_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = gst_pad_query_default (pad, query);
+      res = gst_pad_query_default (pad, GST_OBJECT (dec), query);
       break;
   }
 done:
-  gst_object_unref (dec);
-
   return res;
+
 error:
   GST_DEBUG_OBJECT (dec, "query failed");
   goto done;
@@ -308,38 +289,6 @@ gst_schro_dec_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-gst_schro_dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstSchroDec *src;
-
-  g_return_if_fail (GST_IS_SCHRO_DEC (object));
-  src = GST_SCHRO_DEC (object);
-
-  GST_DEBUG ("gst_schro_dec_set_property");
-  switch (prop_id) {
-    default:
-      break;
-  }
-}
-
-static void
-gst_schro_dec_get_property (GObject * object, guint prop_id, GValue * value,
-    GParamSpec * pspec)
-{
-  GstSchroDec *src;
-
-  g_return_if_fail (GST_IS_SCHRO_DEC (object));
-  src = GST_SCHRO_DEC (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static void
 parse_sequence_header (GstSchroDec * schro_dec, guint8 * data, int size)
 {
@@ -351,18 +300,40 @@ parse_sequence_header (GstSchroDec * schro_dec, guint8 * data, int size)
 
   state = gst_base_video_decoder_get_state (GST_BASE_VIDEO_DECODER (schro_dec));
 
-  schro_dec->seq_header_buffer = gst_buffer_new_and_alloc (size);
-  memcpy (GST_BUFFER_DATA (schro_dec->seq_header_buffer), data, size);
+  schro_dec->seq_header_buffer_seen = TRUE;
 
   ret = schro_parse_decode_sequence_header (data + 13, size - 13,
       &video_format);
   if (ret) {
-    if (video_format.chroma_format == SCHRO_CHROMA_444) {
-      state->format = GST_VIDEO_FORMAT_AYUV;
-    } else if (video_format.chroma_format == SCHRO_CHROMA_422) {
-      state->format = GST_VIDEO_FORMAT_YUY2;
-    } else if (video_format.chroma_format == SCHRO_CHROMA_420) {
-      state->format = GST_VIDEO_FORMAT_I420;
+    int bit_depth;
+
+#if SCHRO_CHECK_VERSION(1,0,11)
+    bit_depth = schro_video_format_get_bit_depth (&video_format);
+#else
+    bit_depth = 8;
+#endif
+
+    if (bit_depth == 8) {
+      if (video_format.chroma_format == SCHRO_CHROMA_444) {
+        state->format = GST_VIDEO_FORMAT_AYUV;
+      } else if (video_format.chroma_format == SCHRO_CHROMA_422) {
+        state->format = GST_VIDEO_FORMAT_UYVY;
+      } else if (video_format.chroma_format == SCHRO_CHROMA_420) {
+        state->format = GST_VIDEO_FORMAT_I420;
+      }
+#if SCHRO_CHECK_VERSION(1,0,11)
+    } else if (bit_depth <= 10) {
+      if (video_format.colour_matrix == SCHRO_COLOUR_MATRIX_REVERSIBLE) {
+        state->format = GST_VIDEO_FORMAT_ARGB;
+      } else {
+        state->format = GST_VIDEO_FORMAT_v210;
+      }
+    } else if (bit_depth <= 16) {
+      state->format = GST_VIDEO_FORMAT_AYUV64;
+    } else {
+      GST_ERROR ("bit depth too large (%d > 16)", bit_depth);
+      state->format = GST_VIDEO_FORMAT_AYUV64;
+#endif
     }
     state->fps_n = video_format.frame_rate_numerator;
     state->fps_d = video_format.frame_rate_denominator;
@@ -382,9 +353,7 @@ parse_sequence_header (GstSchroDec * schro_dec, guint8 * data, int size)
     state->par_d = video_format.aspect_ratio_denominator;
     GST_DEBUG ("Pixel aspect ratio is %d/%d", state->par_n, state->par_d);
 
-    /* FIXME state points to what is actually in the decoder */
-    //gst_base_video_decoder_set_state (GST_BASE_VIDEO_DECODER (schro_dec),
-    //    state);
+    gst_base_video_decoder_set_src_caps (GST_BASE_VIDEO_DECODER (schro_dec));
   } else {
     GST_WARNING ("Failed to get frame rate from sequence header");
   }
@@ -432,7 +401,7 @@ gst_schro_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
   }
 
   if (SCHRO_PARSE_CODE_IS_END_OF_SEQUENCE (parse_code)) {
-    GstVideoFrame *frame;
+    GstVideoFrameState *frame;
 
     if (next != 0 && next != SCHRO_PARSE_HEADER_SIZE) {
       GST_WARNING ("next is not 0 or 13 in EOS packet (%d)", next);
@@ -507,13 +476,13 @@ gst_schro_dec_parse_data (GstBaseVideoDecoder * base_video_decoder,
     g_free (data);
   }
 
-  if (schro_decoder->seq_header_buffer == NULL) {
+  if (!schro_decoder->seq_header_buffer_seen) {
     gst_adapter_flush (base_video_decoder->input_adapter, next);
     return GST_FLOW_OK;
   }
 
   if (SCHRO_PARSE_CODE_IS_PICTURE (parse_code)) {
-    GstVideoFrame *frame;
+    GstVideoFrameState *frame;
     guint8 tmp[4];
 
     frame = base_video_decoder->current_frame;
@@ -538,12 +507,12 @@ gst_schrodec_send_tags (GstSchroDec * schro_dec)
 {
   GstTagList *list;
 
-  list = gst_tag_list_new ();
+  list = gst_tag_list_new_empty ();
   gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
       GST_TAG_VIDEO_CODEC, "Dirac", NULL);
 
-  gst_element_found_tags_for_pad (GST_ELEMENT_CAST (schro_dec),
-      GST_BASE_VIDEO_CODEC_SRC_PAD (schro_dec), list);
+  gst_pad_push_event (GST_BASE_VIDEO_CODEC_SRC_PAD (schro_dec),
+      gst_event_new_tag (gst_tag_list_copy (list)));
 }
 
 static GstFlowReturn
@@ -590,7 +559,7 @@ gst_schro_dec_process (GstSchroDec * schro_dec, gboolean eos)
       {
         SchroFrame *schro_frame;
         SchroTag *tag;
-        GstVideoFrame *frame;
+        GstVideoFrameState *frame;
 
         GST_DEBUG ("got frame");
 
@@ -617,6 +586,8 @@ gst_schro_dec_process (GstSchroDec * schro_dec, gboolean eos)
 
           schro_frame_unref (schro_frame);
         }
+        if (tag)
+          schro_tag_free (tag);
         if (!eos) {
           go = FALSE;
         }
@@ -641,10 +612,9 @@ gst_schro_dec_process (GstSchroDec * schro_dec, gboolean eos)
 
 GstFlowReturn
 gst_schro_dec_handle_frame (GstBaseVideoDecoder * base_video_decoder,
-    GstVideoFrame * frame)
+    GstVideoFrameState * frame)
 {
   GstSchroDec *schro_dec;
-  int schro_ret;
   SchroBuffer *input_buffer;
 
   schro_dec = GST_SCHRO_DEC (base_video_decoder);
@@ -656,7 +626,7 @@ gst_schro_dec_handle_frame (GstBaseVideoDecoder * base_video_decoder,
 
   input_buffer->tag = schro_tag_new (frame, NULL);
 
-  schro_ret = schro_decoder_autoparse_push (schro_dec->decoder, input_buffer);
+  schro_decoder_autoparse_push (schro_dec->decoder, input_buffer);
 
   return gst_schro_dec_process (schro_dec, FALSE);
 }
