@@ -54,6 +54,7 @@
 #  include <config.h>
 #endif
 
+#include <gst/glib-compat-private.h>
 #include "gstbasecamerasrc.h"
 
 enum
@@ -84,7 +85,8 @@ static guint basecamerasrc_signals[LAST_SIGNAL];
 GST_DEBUG_CATEGORY (base_camera_src_debug);
 #define GST_CAT_DEFAULT base_camera_src_debug
 
-GST_BOILERPLATE (GstBaseCameraSrc, gst_base_camera_src, GstBin, GST_TYPE_BIN);
+#define parent_class gst_base_camera_src_parent_class
+G_DEFINE_TYPE (GstBaseCameraSrc, gst_base_camera_src, GST_TYPE_BIN);
 
 static GstStaticPadTemplate vfsrc_template =
 GST_STATIC_PAD_TEMPLATE (GST_BASE_CAMERA_SRC_VIEWFINDER_PAD_NAME,
@@ -110,57 +112,6 @@ GST_STATIC_PAD_TEMPLATE (GST_BASE_CAMERA_SRC_VIDEO_PAD_NAME,
  * be implementing the interface..
  */
 
-/**
- * gst_base_camera_src_get_photography:
- * @self: the camerasrc bin
- *
- * Get object implementing photography interface, if there is one.  Otherwise
- * returns NULL.
- */
-GstPhotography *
-gst_base_camera_src_get_photography (GstBaseCameraSrc * self)
-{
-  GstElement *elem;
-
-  if (GST_IS_PHOTOGRAPHY (self)) {
-    elem = GST_ELEMENT (self);
-  } else {
-    elem = gst_bin_get_by_interface (GST_BIN (self), GST_TYPE_PHOTOGRAPHY);
-  }
-
-  if (elem) {
-    return GST_PHOTOGRAPHY (elem);
-  }
-
-  return NULL;
-}
-
-
-/**
- * gst_base_camera_src_get_colorbalance:
- * @self: the camerasrc bin
- *
- * Get object implementing colorbalance interface, if there is one.  Otherwise
- * returns NULL.
- */
-GstColorBalance *
-gst_base_camera_src_get_color_balance (GstBaseCameraSrc * self)
-{
-  GstElement *elem;
-
-  if (GST_IS_COLOR_BALANCE (self)) {
-    elem = GST_ELEMENT (self);
-  } else {
-    elem = gst_bin_get_by_interface (GST_BIN (self), GST_TYPE_COLOR_BALANCE);
-  }
-
-  if (elem) {
-    return GST_COLOR_BALANCE (self);
-  }
-
-  return NULL;
-}
-
 /**
  * gst_base_camera_src_set_mode:
  * @self: the camerasrc bin
@@ -231,10 +182,10 @@ gst_base_camera_src_start_capture (GstBaseCameraSrc * src)
 
   GST_DEBUG_OBJECT (src, "Starting capture");
 
-  g_mutex_lock (src->capturing_mutex);
+  g_mutex_lock (&src->capturing_mutex);
   if (src->capturing) {
     GST_WARNING_OBJECT (src, "Capturing already ongoing");
-    g_mutex_unlock (src->capturing_mutex);
+    g_mutex_unlock (&src->capturing_mutex);
 
     /* post a warning to notify camerabin2 that the capture failed */
     GST_ELEMENT_WARNING (src, RESOURCE, BUSY, (NULL), (NULL));
@@ -250,7 +201,7 @@ gst_base_camera_src_start_capture (GstBaseCameraSrc * src)
     g_object_notify (G_OBJECT (src), "ready-for-capture");
     GST_WARNING_OBJECT (src, "Failed to start capture");
   }
-  g_mutex_unlock (src->capturing_mutex);
+  g_mutex_unlock (&src->capturing_mutex);
 }
 
 static void
@@ -260,14 +211,14 @@ gst_base_camera_src_stop_capture (GstBaseCameraSrc * src)
 
   g_return_if_fail (klass->stop_capture != NULL);
 
-  g_mutex_lock (src->capturing_mutex);
+  g_mutex_lock (&src->capturing_mutex);
   if (!src->capturing) {
     GST_DEBUG_OBJECT (src, "No ongoing capture");
-    g_mutex_unlock (src->capturing_mutex);
+    g_mutex_unlock (&src->capturing_mutex);
     return;
   }
   klass->stop_capture (src);
-  g_mutex_unlock (src->capturing_mutex);
+  g_mutex_unlock (&src->capturing_mutex);
 }
 
 void
@@ -284,7 +235,7 @@ gst_base_camera_src_dispose (GObject * object)
 {
   GstBaseCameraSrc *src = GST_BASE_CAMERA_SRC_CAST (object);
 
-  g_mutex_free (src->capturing_mutex);
+  g_mutex_clear (&src->capturing_mutex);
 
   if (src->preview_pipeline) {
     gst_camerabin_destroy_preview_pipeline (src->preview_pipeline);
@@ -479,34 +430,15 @@ gst_base_camera_src_change_state (GstElement * element,
   return ret;
 }
 
-static void
-gst_base_camera_src_base_init (gpointer g_class)
-{
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
-
-  GST_DEBUG_CATEGORY_INIT (base_camera_src_debug, "base_camera_src", 0,
-      "Base camera src");
-
-  gst_element_class_set_details_simple (gstelement_class,
-      "Base class for camerabin src bin", "Source/Video",
-      "Abstracts capture device for camerabin2", "Rob Clark <rob@ti.com>");
-
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &vfsrc_template);
-
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &imgsrc_template);
-
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &vidsrc_template);
-}
-
 static void
 gst_base_camera_src_class_init (GstBaseCameraSrcClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
+  GST_DEBUG_CATEGORY_INIT (base_camera_src_debug, "base_camera_src", 0,
+      "Base camera src");
+
   gobject_class = G_OBJECT_CLASS (klass);
   gstelement_class = GST_ELEMENT_CLASS (klass);
 
@@ -586,11 +518,23 @@ gst_base_camera_src_class_init (GstBaseCameraSrcClass * klass)
       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
   gstelement_class->change_state = gst_base_camera_src_change_state;
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "Base class for camerabin src bin", "Source/Video",
+      "Abstracts capture device for camerabin2", "Rob Clark <rob@ti.com>");
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&vfsrc_template));
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&imgsrc_template));
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&vidsrc_template));
 }
 
 static void
-gst_base_camera_src_init (GstBaseCameraSrc * self,
-    GstBaseCameraSrcClass * klass)
+gst_base_camera_src_init (GstBaseCameraSrc * self)
 {
   self->width = DEFAULT_WIDTH;
   self->height = DEFAULT_HEIGHT;
@@ -599,7 +543,7 @@ gst_base_camera_src_init (GstBaseCameraSrc * self,
   self->mode = MODE_IMAGE;
 
   self->capturing = FALSE;
-  self->capturing_mutex = g_mutex_new ();
+  g_mutex_init (&self->capturing_mutex);
 
   self->post_preview = DEFAULT_POST_PREVIEW;
 
