@@ -130,6 +130,7 @@ gst_aiff_mux_change_state (GstElement * element, GstStateChange transition)
       aiffmux->length = 0;
       aiffmux->rate = 0.0;
       aiffmux->sent_header = FALSE;
+      aiffmux->overflow = FALSE;
       break;
     default:
       break;
@@ -160,15 +161,18 @@ gst_aiff_mux_class_init (GstAiffMuxClass * klass)
   (AIFF_FORM_HEADER_LEN + AIFF_COMM_HEADER_LEN + AIFF_SSND_HEADER_LEN)
 
 static void
-gst_aiff_mux_write_form_header (GstAiffMux * aiffmux, guint audio_data_size,
+gst_aiff_mux_write_form_header (GstAiffMux * aiffmux, guint32 audio_data_size,
     GstByteWriter * writer)
 {
   /* ckID == 'FORM' */
-  gst_byte_writer_put_uint32_le (writer, GST_MAKE_FOURCC ('F', 'O', 'R', 'M'));
+  gst_byte_writer_put_uint32_le_unchecked (writer,
+      GST_MAKE_FOURCC ('F', 'O', 'R', 'M'));
   /* ckSize is currently bogus but we'll know what it is later */
-  gst_byte_writer_put_uint32_be (writer, audio_data_size + AIFF_HEADER_LEN - 8);
+  gst_byte_writer_put_uint32_be_unchecked (writer,
+      audio_data_size + AIFF_HEADER_LEN - 8);
   /* formType == 'AIFF' */
-  gst_byte_writer_put_uint32_le (writer, GST_MAKE_FOURCC ('A', 'I', 'F', 'F'));
+  gst_byte_writer_put_uint32_le_unchecked (writer,
+      GST_MAKE_FOURCC ('A', 'I', 'F', 'F'));
 }
 
 /*
@@ -219,8 +223,8 @@ gst_aiff_mux_write_ext (GstByteWriter * writer, double d)
   if (d < 0)
     ext.exponent[0] |= 0x80;
 
-  gst_byte_writer_put_data (writer, ext.exponent, 2);
-  gst_byte_writer_put_data (writer, ext.mantissa, 8);
+  gst_byte_writer_put_data_unchecked (writer, ext.exponent, 2);
+  gst_byte_writer_put_data_unchecked (writer, ext.mantissa, 8);
 }
 
 /*
@@ -228,34 +232,36 @@ gst_aiff_mux_write_ext (GstByteWriter * writer, double d)
  */
 
 static void
-gst_aiff_mux_write_comm_header (GstAiffMux * aiffmux, guint audio_data_size,
+gst_aiff_mux_write_comm_header (GstAiffMux * aiffmux, guint32 audio_data_size,
     GstByteWriter * writer)
 {
-  gst_byte_writer_put_uint32_le (writer, GST_MAKE_FOURCC ('C', 'O', 'M', 'M'));
-  gst_byte_writer_put_uint32_be (writer, 18);
-  gst_byte_writer_put_uint16_be (writer, aiffmux->channels);
+  gst_byte_writer_put_uint32_le_unchecked (writer,
+      GST_MAKE_FOURCC ('C', 'O', 'M', 'M'));
+  gst_byte_writer_put_uint32_be_unchecked (writer, 18);
+  gst_byte_writer_put_uint16_be_unchecked (writer, aiffmux->channels);
   /* numSampleFrames value will be overwritten when known */
-  gst_byte_writer_put_uint32_be (writer,
-      (audio_data_size * 8) / (aiffmux->width * aiffmux->channels));
-  gst_byte_writer_put_uint16_be (writer, aiffmux->depth);
+  gst_byte_writer_put_uint32_be_unchecked (writer,
+      audio_data_size / (aiffmux->width / 8 * aiffmux->channels));
+  gst_byte_writer_put_uint16_be_unchecked (writer, aiffmux->depth);
   gst_aiff_mux_write_ext (writer, aiffmux->rate);
 }
 
 static void
-gst_aiff_mux_write_ssnd_header (GstAiffMux * aiffmux, guint audio_data_size,
+gst_aiff_mux_write_ssnd_header (GstAiffMux * aiffmux, guint32 audio_data_size,
     GstByteWriter * writer)
 {
-  gst_byte_writer_put_uint32_le (writer, GST_MAKE_FOURCC ('S', 'S', 'N', 'D'));
+  gst_byte_writer_put_uint32_le_unchecked (writer,
+      GST_MAKE_FOURCC ('S', 'S', 'N', 'D'));
   /* ckSize will be overwritten when known */
-  gst_byte_writer_put_uint32_be (writer,
+  gst_byte_writer_put_uint32_be_unchecked (writer,
       audio_data_size + AIFF_SSND_HEADER_LEN - 8);
   /* offset and blockSize are set to 0 as we don't support block-aligned sample data yet */
-  gst_byte_writer_put_uint32_be (writer, 0);
-  gst_byte_writer_put_uint32_be (writer, 0);
+  gst_byte_writer_put_uint32_be_unchecked (writer, 0);
+  gst_byte_writer_put_uint32_be_unchecked (writer, 0);
 }
 
 static GstFlowReturn
-gst_aiff_mux_push_header (GstAiffMux * aiffmux, guint audio_data_size)
+gst_aiff_mux_push_header (GstAiffMux * aiffmux, guint32 audio_data_size)
 {
   GstFlowReturn ret;
   GstBuffer *outbuf;
@@ -296,12 +302,16 @@ gst_aiff_mux_chain (GstPad * pad, GstBuffer * buf)
 {
   GstAiffMux *aiffmux = GST_AIFF_MUX (GST_PAD_PARENT (pad));
   GstFlowReturn flow = GST_FLOW_OK;
+  guint64 cur_size;
 
   if (!aiffmux->channels) {
     gst_buffer_unref (buf);
     return GST_FLOW_NOT_NEGOTIATED;
   }
 
+  if (G_UNLIKELY (aiffmux->overflow))
+    goto overflow;
+
   if (!aiffmux->sent_header) {
     /* use bogus size initially, we'll write the real
      * header when we get EOS and know the exact length */
@@ -316,6 +326,20 @@ gst_aiff_mux_chain (GstPad * pad, GstBuffer * buf)
     aiffmux->sent_header = TRUE;
   }
 
+  /* AIFF has an audio data size limit of slightly under 4 GB.
+     A value of audiosize + AIFF_HEADER_LEN - 8 is written, so
+     I'll error out if writing data that makes this overflow. */
+  cur_size = aiffmux->length + AIFF_HEADER_LEN - 8;
+  if (G_UNLIKELY (cur_size + GST_BUFFER_SIZE (buf) >= G_MAXUINT32)) {
+    GST_ERROR_OBJECT (aiffmux, "AIFF only supports about 4 GB worth of "
+        "audio data, dropping any further data on the floor");
+    GST_ELEMENT_WARNING (aiffmux, STREAM, MUX, ("AIFF has a 4GB size limit"),
+        ("AIFF only supports about 4 GB worth of audio data, "
+            "dropping any further data on the floor"));
+    aiffmux->overflow = TRUE;
+    goto overflow;
+  }
+
   GST_LOG_OBJECT (aiffmux, "pushing %u bytes raw audio, ts=%" GST_TIME_FORMAT,
       GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
@@ -330,6 +354,13 @@ gst_aiff_mux_chain (GstPad * pad, GstBuffer * buf)
   flow = gst_pad_push (aiffmux->srcpad, buf);
 
   return flow;
+
+overflow:
+  {
+    GST_WARNING_OBJECT (aiffmux, "output file too large, dropping buffer");
+    gst_buffer_unref (buf);
+    return GST_FLOW_OK;
+  }
 }
 
 static gboolean
