@@ -136,13 +136,13 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static GstStaticPadTemplate video_sink_factory =
-GST_STATIC_PAD_TEMPLATE ("video_%d",
+GST_STATIC_PAD_TEMPLATE ("video_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/x-wmv, wmvversion = (int) [1,3]"));
 
 static GstStaticPadTemplate audio_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("audio_%d",
+    GST_STATIC_PAD_TEMPLATE ("audio_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("audio/x-wma, wmaversion = (int) [1,3]; "
@@ -238,11 +238,12 @@ gst_asf_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &audio_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &video_sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&audio_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_sink_factory));
 
   gst_element_class_set_details_simple (element_class, "ASF muxer",
       "Codec/Muxer",
@@ -334,9 +335,9 @@ gst_asf_mux_init (GstAsfMux * asfmux)
   gst_pad_use_fixed_caps (asfmux->srcpad);
   gst_element_add_pad (GST_ELEMENT (asfmux), asfmux->srcpad);
 
-  asfmux->collect = gst_collect_pads_new ();
-  gst_collect_pads_set_function (asfmux->collect,
-      (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (gst_asf_mux_collected),
+  asfmux->collect = gst_collect_pads2_new ();
+  gst_collect_pads2_set_function (asfmux->collect,
+      (GstCollectPads2Function) GST_DEBUG_FUNCPTR (gst_asf_mux_collected),
       asfmux);
 
   asfmux->payloads = NULL;
@@ -1850,7 +1851,7 @@ gst_asf_mux_process_buffer (GstAsfMux * asfmux, GstAsfPad * pad,
   AsfPayload *payload;
 
   payload = g_malloc0 (sizeof (AsfPayload));
-  payload->pad = (GstCollectData *) pad;
+  payload->pad = (GstCollectData2 *) pad;
   payload->data = buf;
 
   GST_LOG_OBJECT (asfmux,
@@ -1910,7 +1911,7 @@ gst_asf_mux_process_buffer (GstAsfMux * asfmux, GstAsfPad * pad,
 }
 
 static GstFlowReturn
-gst_asf_mux_collected (GstCollectPads * collect, gpointer data)
+gst_asf_mux_collected (GstCollectPads2 * collect, gpointer data)
 {
   GstAsfMux *asfmux = GST_ASF_MUX_CAST (data);
   GstFlowReturn ret = GST_FLOW_OK;
@@ -1936,15 +1937,15 @@ gst_asf_mux_collected (GstCollectPads * collect, gpointer data)
   walk = asfmux->collect->data;
   while (walk) {
     GstAsfPad *pad;
-    GstCollectData *data;
+    GstCollectData2 *data;
     GstClockTime time;
 
-    data = (GstCollectData *) walk->data;
+    data = (GstCollectData2 *) walk->data;
     pad = (GstAsfPad *) data;
 
     walk = g_slist_next (walk);
 
-    buf = gst_collect_pads_peek (collect, data);
+    buf = gst_collect_pads2_peek (collect, data);
     if (buf == NULL) {
       GST_LOG_OBJECT (asfmux, "Pad %s has no buffers",
           GST_PAD_NAME (pad->collect.pad));
@@ -1979,7 +1980,7 @@ gst_asf_mux_collected (GstCollectPads * collect, gpointer data)
     /* we have data */
     GST_LOG_OBJECT (asfmux, "selected pad %s with time %" GST_TIME_FORMAT,
         GST_PAD_NAME (best_pad->collect.pad), GST_TIME_ARGS (best_time));
-    buf = gst_collect_pads_pop (collect, &best_pad->collect);
+    buf = gst_collect_pads2_pop (collect, &best_pad->collect);
     ret = gst_asf_mux_process_buffer (asfmux, best_pad, buf);
   } else {
     /* no data, let's finish it up */
@@ -2278,8 +2279,8 @@ gst_asf_mux_request_new_pad (GstElement * element,
     collect_size = sizeof (GstAsfVideoPad);
   }
   collect_pad = (GstAsfPad *)
-      gst_collect_pads_add_pad_full (asfmux->collect, newpad, collect_size,
-      (GstCollectDataDestroyNotify) (gst_asf_mux_pad_reset));
+      gst_collect_pads2_add_pad_full (asfmux->collect, newpad, collect_size,
+      (GstCollectData2DestroyNotify) (gst_asf_mux_pad_reset), TRUE);
 
   /* set up pad */
   collect_pad->is_audio = is_audio;
@@ -2293,7 +2294,7 @@ gst_asf_mux_request_new_pad (GstElement * element,
   collect_pad->stream_number = asfmux->stream_number;
 
   /* FIXME: hacked way to override/extend the event function of
-   * GstCollectPads; because it sets its own event function giving
+   * GstCollectPads2; because it sets its own event function giving
    * the element no access to events.
    */
   asfmux->collect_event = (GstPadEventFunction) GST_PAD_EVENTFUNC (newpad);
@@ -2390,12 +2391,12 @@ gst_asf_mux_change_state (GstElement * element, GstStateChange transition)
       asfmux->packet_size = asfmux->prop_packet_size;
       asfmux->preroll = asfmux->prop_preroll;
       asfmux->merge_stream_tags = asfmux->prop_merge_stream_tags;
-      gst_collect_pads_start (asfmux->collect);
+      gst_collect_pads2_start (asfmux->collect);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_collect_pads_stop (asfmux->collect);
+      gst_collect_pads2_stop (asfmux->collect);
       asfmux->state = GST_ASF_MUX_STATE_NONE;
       break;
     default:
