@@ -181,7 +181,7 @@ gst_asf_parse_pull_headers (GstAsfParse * asfparse)
     GST_ERROR_OBJECT (asfparse, "Failed to pull data from headers");
     goto leave;
   }
-  headers = gst_buffer_join (guid_and_size, headers);
+  headers = gst_buffer_append (guid_and_size, headers);
   guid_and_size = NULL;
   asfparse->offset += size - ASF_GUID_OBJSIZE_SIZE;
   if (!gst_asf_parse_headers (headers, asfparse->asfinfo)) {
@@ -226,12 +226,13 @@ gst_asf_parse_pull_packets (GstAsfParse * asfparse)
   GstFlowReturn ret;
   while (asfparse->asfinfo->broadcast ||
       asfparse->parsed_packets < asfparse->asfinfo->packets_count) {
-    GstBuffer *packet = NULL;
+    GstBuffer *packet;
 
     GST_DEBUG_OBJECT (asfparse, "Parsing packet %" G_GUINT64_FORMAT,
         asfparse->parsed_packets);
 
     /* get the packet */
+    packet = NULL;
     ret = gst_pad_pull_range (asfparse->sinkpad, asfparse->offset,
         asfparse->asfinfo->packet_size, &packet);
     if (ret != GST_FLOW_OK)
@@ -250,11 +251,12 @@ gst_asf_parse_pull_packets (GstAsfParse * asfparse)
 static GstFlowReturn
 gst_asf_parse_pull_indexes (GstAsfParse * asfparse)
 {
-  GstBuffer *guid_and_size = NULL;
-  GstBuffer *buf = NULL;
+  GstBuffer *guid_and_size;
+  GstBuffer *buf;
   guint64 obj_size;
   GstFlowReturn ret = GST_FLOW_OK;
   while (1) {
+    guid_and_size = NULL;
     ret = gst_pad_pull_range (asfparse->sinkpad, asfparse->offset,
         ASF_GUID_OBJSIZE_SIZE, &guid_and_size);
     if (ret != GST_FLOW_OK)
@@ -271,6 +273,7 @@ gst_asf_parse_pull_indexes (GstAsfParse * asfparse)
     asfparse->offset += ASF_GUID_OBJSIZE_SIZE;
 
     /* pull the rest of the object */
+    buf = NULL;
     ret = gst_pad_pull_range (asfparse->sinkpad, asfparse->offset, obj_size,
         &buf);
     if (ret != GST_FLOW_OK) {
@@ -279,7 +282,7 @@ gst_asf_parse_pull_indexes (GstAsfParse * asfparse)
     }
     asfparse->offset += obj_size - ASF_GUID_OBJSIZE_SIZE;
 
-    buf = gst_buffer_join (guid_and_size, buf);
+    buf = gst_buffer_append (guid_and_size, buf);
     ret = gst_asf_parse_push (asfparse, buf);
     if (ret != GST_FLOW_OK)
       break;
@@ -469,8 +472,10 @@ gst_asf_parse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
   gst_element_class_set_details_simple (element_class, "ASF parser",
       "Parser", "Parses ASF", "Thiago Santos <thiagoss@embedded.ufcg.edu.br>");
