@@ -85,10 +85,10 @@ gst_rtp_asf_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_asf_pay_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_asf_pay_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_asf_pay_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_asf_pay_src_template));
   gst_element_class_set_details_simple (element_class, "RTP ASF payloader",
       "Codec/Payloader/Network",
       "Payload-encodes ASF into RTP packets (MS_RTSP)",
@@ -425,7 +425,7 @@ gst_rtp_asf_pay_handle_buffer (GstBaseRTPPayload * rtppay, GstBuffer * buffer)
     if (gst_asf_match_guid (GST_BUFFER_DATA (buffer),
             &(guids[ASF_DATA_OBJECT_INDEX]))) {
       GST_DEBUG_OBJECT (rtpasfpay, "Received data object header");
-      rtpasfpay->headers = gst_buffer_join (rtpasfpay->headers, buffer);
+      rtpasfpay->headers = gst_buffer_append (rtpasfpay->headers, buffer);
       rtpasfpay->state = ASF_PACKETS;
 
       return gst_rtp_asf_pay_parse_headers (rtpasfpay);
