@@ -102,7 +102,8 @@ gst_camerabin_image_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (eklass, &sink_template);
+  gst_element_class_add_pad_template (eklass,
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_details_simple (eklass,
       "Image capture bin for camerabin", "Bin/Image",
       "Process and store image data",
@@ -216,9 +217,10 @@ gst_camerabin_image_dispose (GstCameraBinImage * img)
      taken by bin and therefore gst_object_sink is called for
      these elements (they may still be in floating state
      and not unreffed properly without sinking first)
+     FIXME, something else is wrong if you have to sink here
    */
   if (img->app_enc) {
-    gst_object_sink (img->app_enc);
+    //gst_object_ref_sink (img->app_enc);
     GST_LOG_OBJECT (img, "disposing %s with refcount %d",
         GST_ELEMENT_NAME (img->app_enc),
         GST_OBJECT_REFCOUNT_VALUE (img->app_enc));
@@ -227,7 +229,7 @@ gst_camerabin_image_dispose (GstCameraBinImage * img)
   }
 
   if (img->post) {
-    gst_object_sink (img->post);
+    //gst_object_ref_sink (img->post);
     GST_LOG_OBJECT (img, "disposing %s with refcount %d",
         GST_ELEMENT_NAME (img->post), GST_OBJECT_REFCOUNT_VALUE (img->post));
     gst_object_unref (img->post);
