@@ -118,8 +118,10 @@ gst_camerabin_video_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (eklass, &sink_template);
-  gst_element_class_add_static_pad_template (eklass, &src_template);
+  gst_element_class_add_pad_template (eklass,
+      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_pad_template (eklass,
+      gst_static_pad_template_get (&src_template));
   gst_element_class_set_details_simple (eklass,
       "Video capture bin for camerabin", "Bin/Video",
       "Process and store video data",
@@ -215,33 +217,34 @@ gst_camerabin_video_dispose (GstCameraBinVideo * vid)
      taken by bin and therefore gst_object_sink is called for
      these elements (they may still be in floating state
      and not unreffed properly without sinking first)
+     FIXME, something else is wrong when you have to sink here
    */
   if (vid->app_post) {
-    gst_object_sink (vid->app_post);
+    //gst_object_sink (vid->app_post);
     gst_object_unref (vid->app_post);
     vid->app_post = NULL;
   }
 
   if (vid->app_vid_enc) {
-    gst_object_sink (vid->app_vid_enc);
+    //gst_object_sink (vid->app_vid_enc);
     gst_object_unref (vid->app_vid_enc);
     vid->app_vid_enc = NULL;
   }
 
   if (vid->app_aud_enc) {
-    gst_object_sink (vid->app_aud_enc);
+    //gst_object_sink (vid->app_aud_enc);
     gst_object_unref (vid->app_aud_enc);
     vid->app_aud_enc = NULL;
   }
 
   if (vid->app_aud_src) {
-    gst_object_sink (vid->app_aud_src);
+    //gst_object_sink (vid->app_aud_src);
     gst_object_unref (vid->app_aud_src);
     vid->app_aud_src = NULL;
   }
 
   if (vid->app_mux) {
-    gst_object_sink (vid->app_mux);
+    //gst_object_sink (vid->app_mux);
     gst_object_unref (vid->app_mux);
     vid->app_mux = NULL;
   }
@@ -506,7 +509,7 @@ gst_camerabin_video_create_elements (GstCameraBinVideo * vid)
   gst_object_unref (vid_sinkpad);
 
   /* Add queue element for video */
-  vid->tee_video_srcpad = gst_element_get_request_pad (vid->tee, "src%d");
+  vid->tee_video_srcpad = gst_element_get_request_pad (vid->tee, "src_%u");
 
   vid->video_queue = gst_element_factory_make ("queue", "video-queue");
   if (!gst_camerabin_add_element (vidbin, vid->video_queue)) {
@@ -617,7 +620,7 @@ gst_camerabin_video_create_elements (GstCameraBinVideo * vid)
     }
   }
   /* Add queue leading out of the video bin and to view finder */
-  vid->tee_vf_srcpad = gst_element_get_request_pad (vid->tee, "src%d");
+  vid->tee_vf_srcpad = gst_element_get_request_pad (vid->tee, "src_%u");
   queue = gst_element_factory_make ("queue", "viewfinder-queue");
   if (!gst_camerabin_add_element (vidbin, queue)) {
     goto error;
