@@ -35,6 +35,7 @@
 
 #include <string.h>
 
+#include <gst/glib-compat-private.h>
 #include "gstinputselector.h"
 #include "gstcamerabin-marshal.h"
 
@@ -42,7 +43,7 @@ GST_DEBUG_CATEGORY_STATIC (input_selector_debug);
 #define GST_CAT_DEFAULT input_selector_debug
 
 static GstStaticPadTemplate gst_input_selector_sink_factory =
-GST_STATIC_PAD_TEMPLATE ("sink%d",
+GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS_ANY);
@@ -675,7 +676,7 @@ flushing:
     GST_DEBUG_OBJECT (pad, "We are flushing, discard buffer %p", buf);
     GST_INPUT_SELECTOR_UNLOCK (sel);
     gst_buffer_unref (buf);
-    res = GST_FLOW_WRONG_STATE;
+    res = GST_FLOW_FLUSHING;
     goto done;
   }
 }
@@ -745,10 +746,10 @@ gst_input_selector_base_init (GstInputSelectorClass * klass)
       "Julien Moutte <julien@moutte.net>, "
       "Jan Schmidt <thaytan@mad.scientist.com>, "
       "Wim Taymans <wim.taymans@gmail.com>");
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_input_selector_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_input_selector_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_input_selector_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_input_selector_src_factory));
 }
 
 static void
@@ -1298,7 +1299,7 @@ gst_input_selector_request_new_pad (GstElement * element,
   GST_INPUT_SELECTOR_LOCK (sel);
 
   GST_LOG_OBJECT (sel, "Creating new pad %d", sel->padcount);
-  name = g_strdup_printf ("sink%d", sel->padcount++);
+  name = g_strdup_printf ("sink_%u", sel->padcount++);
   sinkpad = g_object_new (GST_TYPE_SELECTOR_PAD,
       "name", name, "direction", templ->direction, "template", templ, NULL);
   g_free (name);
