@@ -165,6 +165,7 @@
 #include "gstcamerabin2.h"
 #include <gst/gst-i18n-plugin.h>
 #include <gst/pbutils/pbutils.h>
+#include <gst/glib-compat-private.h>
 
 #if GLIB_CHECK_VERSION(2,29,6)
 #define gst_camerabin2_atomic_int_add g_atomic_int_add
@@ -350,7 +351,7 @@ static GstEvent *
 gst_camera_bin_new_event_renegotiate (void)
 {
   return gst_event_new_custom (GST_EVENT_CUSTOM_BOTH,
-      gst_structure_new ("renegotiate", NULL));
+      gst_structure_new_empty ("renegotiate"));
 }
 
 static GstEvent *
@@ -377,14 +378,15 @@ gst_camera_bin_start_capture (GstCameraBin2 * camerabin)
       return;
     }
 
-    g_mutex_lock (camerabin->video_capture_mutex);
+    g_mutex_lock (&camerabin->video_capture_mutex);
     while (camerabin->video_state == GST_CAMERA_BIN_VIDEO_FINISHING) {
-      g_cond_wait (camerabin->video_state_cond, camerabin->video_capture_mutex);
+      g_cond_wait (&camerabin->video_state_cond,
+          &camerabin->video_capture_mutex);
     }
     if (camerabin->video_state != GST_CAMERA_BIN_VIDEO_IDLE) {
       GST_WARNING_OBJECT (camerabin, "Another video recording is ongoing"
           " (state %d), cannot start a new one", camerabin->video_state);
-      g_mutex_unlock (camerabin->video_capture_mutex);
+      g_mutex_unlock (&camerabin->video_capture_mutex);
       return;
     }
     camerabin->video_state = GST_CAMERA_BIN_VIDEO_STARTING;
@@ -410,20 +412,20 @@ gst_camera_bin_start_capture (GstCameraBin2 * camerabin)
     }
   } else {
     /* store the next capture buffer filename */
-    g_mutex_lock (camerabin->image_capture_mutex);
+    g_mutex_lock (&camerabin->image_capture_mutex);
     camerabin->image_location_list =
         g_slist_append (camerabin->image_location_list, g_strdup (location));
-    g_mutex_unlock (camerabin->image_capture_mutex);
+    g_mutex_unlock (&camerabin->image_capture_mutex);
   }
 
   if (camerabin->post_previews) {
     /* Count processing of preview images too */
     GST_CAMERA_BIN2_PROCESSING_INC (camerabin);
     /* store the next preview filename */
-    g_mutex_lock (camerabin->preview_list_mutex);
+    g_mutex_lock (&camerabin->preview_list_mutex);
     camerabin->preview_location_list =
         g_slist_append (camerabin->preview_location_list, location);
-    g_mutex_unlock (camerabin->preview_list_mutex);
+    g_mutex_unlock (&camerabin->preview_list_mutex);
   } else {
     g_free (location);
   }
@@ -435,7 +437,7 @@ gst_camera_bin_start_capture (GstCameraBin2 * camerabin)
       gst_element_set_state (camerabin->audio_src, GST_STATE_PLAYING);
 
     camerabin->video_state = GST_CAMERA_BIN_VIDEO_RECORDING;
-    g_mutex_unlock (camerabin->video_capture_mutex);
+    g_mutex_unlock (&camerabin->video_capture_mutex);
   }
 
   /*
@@ -450,11 +452,11 @@ gst_camera_bin_start_capture (GstCameraBin2 * camerabin)
   if (camerabin->mode == MODE_IMAGE) {
     /* Store image tags in a list and push them later, this prevents
        start_capture() from blocking in pad_push_event call */
-    g_mutex_lock (camerabin->image_capture_mutex);
+    g_mutex_lock (&camerabin->image_capture_mutex);
     camerabin->image_tags_list =
         g_slist_append (camerabin->image_tags_list,
         taglist ? gst_tag_list_copy (taglist) : NULL);
-    g_mutex_unlock (camerabin->image_capture_mutex);
+    g_mutex_unlock (&camerabin->image_capture_mutex);
   } else if (taglist) {
     GstPad *active_pad;
 
@@ -474,7 +476,7 @@ gst_camera_bin_stop_capture (GstCameraBin2 * camerabin)
 {
   GST_DEBUG_OBJECT (camerabin, "Received stop-capture");
   if (camerabin->mode == MODE_VIDEO) {
-    g_mutex_lock (camerabin->video_capture_mutex);
+    g_mutex_lock (&camerabin->video_capture_mutex);
     if (camerabin->video_state == GST_CAMERA_BIN_VIDEO_RECORDING) {
       if (camerabin->src)
         g_signal_emit_by_name (camerabin->src, "stop-capture", NULL);
@@ -485,7 +487,7 @@ gst_camera_bin_stop_capture (GstCameraBin2 * camerabin)
         gst_element_send_event (camerabin->audio_src, gst_event_new_eos ());
       }
     }
-    g_mutex_unlock (camerabin->video_capture_mutex);
+    g_mutex_unlock (&camerabin->video_capture_mutex);
   }
 }
 
@@ -540,10 +542,10 @@ gst_camera_bin_dispose (GObject * object)
   GstCameraBin2 *camerabin = GST_CAMERA_BIN2_CAST (object);
 
   g_free (camerabin->location);
-  g_mutex_free (camerabin->preview_list_mutex);
-  g_mutex_free (camerabin->image_capture_mutex);
-  g_mutex_free (camerabin->video_capture_mutex);
-  g_cond_free (camerabin->video_state_cond);
+  g_mutex_clear (&camerabin->preview_list_mutex);
+  g_mutex_clear (&camerabin->image_capture_mutex);
+  g_mutex_clear (&camerabin->video_capture_mutex);
+  g_cond_clear (&camerabin->video_state_cond);
 
   if (camerabin->src_capture_notify_id)
     g_signal_handler_disconnect (camerabin->src,
@@ -755,7 +757,7 @@ gst_camera_bin_class_init (GstCameraBin2Class * klass)
           GST_TYPE_CAPS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (object_class, PROP_VIDEO_ENCODING_PROFILE,
-      gst_param_spec_mini_object ("video-profile", "Video Profile",
+      g_param_spec_object ("video-profile", "Video Profile",
           "The GstEncodingProfile to use for video recording. Audio is enabled "
           "when this profile supports audio.", GST_TYPE_ENCODING_PROFILE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
@@ -826,7 +828,7 @@ gst_camera_bin_class_init (GstCameraBin2Class * klass)
    *   getting the 2nd buffer.
    */
   g_object_class_install_property (object_class, PROP_IMAGE_ENCODING_PROFILE,
-      gst_param_spec_mini_object ("image-profile", "Image Profile",
+      g_param_spec_object ("image-profile", "Image Profile",
           "The GstEncodingProfile to use for image captures.",
           GST_TYPE_ENCODING_PROFILE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
@@ -897,10 +899,10 @@ gst_camera_bin_init (GstCameraBin2 * camera)
   camera->zoom = DEFAULT_ZOOM;
   camera->max_zoom = MAX_ZOOM;
   camera->flags = DEFAULT_FLAGS;
-  camera->preview_list_mutex = g_mutex_new ();
-  camera->image_capture_mutex = g_mutex_new ();
-  camera->video_capture_mutex = g_mutex_new ();
-  camera->video_state_cond = g_cond_new ();
+  g_mutex_init (&camera->preview_list_mutex);
+  g_mutex_init (&camera->image_capture_mutex);
+  g_mutex_init (&camera->video_capture_mutex);
+  g_cond_init (&camera->video_state_cond);
 
   /* capsfilters are created here as we proxy their caps properties and
    * this way we avoid having to store the caps while on NULL state to 
@@ -946,7 +948,7 @@ gst_video_capture_bin_post_video_done (GstCameraBin2 * camera)
   GstMessage *msg;
 
   msg = gst_message_new_element (GST_OBJECT_CAST (camera),
-      gst_structure_new ("video-done", NULL));
+      gst_structure_new_empty ("video-done"));
 
   if (!gst_element_post_message (GST_ELEMENT_CAST (camera), msg))
     GST_WARNING_OBJECT (camera, "Failed to post video-done message");
@@ -957,7 +959,7 @@ gst_camera_bin_skip_next_preview (GstCameraBin2 * camerabin)
 {
   gchar *location;
 
-  g_mutex_lock (camerabin->preview_list_mutex);
+  g_mutex_lock (&camerabin->preview_list_mutex);
   if (camerabin->preview_location_list) {
     location = camerabin->preview_location_list->data;
     GST_DEBUG_OBJECT (camerabin, "Skipping preview for %s", location);
@@ -969,7 +971,7 @@ gst_camera_bin_skip_next_preview (GstCameraBin2 * camerabin)
   } else {
     GST_WARNING_OBJECT (camerabin, "No previews to skip");
   }
-  g_mutex_unlock (camerabin->preview_list_mutex);
+  g_mutex_unlock (&camerabin->preview_list_mutex);
 }
 
 static gpointer
@@ -978,7 +980,7 @@ gst_camera_bin_video_reset_elements (gpointer u_data)
   GstCameraBin2 *camerabin = GST_CAMERA_BIN2_CAST (u_data);
 
   GST_DEBUG_OBJECT (camerabin, "Resetting video elements state");
-  g_mutex_lock (camerabin->video_capture_mutex);
+  g_mutex_lock (&camerabin->video_capture_mutex);
 
   /* reset element states to clear eos/flushing pads */
   gst_element_set_state (camerabin->video_encodebin, GST_STATE_READY);
@@ -1014,8 +1016,8 @@ gst_camera_bin_video_reset_elements (gpointer u_data)
 
   GST_DEBUG_OBJECT (camerabin, "Setting video state to idle");
   camerabin->video_state = GST_CAMERA_BIN_VIDEO_IDLE;
-  g_cond_signal (camerabin->video_state_cond);
-  g_mutex_unlock (camerabin->video_capture_mutex);
+  g_cond_signal (&camerabin->video_state_cond);
+  g_mutex_unlock (&camerabin->video_capture_mutex);
 
   gst_object_unref (camerabin);
   return NULL;
@@ -1044,7 +1046,7 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
       } else if (gst_structure_has_name (structure, "preview-image")) {
         gchar *location = NULL;
 
-        g_mutex_lock (camerabin->preview_list_mutex);
+        g_mutex_lock (&camerabin->preview_list_mutex);
         if (camerabin->preview_location_list) {
           location = camerabin->preview_location_list->data;
           camerabin->preview_location_list =
@@ -1058,7 +1060,7 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
               "happen if the source is posting previews while camerabin2 is "
               "shutting down");
         }
-        g_mutex_unlock (camerabin->preview_list_mutex);
+        g_mutex_unlock (&camerabin->preview_list_mutex);
 
         if (location) {
           GValue value = { 0 };
@@ -1095,23 +1097,25 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
       GstElement *src = GST_ELEMENT (GST_MESSAGE_SRC (message));
       if (src == GST_CAMERA_BIN2_CAST (bin)->videosink) {
 
-        g_mutex_lock (camerabin->video_capture_mutex);
+        g_mutex_lock (&camerabin->video_capture_mutex);
         GST_DEBUG_OBJECT (bin, "EOS from video branch");
         g_assert (camerabin->video_state == GST_CAMERA_BIN_VIDEO_FINISHING);
 
         gst_video_capture_bin_post_video_done (GST_CAMERA_BIN2_CAST (bin));
         dec_counter = TRUE;
 
-        if (!g_thread_create (gst_camera_bin_video_reset_elements,
-                gst_object_ref (camerabin), FALSE, NULL)) {
-          GST_WARNING_OBJECT (camerabin, "Failed to create thread to "
+        if (!g_thread_try_new ("reset-element-thread",
+                gst_camera_bin_video_reset_elements, gst_object_ref (camerabin),
+                NULL)) {
+          GST_WARNING_OBJECT (camerabin,
+              "Failed to create thread to "
               "reset video elements' state, video recordings may not work "
               "anymore");
           gst_object_unref (camerabin);
           camerabin->video_state = GST_CAMERA_BIN_VIDEO_IDLE;
         }
 
-        g_mutex_unlock (camerabin->video_capture_mutex);
+        g_mutex_unlock (&camerabin->video_capture_mutex);
       }
     }
       break;
@@ -1187,17 +1191,19 @@ encodebin_element_added (GstElement * encodebin, GstElement * new_element,
   GstElementFactory *factory = gst_element_get_factory (new_element);
 
   if (factory != NULL) {
-    if (strcmp (GST_PLUGIN_FEATURE_NAME (factory), "audiorate") == 0 ||
-        strcmp (GST_PLUGIN_FEATURE_NAME (factory), "videorate") == 0) {
+    if (strcmp (GST_OBJECT_NAME (factory), "audiorate") == 0 ||
+        strcmp (GST_OBJECT_NAME (factory), "videorate") == 0) {
       g_object_set (new_element, "skip-to-first", TRUE, NULL);
     }
   }
 
-  if (gst_element_implements_interface (new_element, GST_TYPE_TAG_SETTER)) {
-    GstTagSetter *tagsetter = GST_TAG_SETTER (new_element);
+  /* TODO porting
+     if (gst_element_implements_interface (new_element, GST_TYPE_TAG_SETTER)) {
+     GstTagSetter *tagsetter = GST_TAG_SETTER (new_element);
 
-    gst_tag_setter_set_tag_merge_mode (tagsetter, GST_TAG_MERGE_REPLACE);
-  }
+     gst_tag_setter_set_tag_merge_mode (tagsetter, GST_TAG_MERGE_REPLACE);
+     }
+   */
 }
 
 #define VIDEO_PAD 1
@@ -1206,6 +1212,7 @@ static GstPad *
 encodebin_find_pad (GstCameraBin2 * camera, GstElement * encodebin,
     gint pad_type)
 {
+  GValue value = { 0 };
   GstPad *pad = NULL;
   GstIterator *iter;
   gboolean done;
@@ -1216,8 +1223,10 @@ encodebin_find_pad (GstCameraBin2 * camera, GstElement * encodebin,
   iter = gst_element_iterate_sink_pads (encodebin);
   done = FALSE;
   while (!done) {
-    switch (gst_iterator_next (iter, (gpointer *) & pad)) {
+    switch (gst_iterator_next (iter, &value)) {
       case GST_ITERATOR_OK:
+        pad = g_value_dup_object (&value);
+        g_value_unset (&value);
         if (pad_type == VIDEO_PAD) {
           if (strstr (GST_PAD_NAME (pad), "video") != NULL) {
             GST_DEBUG_OBJECT (camera, "Found video pad %s", GST_PAD_NAME (pad));
@@ -1338,18 +1347,18 @@ gst_camera_bin_src_notify_zoom_cb (GObject * self, GParamSpec * pspec,
   g_object_notify (G_OBJECT (camera), "zoom");
 }
 
-static gboolean
-gst_camera_bin_image_src_buffer_probe (GstPad * pad, GstBuffer * buf,
+static GstPadProbeReturn
+gst_camera_bin_image_src_buffer_probe (GstPad * pad, GstPadProbeInfo * info,
     gpointer data)
 {
-  gboolean ret = TRUE;
+  GstPadProbeReturn ret = GST_PAD_PROBE_OK;
   GstCameraBin2 *camerabin = data;
   GstEvent *evt;
   gchar *location = NULL;
   GstPad *peer;
   GstTagList *tags;
 
-  g_mutex_lock (camerabin->image_capture_mutex);
+  g_mutex_lock (&camerabin->image_capture_mutex);
 
   /* Push pending image tags */
   if (camerabin->image_tags_list) {
@@ -1378,10 +1387,10 @@ gst_camera_bin_image_src_buffer_probe (GstPad * pad, GstBuffer * buf,
         location);
   } else {
     GST_DEBUG_OBJECT (camerabin, "No filename location change to send");
-    g_mutex_unlock (camerabin->image_capture_mutex);
+    g_mutex_unlock (&camerabin->image_capture_mutex);
     return ret;
   }
-  g_mutex_unlock (camerabin->image_capture_mutex);
+  g_mutex_unlock (&camerabin->image_capture_mutex);
 
   if (location) {
     evt = gst_camera_bin_new_event_file_location (location);
@@ -1392,7 +1401,7 @@ gst_camera_bin_image_src_buffer_probe (GstPad * pad, GstBuffer * buf,
   } else {
     /* This means we don't have to encode the capture, it is used for
      * signaling the application just wants the preview */
-    ret = FALSE;
+    ret = GST_PAD_PROBE_DROP;
     GST_CAMERA_BIN2_PROCESSING_DEC (camerabin);
   }
 
@@ -1400,11 +1409,12 @@ gst_camera_bin_image_src_buffer_probe (GstPad * pad, GstBuffer * buf,
 }
 
 
-static gboolean
-gst_camera_bin_image_sink_event_probe (GstPad * pad, GstEvent * event,
+static GstPadProbeReturn
+gst_camera_bin_image_sink_event_probe (GstPad * pad, GstPadProbeInfo * info,
     gpointer data)
 {
   GstCameraBin2 *camerabin = data;
+  GstEvent *event = GST_EVENT (info->data);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_CUSTOM_DOWNSTREAM:{
@@ -1430,21 +1440,22 @@ gst_camera_bin_image_sink_event_probe (GstPad * pad, GstEvent * event,
       break;
   }
 
-  return TRUE;
+  return GST_PAD_PROBE_OK;
 }
 
-static gboolean
-gst_camera_bin_audio_src_data_probe (GstPad * pad, GstMiniObject * obj,
+static GstPadProbeReturn
+gst_camera_bin_audio_src_data_probe (GstPad * pad, GstPadProbeInfo * info,
     gpointer data)
 {
   GstCameraBin2 *camera = data;
-  gboolean ret = TRUE;
+  gboolean ret = GST_PAD_PROBE_OK;
 
-  if (GST_IS_BUFFER (obj)) {
+  if (GST_IS_BUFFER (data)) {
     if (G_UNLIKELY (camera->audio_send_newseg)) {
-      GstBuffer *buf = GST_BUFFER_CAST (obj);
+      GstBuffer *buf = GST_BUFFER_CAST (data);
       GstClockTime ts = GST_BUFFER_TIMESTAMP (buf);
       GstPad *peer;
+      GstSegment segment;
 
       if (!GST_CLOCK_TIME_IS_VALID (ts)) {
         ts = 0;
@@ -1453,27 +1464,28 @@ gst_camera_bin_audio_src_data_probe (GstPad * pad, GstMiniObject * obj,
       peer = gst_pad_get_peer (pad);
       g_return_val_if_fail (peer != NULL, TRUE);
 
-      gst_pad_send_event (peer, gst_event_new_new_segment (FALSE, 1.0,
-              GST_FORMAT_TIME, ts, -1, 0));
+      gst_segment_init (&segment, GST_FORMAT_TIME);
+      segment.start = ts;
+      gst_pad_send_event (peer, gst_event_new_segment (&segment));
 
       gst_object_unref (peer);
 
       camera->audio_send_newseg = FALSE;
     }
   } else {
-    GstEvent *event = GST_EVENT_CAST (obj);
+    GstEvent *event = GST_EVENT_CAST (data);
     if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
       /* we only let an EOS pass when the user is stopping a capture */
       if (camera->audio_drop_eos) {
-        ret = FALSE;
+        ret = GST_PAD_PROBE_DROP;
       } else {
         camera->audio_drop_eos = TRUE;
         /* should already be false, but reinforce in case no buffers get
          * pushed */
         camera->audio_send_newseg = FALSE;
       }
-    } else if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
-      ret = FALSE;
+    } else if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
+      ret = GST_PAD_PROBE_DROP;
     }
   }
 
@@ -1538,12 +1550,12 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
       GstEncodingContainerProfile *prof;
       GstCaps *caps;
 
-      caps = gst_caps_new_simple ("application/ogg", NULL);
+      caps = gst_caps_new_empty_simple ("application/ogg");
       prof = gst_encoding_container_profile_new ("ogg", "theora+vorbis+ogg",
           caps, NULL);
       gst_caps_unref (caps);
 
-      caps = gst_caps_new_simple ("video/x-theora", NULL);
+      caps = gst_caps_new_empty_simple ("video/x-theora");
       if (!gst_encoding_container_profile_add_profile (prof,
               (GstEncodingProfile *) gst_encoding_video_profile_new (caps,
                   NULL, NULL, 1))) {
@@ -1551,7 +1563,7 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
       }
       gst_caps_unref (caps);
 
-      caps = gst_caps_new_simple ("audio/x-vorbis", NULL);
+      caps = gst_caps_new_empty_simple ("audio/x-vorbis");
       if (!gst_encoding_container_profile_add_profile (prof,
               (GstEncodingProfile *) gst_encoding_audio_profile_new (caps,
                   NULL, NULL, 1))) {
@@ -1590,7 +1602,7 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
       GstEncodingVideoProfile *vprof;
       GstCaps *caps;
 
-      caps = gst_caps_new_simple ("image/jpeg", NULL);
+      caps = gst_caps_new_empty_simple ("image/jpeg");
       vprof = gst_encoding_video_profile_new (caps, NULL, NULL, 1);
       gst_encoding_video_profile_set_variableframerate (vprof, TRUE);
 
@@ -1637,8 +1649,9 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
 
       srcpad = gst_element_get_static_pad (camera->image_encodebin, "src");
 
-      gst_pad_add_event_probe (srcpad,
-          (GCallback) gst_camera_bin_image_sink_event_probe, camera);
+      gst_pad_add_probe (srcpad, GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM,
+          gst_camera_bin_image_sink_event_probe, gst_object_ref (camera),
+          gst_object_unref);
 
       gst_object_unref (srcpad);
     }
@@ -1766,8 +1779,9 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
       goto fail;
     }
 
-    gst_pad_add_buffer_probe (imgsrc,
-        (GCallback) gst_camera_bin_image_src_buffer_probe, camera);
+    gst_pad_add_probe (imgsrc, GST_PAD_PROBE_TYPE_BUFFER,
+        gst_camera_bin_image_src_buffer_probe, gst_object_ref (camera),
+        gst_object_unref);
     gst_object_unref (imgsrc);
   }
 
@@ -1829,11 +1843,11 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
 
     srcpad = gst_element_get_static_pad (camera->audio_src, "src");
 
-    /* 1) drop EOS for audiosrc elements that push them on state_changes
-     * (basesrc does this) 
-     * 2) Fix newsegment events to have start time = first buffer ts */
-    gst_pad_add_data_probe (srcpad,
-        (GCallback) gst_camera_bin_audio_src_data_probe, camera);
+    /* drop EOS for audiosrc elements that push them on state_changes
+     * (basesrc does this) */
+    gst_pad_add_probe (srcpad, GST_PAD_PROBE_TYPE_DATA_DOWNSTREAM,
+        gst_camera_bin_audio_src_data_probe, gst_object_ref (camera),
+        gst_object_unref);
 
     gst_object_unref (srcpad);
   }
@@ -1918,7 +1932,7 @@ gst_camera_bin_change_state (GstElement * element, GstStateChange trans)
       GST_CAMERA_BIN2_RESET_PROCESSING_COUNTER (camera);
       camera->video_state = GST_CAMERA_BIN_VIDEO_IDLE;
 
-      g_mutex_lock (camera->image_capture_mutex);
+      g_mutex_lock (&camera->image_capture_mutex);
       g_slist_foreach (camera->image_location_list, (GFunc) g_free, NULL);
       g_slist_free (camera->image_location_list);
       camera->image_location_list = NULL;
@@ -1927,13 +1941,13 @@ gst_camera_bin_change_state (GstElement * element, GstStateChange trans)
           (GFunc) _gst_tag_list_free_maybe, NULL);
       g_slist_free (camera->image_tags_list);
       camera->image_tags_list = NULL;
-      g_mutex_unlock (camera->image_capture_mutex);
+      g_mutex_unlock (&camera->image_capture_mutex);
 
-      g_mutex_lock (camera->preview_list_mutex);
+      g_mutex_lock (&camera->preview_list_mutex);
       g_slist_foreach (camera->preview_location_list, (GFunc) g_free, NULL);
       g_slist_free (camera->preview_location_list);
       camera->preview_location_list = NULL;
-      g_mutex_unlock (camera->preview_list_mutex);
+      g_mutex_unlock (&camera->preview_list_mutex);
 
       /* explicitly set to READY as they might be outside of the bin */
       gst_element_set_state (camera->audio_volume, GST_STATE_READY);
@@ -2169,8 +2183,7 @@ gst_camera_bin_set_property (GObject * object, guint prop_id,
     case PROP_VIDEO_ENCODING_PROFILE:
       if (camera->video_profile)
         gst_encoding_profile_unref (camera->video_profile);
-      camera->video_profile =
-          (GstEncodingProfile *) gst_value_dup_mini_object (value);
+      camera->video_profile = (GstEncodingProfile *) g_value_dup_boxed (value);
       camera->video_profile_switch = TRUE;
       break;
     case PROP_IMAGE_FILTER:
@@ -2226,8 +2239,7 @@ gst_camera_bin_set_property (GObject * object, guint prop_id,
     case PROP_IMAGE_ENCODING_PROFILE:
       if (camera->image_profile)
         gst_encoding_profile_unref (camera->image_profile);
-      camera->image_profile =
-          (GstEncodingProfile *) gst_value_dup_mini_object (value);
+      camera->image_profile = (GstEncodingProfile *) g_value_dup_boxed (value);
       camera->image_profile_switch = TRUE;
       break;
     case PROP_FLAGS:
@@ -2298,7 +2310,7 @@ gst_camera_bin_get_property (GObject * object, guint prop_id,
          * won't negotiate. Need to take care on the special case of the
          * pad being unlinked.
          */
-        caps = gst_pad_get_caps_reffed (pad);
+        caps = gst_pad_query_caps (pad, NULL);
         if (caps) {
           gst_value_set_caps (value, caps);
           gst_caps_unref (caps);
@@ -2364,8 +2376,7 @@ gst_camera_bin_get_property (GObject * object, guint prop_id,
       break;
     case PROP_VIDEO_ENCODING_PROFILE:
       if (camera->video_profile) {
-        gst_value_set_mini_object (value,
-            (GstMiniObject *) camera->video_profile);
+        g_value_set_boxed (value, camera->video_profile);
       }
       break;
     case PROP_VIDEO_FILTER:
@@ -2403,8 +2414,7 @@ gst_camera_bin_get_property (GObject * object, guint prop_id,
       break;
     case PROP_IMAGE_ENCODING_PROFILE:
       if (camera->image_profile) {
-        gst_value_set_mini_object (value,
-            (GstMiniObject *) camera->image_profile);
+        g_value_set_boxed (value, camera->image_profile);
       }
       break;
     case PROP_IDLE:
