@@ -24,7 +24,6 @@
 #endif
 
 #include "gstviewfinderbin.h"
-#include "gstimagecapturebin.h"
 #include "gstwrappercamerabinsrc.h"
 #include "gstcamerabin2.h"
 
@@ -33,11 +32,9 @@ plugin_init (GstPlugin * plugin)
 {
   if (!gst_viewfinder_bin_plugin_init (plugin))
     return FALSE;
-  if (!gst_image_capture_bin_plugin_init (plugin))
-    return FALSE;
   if (!gst_wrapper_camera_bin_src_plugin_init (plugin))
     return FALSE;
-  if (!gst_camera_bin_plugin_init (plugin))
+  if (!gst_camera_bin2_plugin_init (plugin))
     return FALSE;
 
   return TRUE;
@@ -45,5 +42,5 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "camerabin2", "camerabin2",
+    camerabin2, "camerabin2",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
