@@ -136,8 +136,10 @@ gst_field_analysis_base_init (gpointer gclass)
       "Analyse fields from video frames to identify if they are progressive/telecined/interlaced",
       "Robert Swain <robert.swain@collabora.co.uk>");
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 }
 
 typedef enum
@@ -1799,14 +1801,12 @@ fieldanalysis_init (GstPlugin * fieldanalysis)
   GST_DEBUG_CATEGORY_INIT (gst_field_analysis_debug, "fieldanalysis",
       0, "Video field analysis");
 
-  gst_fieldanalysis_orc_init ();
-
   return gst_element_register (fieldanalysis, "fieldanalysis", GST_RANK_NONE,
       GST_TYPE_FIELDANALYSIS);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "fieldanalysis",
+    fieldanalysis,
     "Video field analysis",
     fieldanalysis_init, VERSION, "LGPL", "GStreamer", "http://gstreamer.net/")
