@@ -26,7 +26,6 @@
 #include "gstfrei0rsrc.h"
 #include "gstfrei0rmixer.h"
 
-#include <gst/controller/gstcontroller.h>
 #include <string.h>
 
 GST_DEBUG_CATEGORY (frei0r_debug);
@@ -669,8 +668,6 @@ plugin_init (GstPlugin * plugin)
 
   GST_DEBUG_CATEGORY_INIT (frei0r_debug, "frei0r", 0, "frei0r");
 
-  gst_controller_init (NULL, NULL);
-
   gst_plugin_add_dependency_simple (plugin,
       "FREI0R_PATH:HOME/.frei0r-1/lib",
       "/usr/lib/frei0r-1:/usr/local/lib/frei0r-1:"
@@ -713,6 +710,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "frei0r",
+    frei0r,
     "frei0r plugin library",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
