@@ -71,8 +71,10 @@ gauss_blur_base_init (gpointer gclass)
       "Perform Gaussian blur/sharpen on a video",
       "Jan Schmidt <thaytan@noraisin.net>");
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 }
 
 static void
@@ -167,7 +169,7 @@ gauss_blur_process_frame (GstBaseTransform * btrans,
   stream_time =
       gst_segment_to_stream_time (&btrans->segment, GST_FORMAT_TIME, timestamp);
   if (GST_CLOCK_TIME_IS_VALID (stream_time))
-    gst_object_sync_values (G_OBJECT (gb), stream_time);
+    gst_object_sync_values (GST_OBJECT (gb), stream_time);
 
   GST_OBJECT_LOCK (gb);
   sigma = gb->sigma;
