@@ -22,6 +22,6 @@ fragmented_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "fragmented",
+    fragmented,
     "Fragmented streaming plugins",
     fragmented_init, VERSION, "LGPL", PACKAGE_NAME, "http://www.gstreamer.org/")
