@@ -27,6 +27,8 @@
 #include <gst/gst.h>
 #include <gst/base/gstadapter.h>
 #include "m3u8.h"
+#include "gstfragmented.h"
+#include "gsturidownloader.h"
 
 G_BEGIN_DECLS
 #define GST_TYPE_HLS_DEMUX \
@@ -51,12 +53,12 @@ struct _GstHLSDemux
 {
   GstElement parent;
 
-  GstTask *task;
-  GStaticRecMutex task_lock;
   GstPad *srcpad;
   GstPad *sinkpad;
+
   GstBuffer *playlist;
   GstCaps *input_caps;
+  GstUriDownloader *downloader;
   GstM3U8Client *client;        /* M3U8 client */
   GQueue *queue;                /* Queue storing the fetched fragments */
   gboolean need_cache;          /* Wheter we need to cache some fragments before starting to push data */
@@ -67,25 +69,18 @@ struct _GstHLSDemux
   guint fragments_cache;        /* number of fragments needed to be cached to start playing */
   gfloat bitrate_switch_tol;    /* tolerance with respect to the fragment duration to switch the bitarate*/
 
-  /* Updates thread */
-  GThread *updates_thread;      /* Thread handling the playlist and fragments updates */
-  GMutex *thread_lock;          /* Thread lock */
-  GCond *thread_cond;           /* Signals the thread to quit */
-  gboolean thread_return;       /* Instructs the thread to return after the thread_quit condition is meet */
+  /* Streaming task */
+  GstTask *stream_task;
+  GStaticRecMutex stream_lock;
+  gboolean stop_stream_task;
+
+  /* Updates task */
+  GstTask *updates_task;
+  GStaticRecMutex updates_lock;
+  GMutex *updates_timed_lock;
   GTimeVal next_update;         /* Time of the next update */
   gint64 accumulated_delay;     /* Delay accumulated fetching fragments, used to decide a playlist switch */
-
-  /* Fragments fetcher */
-  GstElement *fetcher;
-  GstBus *fetcher_bus;
-  GstPad *fetcherpad;
-  GMutex *fetcher_lock;
-  GCond *fetcher_cond;
-  GTimeVal *timeout;
-  gboolean fetcher_error;
-  gboolean stopping_fetcher;
   gboolean cancelled;
-  GstAdapter *download;
 
   /* Position in the stream */
   GstClockTime position;
