@@ -89,8 +89,8 @@ gst_rfb_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rfb_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rfb_src_template));
 
   gst_element_class_set_details_simple (element_class, "Rfb source",
       "Source/Video",
@@ -546,6 +546,6 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "rfbsrc",
+    rfbsrc,
     "Connects to a VNC server and decodes RFB stream",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
