@@ -60,7 +60,7 @@ enum
 #define DEFAULT_AGGREGATE_GOPS FALSE
 
 static GstStaticPadTemplate mpegpsmux_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("sink_%d",
+    GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/mpeg, "
@@ -98,7 +98,7 @@ static gboolean new_packet_cb (guint8 * data, guint len, void *user_data);
 static void release_buffer_cb (guint8 * data, void *user_data);
 
 static gboolean mpegpsdemux_prepare_srcpad (MpegPsMux * mux);
-static GstFlowReturn mpegpsmux_collected (GstCollectPads * pads,
+static GstFlowReturn mpegpsmux_collected (GstCollectPads2 * pads,
     MpegPsMux * mux);
 static GstPad *mpegpsmux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
@@ -114,10 +114,10 @@ mpegpsmux_base_init (gpointer g_class)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegpsmux_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegpsmux_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegpsmux_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegpsmux_src_factory));
 
   gst_element_class_set_details_simple (element_class,
       "MPEG Program Stream Muxer", "Codec/Muxer",
@@ -153,9 +153,9 @@ mpegpsmux_init (MpegPsMux * mux, MpegPsMuxClass * g_class)
   gst_pad_use_fixed_caps (mux->srcpad);
   gst_element_add_pad (GST_ELEMENT (mux), mux->srcpad);
 
-  mux->collect = gst_collect_pads_new ();
-  gst_collect_pads_set_function (mux->collect,
-      (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (mpegpsmux_collected), mux);
+  mux->collect = gst_collect_pads2_new ();
+  gst_collect_pads2_set_function (mux->collect,
+      (GstCollectPads2Function) GST_DEBUG_FUNCPTR (mpegpsmux_collected), mux);
 
   mux->psmux = psmux_new ();
   psmux_set_write_func (mux->psmux, new_packet_cb, mux);
@@ -371,7 +371,7 @@ mpegpsmux_create_streams (MpegPsMux * mux)
 
   /* Create the streams */
   while (walk) {
-    GstCollectData *c_data = (GstCollectData *) walk->data;
+    GstCollectData2 *c_data = (GstCollectData2 *) walk->data;
     MpegPsPadData *ps_data = (MpegPsPadData *) walk->data;
 
     walk = g_slist_next (walk);
@@ -396,11 +396,11 @@ mpegpsmux_choose_best_stream (MpegPsMux * mux)
   /* Choose from which stream to mux with */
 
   MpegPsPadData *best = NULL;
-  GstCollectData *c_best = NULL;
+  GstCollectData2 *c_best = NULL;
   GSList *walk;
 
   for (walk = mux->collect->data; walk != NULL; walk = g_slist_next (walk)) {
-    GstCollectData *c_data = (GstCollectData *) walk->data;
+    GstCollectData2 *c_data = (GstCollectData2 *) walk->data;
     MpegPsPadData *ps_data = (MpegPsPadData *) walk->data;
 
     if (ps_data->eos == FALSE) {
@@ -408,7 +408,7 @@ mpegpsmux_choose_best_stream (MpegPsMux * mux)
         GstBuffer *buf;
 
         ps_data->queued_buf = buf =
-            gst_collect_pads_peek (mux->collect, c_data);
+            gst_collect_pads2_peek (mux->collect, c_data);
 
         if (buf != NULL) {
           if (ps_data->prepare_func) {
@@ -469,7 +469,7 @@ mpegpsmux_choose_best_stream (MpegPsMux * mux)
     }
   }
   if (c_best) {
-    gst_buffer_unref (gst_collect_pads_pop (mux->collect, c_best));
+    gst_buffer_unref (gst_collect_pads2_pop (mux->collect, c_best));
   }
 
   return best;
@@ -490,7 +490,7 @@ mpegpsmux_push_gop_list (MpegPsMux * mux)
 }
 
 static GstFlowReturn
-mpegpsmux_collected (GstCollectPads * pads, MpegPsMux * mux)
+mpegpsmux_collected (GstCollectPads2 * pads, MpegPsMux * mux)
 {
   /* main muxing function */
 
@@ -597,7 +597,7 @@ mpegpsmux_request_new_pad (GstElement * element,
 
   pad = gst_pad_new_from_template (templ, name);
 
-  pad_data = (MpegPsPadData *) gst_collect_pads_add_pad (mux->collect, pad,
+  pad_data = (MpegPsPadData *) gst_collect_pads2_add_pad (mux->collect, pad,
       sizeof (MpegPsPadData));
   if (pad_data == NULL)
     goto pad_failure;
@@ -614,7 +614,7 @@ mpegpsmux_request_new_pad (GstElement * element,
 could_not_add:
   GST_ELEMENT_ERROR (element, STREAM, FAILED,
       ("Internal data stream error."), ("Could not add pad to element"));
-  gst_collect_pads_remove_pad (mux->collect, pad);
+  gst_collect_pads2_remove_pad (mux->collect, pad);
   gst_object_unref (pad);
   return NULL;
 pad_failure:
@@ -649,7 +649,7 @@ mpegpsmux_release_pad (GstElement * element, GstPad * pad)
     mux->video_stream_id = 0;
   GST_OBJECT_UNLOCK (pad);
 
-  gst_collect_pads_remove_pad (mux->collect, pad);
+  gst_collect_pads2_remove_pad (mux->collect, pad);
 }
 
 static void
@@ -763,12 +763,12 @@ mpegpsmux_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_collect_pads_start (mux->collect);
+      gst_collect_pads2_start (mux->collect);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_collect_pads_stop (mux->collect);
+      gst_collect_pads2_stop (mux->collect);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -800,5 +800,5 @@ plugin_init (GstPlugin * plugin)
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
-    "mpegpsmux", "MPEG-PS muxer",
+    mpegpsmux, "MPEG-PS muxer",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
