@@ -88,6 +88,10 @@
 #include <stdio.h>
 #include <string.h>
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <gst/video/video.h>
 
 #include "mpegtsmux.h"
@@ -108,7 +112,7 @@ enum
 };
 
 static GstStaticPadTemplate mpegtsmux_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("sink_%d",
+    GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/mpeg, "
@@ -147,7 +151,7 @@ static gboolean new_packet_cb (guint8 * data, guint len, void *user_data,
 static void release_buffer_cb (guint8 * data, void *user_data);
 
 static void mpegtsdemux_prepare_srcpad (MpegTsMux * mux);
-static GstFlowReturn mpegtsmux_collected (GstCollectPads * pads,
+static GstFlowReturn mpegtsmux_collected (GstCollectPads2 * pads,
     MpegTsMux * mux);
 static GstPad *mpegtsmux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
@@ -166,10 +170,10 @@ mpegtsmux_base_init (gpointer g_class)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegtsmux_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegtsmux_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegtsmux_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegtsmux_src_factory));
 
   gst_element_class_set_details_simple (element_class,
       "MPEG Transport Stream Muxer", "Codec/Muxer",
@@ -224,9 +228,9 @@ mpegtsmux_init (MpegTsMux * mux, MpegTsMuxClass * g_class)
   gst_pad_set_event_function (mux->srcpad, mpegtsmux_src_event);
   gst_element_add_pad (GST_ELEMENT (mux), mux->srcpad);
 
-  mux->collect = gst_collect_pads_new ();
-  gst_collect_pads_set_function (mux->collect,
-      (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (mpegtsmux_collected), mux);
+  mux->collect = gst_collect_pads2_new ();
+  gst_collect_pads2_set_function (mux->collect,
+      (GstCollectPads2Function) GST_DEBUG_FUNCPTR (mpegtsmux_collected), mux);
 
   mux->tsmux = tsmux_new ();
   tsmux_set_write_func (mux->tsmux, new_packet_cb, mux);
@@ -518,7 +522,7 @@ mpegtsmux_create_streams (MpegTsMux * mux)
 
   /* Create the streams */
   while (walk) {
-    GstCollectData *c_data = (GstCollectData *) walk->data;
+    GstCollectData2 *c_data = (GstCollectData2 *) walk->data;
     MpegTsPadData *ts_data = (MpegTsPadData *) walk->data;
     gchar *name = NULL;
 
@@ -578,11 +582,11 @@ static MpegTsPadData *
 mpegtsmux_choose_best_stream (MpegTsMux * mux)
 {
   MpegTsPadData *best = NULL;
-  GstCollectData *c_best = NULL;
+  GstCollectData2 *c_best = NULL;
   GSList *walk;
 
   for (walk = mux->collect->data; walk != NULL; walk = g_slist_next (walk)) {
-    GstCollectData *c_data = (GstCollectData *) walk->data;
+    GstCollectData2 *c_data = (GstCollectData2 *) walk->data;
     MpegTsPadData *ts_data = (MpegTsPadData *) walk->data;
 
     if (ts_data->eos == FALSE) {
@@ -590,7 +594,7 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
         GstBuffer *buf;
 
         ts_data->queued_buf = buf =
-            gst_collect_pads_peek (mux->collect, c_data);
+            gst_collect_pads2_peek (mux->collect, c_data);
 
         if (buf != NULL) {
           if (ts_data->prepare_func) {
@@ -652,35 +656,14 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
   }
   if (c_best) {
     GstBuffer *buffer;
-    if ((buffer = gst_collect_pads_pop (mux->collect, c_best)))
+    if ((buffer = gst_collect_pads2_pop (mux->collect, c_best)))
       gst_buffer_unref (buffer);
   }
 
   return best;
 }
 
-#define COLLECT_DATA_PAD(collect_data) (((GstCollectData *)(collect_data))->pad)
-
-static MpegTsPadData *
-find_pad_data (MpegTsMux * mux, GstPad * pad)
-{
-  GSList *walk;
-  MpegTsPadData *ts_data = NULL;
-
-  GST_COLLECT_PADS_PAD_LOCK (mux->collect);
-  walk = mux->collect->abidata.ABI.pad_list;
-  while (walk) {
-    if (((GstCollectData *) walk->data)->pad == pad) {
-      ts_data = (MpegTsPadData *) walk->data;
-      break;
-    }
-
-    walk = g_slist_next (walk);
-  }
-  GST_COLLECT_PADS_PAD_UNLOCK (mux->collect);
-
-  return ts_data;
-}
+#define COLLECT_DATA_PAD(collect_data) (((GstCollectData2 *)(collect_data))->pad)
 
 static gboolean
 mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
@@ -690,7 +673,7 @@ mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   gboolean forward = TRUE;
 
-  ts_data = find_pad_data (mux, pad);
+  ts_data = (MpegTsPadData *) gst_pad_get_element_private (pad);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_CUSTOM_DOWNSTREAM:
@@ -857,7 +840,7 @@ out:
 }
 
 static GstFlowReturn
-mpegtsmux_collected (GstCollectPads * pads, MpegTsMux * mux)
+mpegtsmux_collected (GstCollectPads2 * pads, MpegTsMux * mux)
 {
   GstFlowReturn ret = GST_FLOW_OK;
   MpegTsPadData *best = NULL;
@@ -1000,7 +983,7 @@ mpegtsmux_request_new_pad (GstElement * element,
   pad = gst_pad_new_from_template (templ, pad_name);
   g_free (pad_name);
 
-  pad_data = (MpegTsPadData *) gst_collect_pads_add_pad (mux->collect, pad,
+  pad_data = (MpegTsPadData *) gst_collect_pads2_add_pad (mux->collect, pad,
       sizeof (MpegTsPadData));
   if (pad_data == NULL)
     goto pad_failure;
@@ -1029,7 +1012,7 @@ stream_exists:
 could_not_add:
   GST_ELEMENT_ERROR (element, STREAM, FAILED,
       ("Internal data stream error."), ("Could not add pad to element"));
-  gst_collect_pads_remove_pad (mux->collect, pad);
+  gst_collect_pads2_remove_pad (mux->collect, pad);
   gst_object_unref (pad);
   return NULL;
 pad_failure:
@@ -1047,7 +1030,7 @@ mpegtsmux_release_pad (GstElement * element, GstPad * pad)
   GST_DEBUG_OBJECT (mux, "Pad %" GST_PTR_FORMAT " being released", pad);
 
   if (mux->collect) {
-    gst_collect_pads_remove_pad (mux->collect, pad);
+    gst_collect_pads2_remove_pad (mux->collect, pad);
   }
 
   /* chain up */
@@ -1104,11 +1087,11 @@ new_packet_m2ts (MpegTsMux * mux, guint8 * data, guint len, gint64 new_pcr)
     return FALSE;
   }
 
-  new_packet_common_init (mux, buf, data, len);
-
   /* copies the TS data of 188 bytes to the m2ts buffer at an offset
      of 4 bytes to leave space for writing the timestamp later */
   memcpy (GST_BUFFER_DATA (buf) + 4, data, len);
+  /* After copying the data into the buffer, do other common init (flags and streamheaders) */
+  new_packet_common_init (mux, buf, data, len);
 
   if (new_pcr < 0) {
     /* If theres no pcr in current ts packet then just add the packet
@@ -1215,9 +1198,10 @@ new_packet_normal_ts (MpegTsMux * mux, guint8 * data, guint len, gint64 new_pcr)
     return FALSE;
   }
 
+  memcpy (GST_BUFFER_DATA (buf), data, len);
+  /* After copying the data into the buffer, do other common init (flags and streamheaders) */
   new_packet_common_init (mux, buf, data, len);
 
-  memcpy (GST_BUFFER_DATA (buf), data, len);
   GST_BUFFER_TIMESTAMP (buf) = mux->last_ts;
 
   ret = gst_pad_push (mux->srcpad, buf);
@@ -1308,12 +1292,12 @@ mpegtsmux_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_collect_pads_start (mux->collect);
+      gst_collect_pads2_start (mux->collect);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_collect_pads_stop (mux->collect);
+      gst_collect_pads2_stop (mux->collect);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       if (mux->adapter)
@@ -1347,5 +1331,5 @@ plugin_init (GstPlugin * plugin)
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
-    "mpegtsmux", "MPEG-TS muxer",
+    mpegtsmux, "MPEG-TS muxer",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
