@@ -141,8 +141,10 @@ gst_nsfdec_base_init (gpointer g_class)
       "Using nosefart to decode NSF audio tunes",
       "Johan Dahlin <johan@gnome.org>");
 
-  gst_element_class_add_static_pad_template (element_class, &src_templ);
-  gst_element_class_add_static_pad_template (element_class, &sink_templ);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_templ));
 }
 
 static void
@@ -449,7 +451,7 @@ gst_nsfdec_chain (GstPad * pad, GstBuffer * buffer)
   /* collect all data, we start doing something when we get an EOS
    * event */
   if (nsfdec->tune_buffer) {
-    nsfdec->tune_buffer = gst_buffer_join (nsfdec->tune_buffer, buffer);
+    nsfdec->tune_buffer = gst_buffer_append (nsfdec->tune_buffer, buffer);
   } else {
     nsfdec->tune_buffer = buffer;
   }
@@ -637,6 +639,6 @@ plugin_init (GstPlugin * plugin)
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
-    "nsf",
+    nsf,
     "Uses nosefart to decode .nsf files",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
