@@ -78,7 +78,7 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
   "depth = (int) 8 "
 
 static GstStaticPadTemplate gst_ssim_src_template =
-GST_STATIC_PAD_TEMPLATE ("src%d",
+GST_STATIC_PAD_TEMPLATE ("src_%u",
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,
     GST_STATIC_CAPS (SRC_CAPS)
@@ -92,7 +92,7 @@ GST_STATIC_PAD_TEMPLATE ("original",
     );
 
 static GstStaticPadTemplate gst_ssim_sink_modified_template =
-GST_STATIC_PAD_TEMPLATE ("modified%d",
+GST_STATIC_PAD_TEMPLATE ("modified_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS (SINK_CAPS)
@@ -114,7 +114,7 @@ static void gst_ssim_release_pad (GstElement * element, GstPad * pad);
 static GstStateChangeReturn gst_ssim_change_state (GstElement * element,
     GstStateChange transition);
 
-static GstFlowReturn gst_ssim_collected (GstCollectPads * pads,
+static GstFlowReturn gst_ssim_collected (GstCollectPads2 * pads,
     gpointer user_data);
 
 static GstElementClass *parent_class = NULL;
@@ -901,7 +901,7 @@ gst_ssim_src_event (GstPad * pad, GstEvent * event)
       /* check if we are flushing */
       if (flags & GST_SEEK_FLAG_FLUSH) {
         /* make sure we accept nothing anymore and return WRONG_STATE */
-        gst_collect_pads_set_flushing (ssim->collect, TRUE);
+        gst_collect_pads2_set_flushing (ssim->collect, TRUE);
 
         /* flushing seek, start flush downstream, the flush will be done
          * when all pads received a FLUSH_STOP. */
@@ -991,7 +991,7 @@ gst_ssim_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-  /* now GstCollectPads can take care of the rest, e.g. EOS */
+  /* now GstCollectPads2 can take care of the rest, e.g. EOS */
   GST_DEBUG ("Dispatching %s event on pad %s:%s", GST_EVENT_TYPE_NAME (event),
       GST_DEBUG_PAD_NAME (pad));
   ret = ssim->collect_event (pad, event);
@@ -1095,12 +1095,12 @@ gst_ssim_class_init (GstSSimClass * klass)
           "(only when using Gaussian window).",
           G_MINFLOAT, 10, 1.5, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_ssim_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_ssim_sink_original_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_ssim_sink_modified_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_ssim_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_ssim_sink_original_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_ssim_sink_modified_template));
   gst_element_class_set_details_simple (gstelement_class, "SSim",
       "Filter/Analyzer/Video",
       "Calculate Y-SSIM for n+2 YUV video streams",
@@ -1125,7 +1125,7 @@ gst_ssim_request_new_pad (GstElement * element, GstPadTemplate * templ,
   GstPad *newsrc;
   gint padcount;
   GstPadTemplate *template;
-  gint num = -1;
+  guint num = -1;
 
   if (templ->direction != GST_PAD_SINK)
     goto not_sink;
@@ -1145,9 +1145,9 @@ gst_ssim_request_new_pad (GstElement * element, GstPadTemplate * templ,
     newpad = gst_pad_new_from_template (templ, "original");
     GST_DEBUG_OBJECT (ssim, "request new sink pad original");
     ssim->orig = newpad;
-  } else if (strncmp (padname, "modified", 8) == 0) {
-    const gchar *numstr = &padname[8];
-    num = strtol (numstr, NULL, 10);
+  } else if (strncmp (padname, "modified_", 9) == 0) {
+    const gchar *numstr = &padname[9];
+    num = strtoul (numstr, NULL, 10);
     if (errno == EINVAL || errno == ERANGE)
       goto bad_name;
     newpad = gst_pad_new_from_template (templ, padname);
@@ -1158,10 +1158,10 @@ gst_ssim_request_new_pad (GstElement * element, GstPadTemplate * templ,
   gst_pad_set_getcaps_function (newpad,
       GST_DEBUG_FUNCPTR (gst_ssim_sink_getcaps));
   gst_pad_set_setcaps_function (newpad, GST_DEBUG_FUNCPTR (gst_ssim_setcaps));
-  gst_collect_pads_add_pad (ssim->collect, newpad, sizeof (GstCollectData));
+  gst_collect_pads2_add_pad (ssim->collect, newpad, sizeof (GstCollectData2));
 
   /* FIXME: hacked way to override/extend the event function of
-   * GstCollectPads; because it sets its own event function giving the
+   * GstCollectPads2; because it sets its own event function giving the
    * element no access to events
    */
   GST_DEBUG_OBJECT (ssim, "Current collect_event is %p, changing to %p",
@@ -1181,11 +1181,11 @@ gst_ssim_request_new_pad (GstElement * element, GstPadTemplate * templ,
     padcount = g_atomic_int_exchange_and_add (&ssim->padcount, 1);
 #endif
 
-  if (num >= 0) {
+  if (num != -1) {
     GstSSimOutputContext *c;
 
     template = gst_static_pad_template_get (&gst_ssim_src_template);
-    name = g_strdup_printf ("src%d", num);
+    name = g_strdup_printf ("src_%u", num);
     newsrc = gst_pad_new_from_template (template, name);
     GST_DEBUG_OBJECT (ssim, "creating src pad %s", name);
     g_free (name);
@@ -1233,7 +1233,7 @@ could_not_add_src:
 could_not_add_sink:
   {
     GST_DEBUG_OBJECT (ssim, "could not add sink pad");
-    gst_collect_pads_remove_pad (ssim->collect, newpad);
+    gst_collect_pads2_remove_pad (ssim->collect, newpad);
     gst_object_unref (newpad);
     return NULL;
   }
@@ -1248,7 +1248,7 @@ gst_ssim_release_pad (GstElement * element, GstPad * pad)
 
   GST_DEBUG_OBJECT (ssim, "release pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
-  gst_collect_pads_remove_pad (ssim->collect, pad);
+  gst_collect_pads2_remove_pad (ssim->collect, pad);
   gst_element_remove_pad (element, pad);
 }
 
@@ -1267,8 +1267,8 @@ gst_ssim_init (GstSSim * ssim)
   ssim->sinkcaps = NULL;
 
   /* keep track of the sinkpads requested */
-  ssim->collect = gst_collect_pads_new ();
-  gst_collect_pads_set_function (ssim->collect,
+  ssim->collect = gst_collect_pads2_new ();
+  gst_collect_pads2_set_function (ssim->collect,
       GST_DEBUG_FUNCPTR (gst_ssim_collected), ssim);
 }
 
@@ -1407,7 +1407,7 @@ gst_ssim_regenerate_windows (GstSSim * ssim)
 }
 
 static GstFlowReturn
-gst_ssim_collected (GstCollectPads * pads, gpointer user_data)
+gst_ssim_collected (GstCollectPads2 * pads, gpointer user_data)
 {
   GstSSim *ssim;
   GSList *collected;
@@ -1440,12 +1440,12 @@ gst_ssim_collected (GstCollectPads * pads, gpointer user_data)
   }
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
-    GstCollectData *collect_data;
+    GstCollectData2 *collect_data;
     GstBuffer *inbuf;
 
-    collect_data = (GstCollectData *) collected->data;
+    collect_data = (GstCollectData2 *) collected->data;
 
-    inbuf = gst_collect_pads_peek (pads, collect_data);
+    inbuf = gst_collect_pads2_peek (pads, collect_data);
 
     if (inbuf == NULL) {
       GST_LOG_OBJECT (ssim, "channel %p: no bytes available", collect_data);
@@ -1469,12 +1469,12 @@ gst_ssim_collected (GstCollectPads * pads, gpointer user_data)
 
     for (collected = pads->data; collected;
         collected = g_slist_next (collected)) {
-      GstCollectData *collect_data;
+      GstCollectData2 *collect_data;
 
-      collect_data = (GstCollectData *) collected->data;
+      collect_data = (GstCollectData2 *) collected->data;
 
       if (collect_data->pad == ssim->orig) {
-        orgbuf = gst_collect_pads_pop (pads, collect_data);;
+        orgbuf = gst_collect_pads2_pop (pads, collect_data);;
 
         GST_DEBUG_OBJECT (ssim, "Original stream - flags(0x%x), timestamp(%"
             GST_TIME_FORMAT "), duration(%" GST_TIME_FORMAT ")",
@@ -1491,14 +1491,14 @@ gst_ssim_collected (GstCollectPads * pads, gpointer user_data)
   GST_LOG_OBJECT (ssim, "starting to cycle through streams");
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
-    GstCollectData *collect_data;
+    GstCollectData2 *collect_data;
     GstBuffer *inbuf;
     guint8 *indata;
 
-    collect_data = (GstCollectData *) collected->data;
+    collect_data = (GstCollectData2 *) collected->data;
 
     if (collect_data->pad != ssim->orig) {
-      inbuf = gst_collect_pads_pop (pads, collect_data);
+      inbuf = gst_collect_pads2_pop (pads, collect_data);
 
       indata = GST_BUFFER_DATA (inbuf);
 
@@ -1659,7 +1659,7 @@ gst_ssim_change_state (GstElement * element, GstStateChange transition)
       ssim->segment_position = 0;
       ssim->segment_rate = 1.0;
       gst_segment_init (&ssim->segment, GST_FORMAT_UNDEFINED);
-      gst_collect_pads_start (ssim->collect);
+      gst_collect_pads2_start (ssim->collect);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1667,7 +1667,7 @@ gst_ssim_change_state (GstElement * element, GstStateChange transition)
       /* need to unblock the collectpads before calling the
        * parent change_state so that streaming can finish
        */
-      gst_collect_pads_stop (ssim->collect);
+      gst_collect_pads2_stop (ssim->collect);
       break;
     default:
       break;
