@@ -54,17 +54,13 @@ static gboolean gst_dirac_parse_start (GstBaseParse * parse);
 static gboolean gst_dirac_parse_stop (GstBaseParse * parse);
 static gboolean gst_dirac_parse_set_sink_caps (GstBaseParse * parse,
     GstCaps * caps);
-static GstCaps *gst_dirac_parse_get_sink_caps (GstBaseParse * parse);
-static gboolean gst_dirac_parse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize);
-static GstFlowReturn gst_dirac_parse_parse_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame);
+static GstCaps *gst_dirac_parse_get_sink_caps (GstBaseParse * parse,
+    GstCaps * filter);
+static GstFlowReturn gst_dirac_parse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize);
 static gboolean gst_dirac_parse_convert (GstBaseParse * parse,
     GstFormat src_format, gint64 src_value, GstFormat dest_format,
     gint64 * dest_value);
-static gboolean gst_dirac_parse_event (GstBaseParse * parse, GstEvent * event);
-static gboolean gst_dirac_parse_src_event (GstBaseParse * parse,
-    GstEvent * event);
 static GstFlowReturn gst_dirac_parse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 
@@ -96,58 +92,48 @@ GST_STATIC_PAD_TEMPLATE ("src",
 
 /* class initialization */
 
-GST_BOILERPLATE (GstDiracParse, gst_dirac_parse, GstBaseParse,
-    GST_TYPE_BASE_PARSE);
-
-static void
-gst_dirac_parse_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_dirac_parse_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_dirac_parse_sink_template);
-
-  gst_element_class_set_details_simple (element_class, "Dirac parser",
-      "Codec/Parser/Video", "Parses Dirac streams",
-      "David Schleef <ds@schleef.org>");
-}
+#define parent_class gst_dirac_parse_parent_class
+G_DEFINE_TYPE (GstDiracParse, gst_dirac_parse, GST_TYPE_BASE_PARSE);
 
 static void
 gst_dirac_parse_class_init (GstDiracParseClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstBaseParseClass *base_parse_class = GST_BASE_PARSE_CLASS (klass);
 
   gobject_class->set_property = gst_dirac_parse_set_property;
   gobject_class->get_property = gst_dirac_parse_get_property;
   gobject_class->dispose = gst_dirac_parse_dispose;
   gobject_class->finalize = gst_dirac_parse_finalize;
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_dirac_parse_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_dirac_parse_sink_template));
+
+  gst_element_class_set_details_simple (element_class, "Dirac parser",
+      "Codec/Parser/Video", "Parses Dirac streams",
+      "David Schleef <ds@schleef.org>");
+
   base_parse_class->start = GST_DEBUG_FUNCPTR (gst_dirac_parse_start);
   base_parse_class->stop = GST_DEBUG_FUNCPTR (gst_dirac_parse_stop);
   base_parse_class->set_sink_caps =
       GST_DEBUG_FUNCPTR (gst_dirac_parse_set_sink_caps);
   base_parse_class->get_sink_caps =
       GST_DEBUG_FUNCPTR (gst_dirac_parse_get_sink_caps);
-  base_parse_class->check_valid_frame =
-      GST_DEBUG_FUNCPTR (gst_dirac_parse_check_valid_frame);
-  base_parse_class->parse_frame =
-      GST_DEBUG_FUNCPTR (gst_dirac_parse_parse_frame);
+  base_parse_class->handle_frame =
+      GST_DEBUG_FUNCPTR (gst_dirac_parse_handle_frame);
   base_parse_class->convert = GST_DEBUG_FUNCPTR (gst_dirac_parse_convert);
-  base_parse_class->event = GST_DEBUG_FUNCPTR (gst_dirac_parse_event);
-  base_parse_class->src_event = GST_DEBUG_FUNCPTR (gst_dirac_parse_src_event);
   base_parse_class->pre_push_frame =
       GST_DEBUG_FUNCPTR (gst_dirac_parse_pre_push_frame);
 
 }
 
 static void
-gst_dirac_parse_init (GstDiracParse * diracparse,
-    GstDiracParseClass * diracparse_class)
+gst_dirac_parse_init (GstDiracParse * diracparse)
 {
   gst_base_parse_set_min_frame_size (GST_BASE_PARSE (diracparse), 13);
-
 }
 
 void
@@ -218,41 +204,48 @@ gst_dirac_parse_set_sink_caps (GstBaseParse * parse, GstCaps * caps)
   return TRUE;
 }
 
-static gboolean
-gst_dirac_parse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize)
+static GstFlowReturn
+gst_dirac_parse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize)
 {
-  GstByteReader reader = GST_BYTE_READER_INIT_FROM_BUFFER (frame->buffer);
   int off;
   guint32 next_header;
+  GstMapInfo map;
   guint8 *data;
-  int size;
+  gsize size;
   gboolean have_picture = FALSE;
   int offset;
+  guint framesize = 0;
 
-  data = GST_BUFFER_DATA (frame->buffer);
-  size = GST_BUFFER_SIZE (frame->buffer);
+  gst_buffer_map (frame->buffer, &map, GST_MAP_READ);
+  data = map.data;
+  size = map.size;
 
-  if (G_UNLIKELY (size < 13))
-    return FALSE;
+  if (G_UNLIKELY (size < 13)) {
+    *skipsize = 1;
+    goto out;
+  }
 
-  GST_DEBUG ("%d: %02x %02x %02x %02x", size, data[0], data[1], data[2],
-      data[3]);
+  GST_DEBUG ("%" G_GSIZE_FORMAT ": %02x %02x %02x %02x", size, data[0], data[1],
+      data[2], data[3]);
 
   if (GST_READ_UINT32_BE (data) != 0x42424344) {
+    GstByteReader reader;
+
+    gst_byte_reader_init (&reader, data, size);
     off = gst_byte_reader_masked_scan_uint32 (&reader, 0xffffffff,
-        0x42424344, 0, GST_BUFFER_SIZE (frame->buffer));
+        0x42424344, 0, size);
 
     if (off < 0) {
-      *skipsize = GST_BUFFER_SIZE (frame->buffer) - 3;
-      return FALSE;
+      *skipsize = size - 3;
+      goto out;
     }
 
     GST_LOG_OBJECT (parse, "possible sync at buffer offset %d", off);
 
     GST_DEBUG ("skipping %d", off);
     *skipsize = off;
-    return FALSE;
+    goto out;
   }
 
   /* have sync, parse chunks */
@@ -262,8 +255,8 @@ gst_dirac_parse_check_valid_frame (GstBaseParse * parse,
     GST_DEBUG ("offset %d:", offset);
 
     if (offset + 13 >= size) {
-      *framesize = offset + 13;
-      return FALSE;
+      framesize = offset + 13;
+      goto out;
     }
 
     GST_DEBUG ("chunk type %02x", data[offset + 4]);
@@ -271,7 +264,7 @@ gst_dirac_parse_check_valid_frame (GstBaseParse * parse,
     if (GST_READ_UINT32_BE (data + offset) != 0x42424344) {
       GST_DEBUG ("bad header");
       *skipsize = 3;
-      return FALSE;
+      goto out;
     }
 
     next_header = GST_READ_UINT32_BE (data + offset + 5);
@@ -285,35 +278,21 @@ gst_dirac_parse_check_valid_frame (GstBaseParse * parse,
 
     offset += next_header;
     if (offset >= size) {
-      *framesize = offset;
-      return FALSE;
+      framesize = offset;
+      goto out;
     }
   }
 
-  *framesize = offset;
-  GST_DEBUG ("framesize %d", *framesize);
+  gst_buffer_unmap (frame->buffer, &map);
 
-  return TRUE;
-}
+  framesize = offset;
+  GST_DEBUG ("framesize %d", framesize);
 
-static GstFlowReturn
-gst_dirac_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
-{
-  GstDiracParse *diracparse = GST_DIRAC_PARSE (parse);
-  guint8 *data;
-  int size;
-
-  /* Called when processing incoming buffers.  Function should parse
-     a checked frame. */
-  /* MUST implement */
-
-  data = GST_BUFFER_DATA (frame->buffer);
-  size = GST_BUFFER_SIZE (frame->buffer);
-
-  //GST_ERROR("got here %d", size);
+  g_assert (framesize <= size);
 
   if (data[4] == SCHRO_PARSE_CODE_SEQUENCE_HEADER) {
     GstCaps *caps;
+    GstDiracParse *diracparse = GST_DIRAC_PARSE (parse);
     DiracSequenceHeader sequence_header;
     int ret;
 
@@ -342,11 +321,14 @@ gst_dirac_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     }
   }
 
-  gst_buffer_set_caps (frame->buffer,
-      GST_PAD_CAPS (GST_BASE_PARSE_SRC_PAD (parse)));
-
   gst_base_parse_set_min_frame_size (parse, 13);
 
+  return gst_base_parse_finish_frame (parse, frame, framesize);
+
+out:
+  gst_buffer_unmap (frame->buffer, &map);
+  if (framesize)
+    gst_base_parse_set_min_frame_size (parse, framesize);
   return GST_FLOW_OK;
 }
 
@@ -359,22 +341,6 @@ gst_dirac_parse_convert (GstBaseParse * parse, GstFormat src_format,
   return FALSE;
 }
 
-static gboolean
-gst_dirac_parse_event (GstBaseParse * parse, GstEvent * event)
-{
-  /* Sink pad event handler */
-
-  return FALSE;
-}
-
-static gboolean
-gst_dirac_parse_src_event (GstBaseParse * parse, GstEvent * event)
-{
-  /* Src pad event handler */
-
-  return FALSE;
-}
-
 static GstFlowReturn
 gst_dirac_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 {
@@ -383,7 +349,7 @@ gst_dirac_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 }
 
 static GstCaps *
-gst_dirac_parse_get_sink_caps (GstBaseParse * parse)
+gst_dirac_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 {
   GstCaps *peercaps;
   GstCaps *res;
