@@ -31,6 +31,7 @@
 #include <string.h>
 #include <gst/base/gstbytereader.h>
 #include <gst/pbutils/codec-utils.h>
+#include <gst/video/video.h>
 
 #include "gstmpeg4videoparse.h"
 
@@ -64,41 +65,28 @@ enum
   PROP_LAST
 };
 
-GST_BOILERPLATE (GstMpeg4VParse, gst_mpeg4vparse, GstBaseParse,
-    GST_TYPE_BASE_PARSE);
+#define gst_mpeg4vparse_parent_class parent_class
+G_DEFINE_TYPE (GstMpeg4VParse, gst_mpeg4vparse, GST_TYPE_BASE_PARSE);
 
 static gboolean gst_mpeg4vparse_start (GstBaseParse * parse);
 static gboolean gst_mpeg4vparse_stop (GstBaseParse * parse);
-static gboolean gst_mpeg4vparse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize);
+static GstFlowReturn gst_mpeg4vparse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize);
 static GstFlowReturn gst_mpeg4vparse_parse_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 static GstFlowReturn gst_mpeg4vparse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 static gboolean gst_mpeg4vparse_set_caps (GstBaseParse * parse, GstCaps * caps);
-static GstCaps *gst_mpeg4vparse_get_caps (GstBaseParse * parse);
+static GstCaps *gst_mpeg4vparse_get_caps (GstBaseParse * parse,
+    GstCaps * filter);
 
 static void gst_mpeg4vparse_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_mpeg4vparse_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-
-static void
-gst_mpeg4vparse_base_init (gpointer klass)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
-
-  gst_element_class_set_details_simple (element_class,
-      "MPEG 4 video elementary stream parser", "Codec/Parser/Video",
-      "Parses MPEG-4 Part 2 elementary video streams",
-      "Julien Moutte <julien@fluendo.com>");
-
-  GST_DEBUG_CATEGORY_INIT (mpeg4v_parse_debug, "mpeg4videoparse", 0,
-      "MPEG-4 video parser");
-}
+static gboolean gst_mpeg4vparse_event (GstBaseParse * parse, GstEvent * event);
+static gboolean gst_mpeg4vparse_src_event (GstBaseParse * parse,
+    GstEvent * event);
 
 static void
 gst_mpeg4vparse_set_property (GObject * object, guint property_id,
@@ -141,6 +129,7 @@ gst_mpeg4vparse_class_init (GstMpeg4VParseClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstBaseParseClass *parse_class = GST_BASE_PARSE_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -161,20 +150,33 @@ gst_mpeg4vparse_class_init (GstMpeg4VParseClass * klass)
           0, 3600, DEFAULT_CONFIG_INTERVAL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+
+  gst_element_class_set_details_simple (element_class,
+      "MPEG 4 video elementary stream parser", "Codec/Parser/Video",
+      "Parses MPEG-4 Part 2 elementary video streams",
+      "Julien Moutte <julien@fluendo.com>");
+
+  GST_DEBUG_CATEGORY_INIT (mpeg4v_parse_debug, "mpeg4videoparse", 0,
+      "MPEG-4 video parser");
+
   /* Override BaseParse vfuncs */
   parse_class->start = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_start);
   parse_class->stop = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_stop);
-  parse_class->check_valid_frame =
-      GST_DEBUG_FUNCPTR (gst_mpeg4vparse_check_valid_frame);
-  parse_class->parse_frame = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_parse_frame);
+  parse_class->handle_frame = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_handle_frame);
   parse_class->pre_push_frame =
       GST_DEBUG_FUNCPTR (gst_mpeg4vparse_pre_push_frame);
   parse_class->set_sink_caps = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_set_caps);
   parse_class->get_sink_caps = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_get_caps);
+  parse_class->sink_event = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_event);
+  parse_class->src_event = GST_DEBUG_FUNCPTR (gst_mpeg4vparse_src_event);
 }
 
 static void
-gst_mpeg4vparse_init (GstMpeg4VParse * parse, GstMpeg4VParseClass * g_class)
+gst_mpeg4vparse_init (GstMpeg4VParse * parse)
 {
   parse->interval = DEFAULT_CONFIG_INTERVAL;
   parse->last_report = GST_CLOCK_TIME_NONE;
@@ -197,6 +199,8 @@ gst_mpeg4vparse_reset (GstMpeg4VParse * mp4vparse)
   mp4vparse->update_caps = TRUE;
   mp4vparse->profile = NULL;
   mp4vparse->level = NULL;
+  mp4vparse->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
+  mp4vparse->force_key_unit_event = NULL;
 
   gst_buffer_replace (&mp4vparse->config, NULL);
   memset (&mp4vparse->vol, 0, sizeof (mp4vparse->vol));
@@ -233,8 +237,8 @@ gst_mpeg4vparse_process_config (GstMpeg4VParse * mp4vparse,
     const guint8 * data, guint offset, gsize size)
 {
   /* only do stuff if something new */
-  if (mp4vparse->config && size == GST_BUFFER_SIZE (mp4vparse->config) &&
-      memcmp (GST_BUFFER_DATA (mp4vparse->config), data, size) == 0)
+  if (mp4vparse->config
+      && !gst_buffer_memcmp (mp4vparse->config, offset, data, size))
     return TRUE;
 
   if (mp4vparse->vol_offset < 0) {
@@ -264,10 +268,7 @@ gst_mpeg4vparse_process_config (GstMpeg4VParse * mp4vparse,
   if (mp4vparse->config != NULL)
     gst_buffer_unref (mp4vparse->config);
 
-  mp4vparse->config = gst_buffer_new_and_alloc (size);
-
-  memcpy (GST_BUFFER_DATA (mp4vparse->config), data, size);
-
+  mp4vparse->config = gst_buffer_new_wrapped (g_memdup (data, size), size);
 
   /* trigger src caps update */
   mp4vparse->update_caps = TRUE;
@@ -356,31 +357,34 @@ gst_mpeg4vparse_process_sc (GstMpeg4VParse * mp4vparse, GstMpeg4Packet * packet,
   return FALSE;
 }
 
-/* FIXME move into baseparse, or anything equivalent;
- * see https://bugzilla.gnome.org/show_bug.cgi?id=650093 */
-#define GST_BASE_PARSE_FRAME_FLAG_PARSING   0x10000
-
-static gboolean
-gst_mpeg4vparse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize)
+static GstFlowReturn
+gst_mpeg4vparse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize)
 {
   GstMpeg4VParse *mp4vparse = GST_MPEG4VIDEO_PARSE (parse);
   GstMpeg4Packet packet;
-  guint8 *data = GST_BUFFER_DATA (frame->buffer);
-  guint size = GST_BUFFER_SIZE (frame->buffer);
+  GstMapInfo map;
+  guint8 *data = NULL;
+  gsize size;
   gint off = 0;
-  gboolean ret;
+  gboolean ret = FALSE;
+  guint framesize;
+
+  gst_buffer_map (frame->buffer, &map, GST_MAP_READ);
+  data = map.data;
+  size = map.size;
 
 retry:
   /* at least start code and subsequent byte */
-  if (G_UNLIKELY (size - off < 5))
-    return FALSE;
+  if (G_UNLIKELY (size - off < 5)) {
+    *skipsize = 1;
+    goto out;
+  }
 
   /* avoid stale cached parsing state */
-  if (!(frame->flags & GST_BASE_PARSE_FRAME_FLAG_PARSING)) {
+  if (frame->flags & GST_BASE_PARSE_FRAME_FLAG_NEW_FRAME) {
     GST_LOG_OBJECT (mp4vparse, "parsing new frame");
     gst_mpeg4vparse_reset_frame (mp4vparse);
-    frame->flags |= GST_BASE_PARSE_FRAME_FLAG_PARSING;
   } else {
     GST_LOG_OBJECT (mp4vparse, "resuming frame parsing");
   }
@@ -396,7 +400,7 @@ retry:
     case (GST_MPEG4_PARSER_NO_PACKET):
     case (GST_MPEG4_PARSER_ERROR):
       *skipsize = size - 3;
-      return FALSE;
+      goto out;
     default:
       break;
   }
@@ -405,7 +409,7 @@ retry:
   /* possible frame header, but not at offset 0? skip bytes before sync */
   if (G_UNLIKELY (off > 3)) {
     *skipsize = off - 3;
-    return FALSE;
+    goto out;
   }
 
   switch (packet.type) {
@@ -445,15 +449,14 @@ next:
     case (GST_MPEG4_PARSER_ERROR):
       /* if draining, take all */
       if (GST_BASE_PARSE_DRAINING (parse)) {
-        *framesize = size;
-        return TRUE;
+        framesize = size;
+        ret = TRUE;
       } else {
         /* resume scan where we left it */
         mp4vparse->last_sc = size - 3;
-        /* request best next available */
-        *framesize = G_MAXUINT;
-        return FALSE;
       }
+      goto out;
+      break;
     default:
       /* decide whether this startcode ends a frame */
       ret = gst_mpeg4vparse_process_sc (mp4vparse, &packet, size);
@@ -463,12 +466,25 @@ next:
   off = packet.offset;
 
   if (ret) {
-    *framesize = off - 3;
+    framesize = off - 3;
   } else {
     goto next;
   }
 
-  return ret;
+out:
+  gst_buffer_unmap (frame->buffer, &map);
+
+  if (ret) {
+    GstFlowReturn res;
+
+    g_assert (framesize <= map.size);
+    res = gst_mpeg4vparse_parse_frame (parse, frame);
+    if (res == GST_BASE_PARSE_FLOW_DROPPED)
+      frame->flags |= GST_BASE_PARSE_FRAME_FLAG_DROP;
+    return gst_base_parse_finish_frame (parse, frame, framesize);
+  }
+
+  return GST_FLOW_OK;
 }
 
 static void
@@ -479,14 +495,16 @@ gst_mpeg4vparse_update_src_caps (GstMpeg4VParse * mp4vparse)
   GST_LOG_OBJECT (mp4vparse, "Updating caps");
 
   /* only update if no src caps yet or explicitly triggered */
-  if (G_LIKELY (GST_PAD_CAPS (GST_BASE_PARSE_SRC_PAD (mp4vparse)) &&
+  if (G_LIKELY (gst_pad_has_current_caps (GST_BASE_PARSE_SRC_PAD (mp4vparse)) &&
           !mp4vparse->update_caps))
     return;
 
   /* carry over input caps as much as possible; override with our own stuff */
-  caps = GST_PAD_CAPS (GST_BASE_PARSE_SINK_PAD (mp4vparse));
+  caps = gst_pad_get_current_caps (GST_BASE_PARSE_SINK_PAD (mp4vparse));
   if (caps) {
-    caps = gst_caps_copy (caps);
+    GstCaps *tmp = gst_caps_copy (caps);
+    gst_caps_unref (caps);
+    caps = tmp;
   } else {
     caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, 4, NULL);
@@ -532,6 +550,8 @@ gst_mpeg4vparse_update_src_caps (GstMpeg4VParse * mp4vparse)
 
   gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (mp4vparse), caps);
   gst_caps_unref (caps);
+
+  mp4vparse->update_caps = FALSE;
 }
 
 static GstFlowReturn
@@ -554,14 +574,89 @@ gst_mpeg4vparse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     return GST_FLOW_OK;
 }
 
+static GstEvent *
+check_pending_key_unit_event (GstEvent * pending_event, GstSegment * segment,
+    GstClockTime timestamp, guint flags, GstClockTime pending_key_unit_ts)
+{
+  GstClockTime running_time, stream_time;
+  gboolean all_headers;
+  guint count;
+  GstEvent *event = NULL;
+
+  g_return_val_if_fail (segment != NULL, NULL);
+
+  if (pending_event == NULL)
+    goto out;
+
+  if (GST_CLOCK_TIME_IS_VALID (pending_key_unit_ts) &&
+      timestamp == GST_CLOCK_TIME_NONE)
+    goto out;
+
+  running_time = gst_segment_to_running_time (segment,
+      GST_FORMAT_TIME, timestamp);
+
+  GST_INFO ("now %" GST_TIME_FORMAT " wanted %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (running_time), GST_TIME_ARGS (pending_key_unit_ts));
+  if (GST_CLOCK_TIME_IS_VALID (pending_key_unit_ts) &&
+      running_time < pending_key_unit_ts)
+    goto out;
+
+  if (flags & GST_BUFFER_FLAG_DELTA_UNIT) {
+    GST_DEBUG ("pending force key unit, waiting for keyframe");
+    goto out;
+  }
+
+  stream_time = gst_segment_to_stream_time (segment,
+      GST_FORMAT_TIME, timestamp);
+
+  gst_video_event_parse_upstream_force_key_unit (pending_event,
+      NULL, &all_headers, &count);
+
+  event =
+      gst_video_event_new_downstream_force_key_unit (timestamp, stream_time,
+      running_time, all_headers, count);
+  gst_event_set_seqnum (event, gst_event_get_seqnum (pending_event));
+
+out:
+  return event;
+}
+
+static void
+gst_mpeg4vparse_prepare_key_unit (GstMpeg4VParse * parse, GstEvent * event)
+{
+  GstClockTime running_time;
+  guint count;
+
+  parse->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
+  gst_event_replace (&parse->force_key_unit_event, NULL);
+
+  gst_video_event_parse_downstream_force_key_unit (event,
+      NULL, NULL, &running_time, NULL, &count);
+
+  GST_INFO_OBJECT (parse, "pushing downstream force-key-unit event %d "
+      "%" GST_TIME_FORMAT " count %d", gst_event_get_seqnum (event),
+      GST_TIME_ARGS (running_time), count);
+  gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (parse), event);
+}
+
+
 static GstFlowReturn
 gst_mpeg4vparse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 {
   GstMpeg4VParse *mp4vparse = GST_MPEG4VIDEO_PARSE (parse);
   GstBuffer *buffer = frame->buffer;
+  gboolean push_codec = FALSE;
+  GstEvent *event = NULL;
+
+  if ((event = check_pending_key_unit_event (mp4vparse->force_key_unit_event,
+              &parse->segment, GST_BUFFER_TIMESTAMP (buffer),
+              GST_BUFFER_FLAGS (buffer), mp4vparse->pending_key_unit_ts))) {
+    gst_mpeg4vparse_prepare_key_unit (mp4vparse, event);
+    push_codec = TRUE;
+  }
 
   /* periodic config sending */
-  if (mp4vparse->interval > 0) {
+  if (mp4vparse->interval > 0 || push_codec) {
     GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
     guint64 diff;
 
@@ -583,24 +678,33 @@ gst_mpeg4vparse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
       GST_LOG_OBJECT (mp4vparse,
           "interval since last config %" GST_TIME_FORMAT, GST_TIME_ARGS (diff));
 
-      if (GST_TIME_AS_SECONDS (diff) >= mp4vparse->interval) {
+      if (GST_TIME_AS_SECONDS (diff) >= mp4vparse->interval || push_codec) {
+        GstMapInfo cmap;
+        gsize csize;
+        gboolean diffconf;
+
         /* we need to send config now first */
-        GST_LOG_OBJECT (parse, "inserting config in stream");
+        GST_INFO_OBJECT (parse, "inserting config in stream");
+        gst_buffer_map (mp4vparse->config, &cmap, GST_MAP_READ);
+        diffconf = (gst_buffer_get_size (buffer) < cmap.size)
+            || gst_buffer_memcmp (buffer, 0, cmap.data, cmap.size);
+        csize = cmap.size;
+        gst_buffer_unmap (mp4vparse->config, &cmap);
 
         /* avoid inserting duplicate config */
-        if ((GST_BUFFER_SIZE (buffer) < GST_BUFFER_SIZE (mp4vparse->config)) ||
-            memcmp (GST_BUFFER_DATA (buffer),
-                GST_BUFFER_DATA (mp4vparse->config),
-                GST_BUFFER_SIZE (mp4vparse->config))) {
+        if (diffconf) {
           GstBuffer *superbuf;
 
           /* insert header */
-          superbuf = gst_buffer_merge (mp4vparse->config, buffer);
-          gst_buffer_copy_metadata (superbuf, buffer, GST_BUFFER_COPY_ALL);
-          gst_buffer_replace (&frame->buffer, superbuf);
+          superbuf =
+              gst_buffer_append (gst_buffer_ref (mp4vparse->config),
+              gst_buffer_ref (buffer));
+          gst_buffer_copy_into (superbuf, buffer, GST_BUFFER_COPY_METADATA, 0,
+              csize);
+          gst_buffer_replace (&frame->out_buffer, superbuf);
           gst_buffer_unref (superbuf);
         } else {
-          GST_LOG_OBJECT (parse, "... but avoiding duplication");
+          GST_INFO_OBJECT (parse, "... but avoiding duplication");
         }
 
         if (G_UNLIKELY (timestamp != -1)) {
@@ -620,6 +724,7 @@ gst_mpeg4vparse_set_caps (GstBaseParse * parse, GstCaps * caps)
   GstStructure *s;
   const GValue *value;
   GstBuffer *buf;
+  GstMapInfo map;
   guint8 *data;
   gsize size;
 
@@ -635,8 +740,9 @@ gst_mpeg4vparse_set_caps (GstBaseParse * parse, GstCaps * caps)
     /* best possible parse attempt,
      * src caps are based on sink caps so it will end up in there
      * whether sucessful or not */
-    data = GST_BUFFER_DATA (buf);
-    size = GST_BUFFER_SIZE (buf);
+    gst_buffer_map (buf, &map, GST_MAP_READ);
+    data = map.data;
+    size = map.size;
     res = gst_mpeg4_parse (&packet, TRUE, NULL, data, 0, size);
 
     while (res == GST_MPEG4_PARSER_OK || res == GST_MPEG4_PARSER_NO_PACKET_END) {
@@ -649,8 +755,8 @@ gst_mpeg4vparse_set_caps (GstBaseParse * parse, GstCaps * caps)
     }
 
     /* And take it as config */
-    gst_mpeg4vparse_process_config (mp4vparse, GST_BUFFER_DATA (buf),
-        3, GST_BUFFER_SIZE (buf));
+    gst_mpeg4vparse_process_config (mp4vparse, data, 3, size);
+    gst_buffer_unmap (buf, &map);
   }
 
   /* let's not interfere and accept regardless of config parsing success */
@@ -659,7 +765,7 @@ gst_mpeg4vparse_set_caps (GstBaseParse * parse, GstCaps * caps)
 
 
 static GstCaps *
-gst_mpeg4vparse_get_caps (GstBaseParse * parse)
+gst_mpeg4vparse_get_caps (GstBaseParse * parse, GstCaps * filter)
 {
   GstCaps *peercaps;
   GstCaps *res;
@@ -688,5 +794,94 @@ gst_mpeg4vparse_get_caps (GstBaseParse * parse)
             (parse)));
   }
 
+  if (filter) {
+    GstCaps *tmp = gst_caps_intersect_full (res, filter,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (res);
+    res = tmp;
+  }
+
+
+  return res;
+}
+
+static gboolean
+gst_mpeg4vparse_event (GstBaseParse * parse, GstEvent * event)
+{
+  gboolean res;
+  GstMpeg4VParse *mp4vparse = GST_MPEG4VIDEO_PARSE (parse);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    {
+      GstClockTime timestamp, stream_time, running_time;
+      gboolean all_headers;
+      guint count;
+
+      if (gst_video_event_is_force_key_unit (event)) {
+        gst_video_event_parse_downstream_force_key_unit (event,
+            &timestamp, &stream_time, &running_time, &all_headers, &count);
+
+        GST_INFO_OBJECT (mp4vparse, "received downstream force key unit event, "
+            "seqnum %d running_time %" GST_TIME_FORMAT
+            " all_headers %d count %d", gst_event_get_seqnum (event),
+            GST_TIME_ARGS (running_time), all_headers, count);
+
+        if (mp4vparse->force_key_unit_event) {
+          GST_INFO_OBJECT (mp4vparse, "ignoring force key unit event "
+              "as one is already queued");
+        } else {
+          mp4vparse->pending_key_unit_ts = running_time;
+          gst_event_replace (&mp4vparse->force_key_unit_event, event);
+        }
+        gst_event_unref (event);
+        res = TRUE;
+      } else {
+        res = GST_BASE_PARSE_CLASS (parent_class)->sink_event (parse, event);
+      }
+      break;
+    }
+    default:
+      res = GST_BASE_PARSE_CLASS (parent_class)->sink_event (parse, event);
+      break;
+  }
+  return res;
+}
+
+static gboolean
+gst_mpeg4vparse_src_event (GstBaseParse * parse, GstEvent * event)
+{
+  gboolean res;
+  GstMpeg4VParse *mp4vparse = GST_MPEG4VIDEO_PARSE (parse);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_UPSTREAM:
+    {
+      GstClockTime running_time;
+      gboolean all_headers;
+      guint count;
+
+      if (gst_video_event_is_force_key_unit (event)) {
+        gst_video_event_parse_upstream_force_key_unit (event,
+            &running_time, &all_headers, &count);
+
+        GST_INFO_OBJECT (mp4vparse, "received upstream force-key-unit event, "
+            "seqnum %d running_time %" GST_TIME_FORMAT
+            " all_headers %d count %d", gst_event_get_seqnum (event),
+            GST_TIME_ARGS (running_time), all_headers, count);
+
+        if (all_headers) {
+          mp4vparse->pending_key_unit_ts = running_time;
+          gst_event_replace (&mp4vparse->force_key_unit_event, event);
+        }
+      }
+      res = GST_BASE_PARSE_CLASS (parent_class)->src_event (parse, event);
+      break;
+    }
+    default:
+      res = GST_BASE_PARSE_CLASS (parent_class)->src_event (parse, event);
+      break;
+  }
+
   return res;
 }
