@@ -61,17 +61,18 @@ enum
   PROP_LAST
 };
 
-GST_BOILERPLATE (GstMpegvParse, gst_mpegv_parse, GstBaseParse,
-    GST_TYPE_BASE_PARSE);
+#define parent_class gst_mpegv_parse_parent_class
+G_DEFINE_TYPE (GstMpegvParse, gst_mpegv_parse, GST_TYPE_BASE_PARSE);
 
 static gboolean gst_mpegv_parse_start (GstBaseParse * parse);
 static gboolean gst_mpegv_parse_stop (GstBaseParse * parse);
-static gboolean gst_mpegv_parse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize);
+static GstFlowReturn gst_mpegv_parse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize);
 static GstFlowReturn gst_mpegv_parse_parse_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 static gboolean gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps);
-static GstCaps *gst_mpegv_parse_get_caps (GstBaseParse * parse);
+static GstCaps *gst_mpegv_parse_get_caps (GstBaseParse * parse,
+    GstCaps * filter);
 static GstFlowReturn gst_mpegv_parse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 
@@ -80,27 +81,6 @@ static void gst_mpegv_parse_set_property (GObject * object, guint prop_id,
 static void gst_mpegv_parse_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static void
-gst_mpegv_parse_base_init (gpointer klass)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
-
-  gst_element_class_set_details_simple (element_class,
-      "MPEG video elementary stream parser",
-      "Codec/Parser/Video",
-      "Parses and frames MPEG-1 and MPEG-2 elementary video streams",
-      "Wim Taymans <wim.taymans@ccollabora.co.uk>, "
-      "Jan Schmidt <thaytan@mad.scientist.com>, "
-      "Mark Nauwelaerts <mark.nauwelaerts@collabora.co.uk>");
-
-  GST_DEBUG_CATEGORY_INIT (mpegv_parse_debug, "mpegvideoparse", 0,
-      "MPEG-1/2 video parser");
-}
-
 static void
 gst_mpegv_parse_set_property (GObject * object, guint property_id,
     const GValue * value, GParamSpec * pspec)
@@ -141,8 +121,12 @@ static void
 gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstBaseParseClass *parse_class = GST_BASE_PARSE_CLASS (klass);
 
+  GST_DEBUG_CATEGORY_INIT (mpegv_parse_debug, "mpegvideoparse", 0,
+      "MPEG-1/2 video parser");
+
   parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property = gst_mpegv_parse_set_property;
@@ -159,12 +143,23 @@ gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
           "Split frame when encountering GOP", DEFAULT_PROP_GOP_SPLIT,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
+
+  gst_element_class_set_details_simple (element_class,
+      "MPEG video elementary stream parser",
+      "Codec/Parser/Video",
+      "Parses and frames MPEG-1 and MPEG-2 elementary video streams",
+      "Wim Taymans <wim.taymans@ccollabora.co.uk>, "
+      "Jan Schmidt <thaytan@mad.scientist.com>, "
+      "Mark Nauwelaerts <mark.nauwelaerts@collabora.co.uk>");
+
   /* Override BaseParse vfuncs */
   parse_class->start = GST_DEBUG_FUNCPTR (gst_mpegv_parse_start);
   parse_class->stop = GST_DEBUG_FUNCPTR (gst_mpegv_parse_stop);
-  parse_class->check_valid_frame =
-      GST_DEBUG_FUNCPTR (gst_mpegv_parse_check_valid_frame);
-  parse_class->parse_frame = GST_DEBUG_FUNCPTR (gst_mpegv_parse_parse_frame);
+  parse_class->handle_frame = GST_DEBUG_FUNCPTR (gst_mpegv_parse_handle_frame);
   parse_class->set_sink_caps = GST_DEBUG_FUNCPTR (gst_mpegv_parse_set_caps);
   parse_class->get_sink_caps = GST_DEBUG_FUNCPTR (gst_mpegv_parse_get_caps);
   parse_class->pre_push_frame =
@@ -172,7 +167,7 @@ gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
 }
 
 static void
-gst_mpegv_parse_init (GstMpegvParse * parse, GstMpegvParseClass * g_class)
+gst_mpegv_parse_init (GstMpegvParse * parse)
 {
   parse->mpeg_version = 0;
 }
@@ -232,16 +227,21 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
     guint size)
 {
   GList *tmp;
-  guint8 *data = GST_BUFFER_DATA (buf);
-  data = data + mpvparse->seq_offset;
+  GstMapInfo map;
+  guint8 *data;
+
+  gst_buffer_map (buf, &map, GST_MAP_READ);
+  data = map.data + mpvparse->seq_offset;
 
   /* only do stuff if something new */
-  if (mpvparse->config && size == GST_BUFFER_SIZE (mpvparse->config) &&
-      memcmp (GST_BUFFER_DATA (mpvparse->config), data, size) == 0)
+  if (mpvparse->config && size == gst_buffer_get_size (mpvparse->config) &&
+      gst_buffer_memcmp (mpvparse->config, 0, data, size) == 0) {
+    gst_buffer_unmap (buf, &map);
     return TRUE;
+  }
 
   if (gst_mpeg_video_parse_sequence_header (&mpvparse->sequencehdr, data,
-          GST_BUFFER_SIZE (buf) - mpvparse->seq_offset, 0)) {
+          map.size - mpvparse->seq_offset, 0)) {
     if (mpvparse->fps_num == 0 || mpvparse->fps_den == 0) {
       mpvparse->fps_num = mpvparse->sequencehdr.fps_n;
       mpvparse->fps_den = mpvparse->sequencehdr.fps_d;
@@ -250,6 +250,7 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
     GST_DEBUG_OBJECT (mpvparse,
         "failed to parse config data (size %d) at offset %d",
         size, mpvparse->seq_offset);
+    gst_buffer_unmap (buf, &map);
     return FALSE;
   }
 
@@ -267,8 +268,7 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
         mpvparse->mpeg_version = 2;
 
         if (gst_mpeg_video_parse_sequence_extension (&mpvparse->sequenceext,
-                GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf),
-                tpoffsz->offset)) {
+                map.data, map.size, tpoffsz->offset)) {
           mpvparse->fps_num =
               mpvparse->sequencehdr.fps_n * (mpvparse->sequenceext.fps_n_ext +
               1) * 2;
@@ -285,11 +285,13 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
     gst_buffer_unref (mpvparse->config);
 
   mpvparse->config = gst_buffer_new_and_alloc (size);
-  memcpy (GST_BUFFER_DATA (mpvparse->config), data, size);
+  gst_buffer_fill (mpvparse->config, 0, data, size);
 
   /* trigger src caps update */
   mpvparse->update_caps = TRUE;
 
+  gst_buffer_unmap (buf, &map);
+
   return TRUE;
 }
 
@@ -355,8 +357,11 @@ static void
 parse_picture_extension (GstMpegvParse * mpvparse, GstBuffer * buf, guint off)
 {
   GstMpegVideoPictureExt ext;
-  if (gst_mpeg_video_parse_picture_extension (&ext, GST_BUFFER_DATA (buf),
-          GST_BUFFER_SIZE (buf), off)) {
+  GstMapInfo map;
+
+  gst_buffer_map (buf, &map, GST_MAP_READ);
+
+  if (gst_mpeg_video_parse_picture_extension (&ext, map.data, map.size, off)) {
     mpvparse->frame_repeat_count = 1;
 
     if (ext.repeat_first_field) {
@@ -370,6 +375,8 @@ parse_picture_extension (GstMpegvParse * mpvparse, GstBuffer * buf, guint off)
       }
     }
   }
+
+  gst_buffer_unmap (buf, &map);
 }
 
 /* caller guarantees at least start code in @buf at @off */
@@ -381,7 +388,7 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
 {
   gboolean ret = FALSE, packet = TRUE;
 
-  g_return_val_if_fail (buf && GST_BUFFER_SIZE (buf) >= 4, FALSE);
+  g_return_val_if_fail (buf && gst_buffer_get_size (buf) >= 4, FALSE);
 
   GST_LOG_OBJECT (mpvparse, "process startcode %x (%s)", code,
       picture_start_code_name (code));
@@ -428,14 +435,19 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
 
   /* extract some picture info if there is any in the frame being terminated */
   if (ret && mpvparse->pic_offset >= 0 && mpvparse->pic_offset < off) {
+    GstMapInfo map;
+
+    gst_buffer_map (buf, &map, GST_MAP_READ);
     if (gst_mpeg_video_parse_picture_header (&mpvparse->pichdr,
-            GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf), mpvparse->pic_offset))
+            map.data, map.size, mpvparse->pic_offset))
       GST_LOG_OBJECT (mpvparse, "picture_coding_type %d (%s), ending"
           "frame of size %d", mpvparse->pichdr.pic_type,
           picture_type_name (mpvparse->pichdr.pic_type), off - 4);
     else
       GST_LOG_OBJECT (mpvparse, "Couldn't parse picture at offset %d",
           mpvparse->pic_offset);
+
+    gst_buffer_unmap (buf, &map);
   }
 
   return ret;
@@ -460,42 +472,41 @@ get_frame_size (GstMpegvParse * mpvparse, GstBuffer * buf, GList * l_codoffsz)
   return -1;
 }
 
-/* FIXME move into baseparse, or anything equivalent;
- * see https://bugzilla.gnome.org/show_bug.cgi?id=650093 */
-#define GST_BASE_PARSE_FRAME_FLAG_PARSING   0x10000
-
 static inline void
 update_frame_parsing_status (GstMpegvParse * mpvparse,
     GstBaseParseFrame * frame)
 {
   /* avoid stale cached parsing state */
-  if (!(frame->flags & GST_BASE_PARSE_FRAME_FLAG_PARSING)) {
+  if (frame->flags & GST_BASE_PARSE_FRAME_FLAG_NEW_FRAME) {
     GST_LOG_OBJECT (mpvparse, "parsing new frame");
     gst_mpegv_parse_reset_frame (mpvparse);
-    frame->flags |= GST_BASE_PARSE_FRAME_FLAG_PARSING;
   } else {
     GST_LOG_OBJECT (mpvparse, "resuming frame parsing");
   }
 }
 
 
-static gboolean
-gst_mpegv_parse_check_valid_frame (GstBaseParse * parse,
-    GstBaseParseFrame * frame, guint * framesize, gint * skipsize)
+static GstFlowReturn
+gst_mpegv_parse_handle_frame (GstBaseParse * parse,
+    GstBaseParseFrame * frame, gint * skipsize)
 {
   GstMpegvParse *mpvparse = GST_MPEGVIDEO_PARSE (parse);
   GstBuffer *buf = frame->buffer;
   gboolean ret = FALSE;
   GList *tmp;
   gint off = 0, fsize = -1;
+  GstMapInfo map;
+  gsize buf_size;
 
   update_frame_parsing_status (mpvparse, frame);
 
   if (mpvparse->last_sc >= 0)
     off = mpvparse->last_sc;
 
-  mpvparse->typeoffsize =
-      gst_mpeg_video_parse (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf), off);
+  gst_buffer_map (buf, &map, GST_MAP_READ);
+  buf_size = map.size;
+  mpvparse->typeoffsize = gst_mpeg_video_parse (map.data, map.size, off);
+  gst_buffer_unmap (buf, &map);
 
   /* No sc found */
   if (mpvparse->typeoffsize == NULL)
@@ -529,23 +540,20 @@ gst_mpegv_parse_check_valid_frame (GstBaseParse * parse,
 
 end:
   if (fsize > 0) {
-    *framesize = fsize;
     ret = TRUE;
   } else if (GST_BASE_PARSE_DRAINING (parse)) {
-    *framesize = GST_BUFFER_SIZE (buf);
+    fsize = buf_size;
     ret = TRUE;
 
   } else {
     /* resume scan where we left it */
     if (!mpvparse->last_sc)
-      *skipsize = mpvparse->last_sc = GST_BUFFER_SIZE (buf) - 3;
+      *skipsize = mpvparse->last_sc = buf_size - 3;
     else if (mpvparse->typeoffsize)
-      mpvparse->last_sc = GST_BUFFER_SIZE (buf) - 3;
+      mpvparse->last_sc = buf_size - 3;
     else
       *skipsize = 0;
 
-    /* request best next available */
-    *framesize = G_MAXUINT;
     ret = FALSE;
   }
 
@@ -553,7 +561,18 @@ end:
   g_list_free (mpvparse->typeoffsize);
   mpvparse->typeoffsize = NULL;
 
-  return ret;
+  if (ret) {
+    GstFlowReturn res;
+
+    *skipsize = 0;
+    g_assert (fsize <= buf_size);
+    res = gst_mpegv_parse_parse_frame (parse, frame);
+    if (res == GST_BASE_PARSE_FLOW_DROPPED)
+      frame->flags |= GST_BASE_PARSE_FRAME_FLAG_DROP;
+    return gst_base_parse_finish_frame (parse, frame, fsize);
+  }
+
+  return GST_FLOW_OK;
 }
 
 static void
@@ -562,16 +581,16 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
   GstCaps *caps = NULL;
 
   /* only update if no src caps yet or explicitly triggered */
-  if (G_LIKELY (GST_PAD_CAPS (GST_BASE_PARSE_SRC_PAD (mpvparse)) &&
+  if (G_LIKELY (gst_pad_has_current_caps (GST_BASE_PARSE_SRC_PAD (mpvparse)) &&
           !mpvparse->update_caps))
     return;
 
   /* carry over input caps as much as possible; override with our own stuff */
-  caps = GST_PAD_CAPS (GST_BASE_PARSE_SINK_PAD (mpvparse));
+  caps = gst_pad_get_current_caps (GST_BASE_PARSE_SINK_PAD (mpvparse));
   if (caps) {
-    caps = gst_caps_copy (caps);
+    caps = gst_caps_make_writable (caps);
   } else {
-    caps = gst_caps_new_simple ("video/mpeg", NULL);
+    caps = gst_caps_new_empty_simple ("video/mpeg");
   }
 
   /* typically we don't output buffers until we have properly parsed some
@@ -624,18 +643,19 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
      *                      3 => SNR Scalable, 4 => Main, 5 => Simple
      * 4:2:2 and Multi-view have profile = 0, with the escape bit set to 1
      */
-    const gchar *profiles[] = { "high", "spatial", "snr", "main", "simple" };
+    const gchar *const profiles[] =
+        { "high", "spatial", "snr", "main", "simple" };
     /*
      * Level indication - 4 => High, 6 => High-1440, 8 => Main, 10 => Low,
      *                    except in the case of profile = 0
      */
-    const gchar *levels[] = { "high", "high-1440", "main", "low" };
+    const gchar *const levels[] = { "high", "high-1440", "main", "low" };
 
     if (profile_c > 0 && profile_c < 6)
       profile = profiles[profile_c - 1];
 
     if ((level_c > 3) && (level_c < 11) && (level_c % 2 == 0))
-      level = levels[(level_c >> 1) - 1];
+      level = levels[(level_c >> 1) - 2];
 
     if (profile_c == 8) {
       /* Non-hierarchical profile */
@@ -663,6 +683,8 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
 
     /* FIXME does it make sense to expose profile/level in the caps ? */
 
+    GST_DEBUG_OBJECT (mpvparse, "profile:'%s' level:'%s'", profile, level);
+
     if (profile)
       gst_caps_set_simple (caps, "profile", G_TYPE_STRING, profile, NULL);
     else
@@ -726,13 +748,11 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 
     /* codec tag */
     codec = g_strdup_printf ("MPEG %d Video", mpvparse->mpeg_version);
-    taglist = gst_tag_list_new ();
-    gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE,
-        GST_TAG_VIDEO_CODEC, codec, NULL);
+    taglist = gst_tag_list_new (GST_TAG_VIDEO_CODEC, codec, NULL);
     g_free (codec);
 
-    gst_element_found_tags_for_pad (GST_ELEMENT (mpvparse),
-        GST_BASE_PARSE_SRC_PAD (mpvparse), taglist);
+    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (mpvparse),
+        gst_event_new_tag (taglist));
 
     mpvparse->send_codec_tag = FALSE;
   }
@@ -760,7 +780,7 @@ gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
     /* best possible parse attempt,
      * src caps are based on sink caps so it will end up in there
      * whether sucessful or not */
-    gst_mpegv_parse_process_config (mpvparse, buf, GST_BUFFER_SIZE (buf));
+    gst_mpegv_parse_process_config (mpvparse, buf, gst_buffer_get_size (buf));
   }
 
   /* let's not interfere and accept regardless of config parsing success */
@@ -768,7 +788,7 @@ gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
 }
 
 static GstCaps *
-gst_mpegv_parse_get_caps (GstBaseParse * parse)
+gst_mpegv_parse_get_caps (GstBaseParse * parse, GstCaps * filter)
 {
   GstCaps *peercaps;
   GstCaps *res;
