@@ -28,12 +28,6 @@
 
 #include <gst/video/video.h>
 
-static const GstElementDetails gst_dshowvideosrc_details =
-GST_ELEMENT_DETAILS ("DirectShow video capture source",
-    "Source/Video",
-    "Receive data from a directshow video capture graph",
-    "Sebastien Moutte <sebastien@moutte.net>");
-
 GST_DEBUG_CATEGORY_STATIC (dshowvideosrc_debug);
 #define GST_CAT_DEFAULT dshowvideosrc_debug
 
@@ -42,6 +36,7 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_BGR ";"
         GST_VIDEO_CAPS_YUV ("{ I420 }") ";"
+        GST_VIDEO_CAPS_YUV ("{ YUY2 }") ";"
         "video/x-dv,"
         "systemstream = (boolean) FALSE,"
         "width = (int) [ 1, MAX ],"
@@ -101,8 +96,8 @@ static GstCaps *gst_dshowvideosrc_getcaps_from_streamcaps (GstDshowVideoSrc *
     src, IPin * pin);
 static GstCaps *gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc *
     src, IPin * pin);
-static gboolean gst_dshowvideosrc_push_buffer (byte * buffer, long size,
-    gpointer src_object, UINT64 start, UINT64 stop);
+static gboolean gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size,
+    gpointer src_object, GstClockTime duration);
 
 static void
 gst_dshowvideosrc_init_interfaces (GType type)
@@ -134,7 +129,10 @@ gst_dshowvideosrc_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details (element_class, &gst_dshowvideosrc_details);
+  gst_element_class_set_details_simple (element_class,
+      "DirectShow video capture source", "Source/Video",
+      "Receive data from a directshow video capture graph",
+      "Sebastien Moutte <sebastien@moutte.net>");
 }
 
 static void
@@ -870,7 +868,7 @@ gst_dshowvideosrc_create (GstPushSrc * psrc, GstBuffer ** buf)
       gst_buffer_unref (*buf);
       *buf = NULL;
     }
-    return GST_FLOW_WRONG_STATE;
+    return GST_FLOW_FLUSHING;
   }
 
   GST_DEBUG ("dshowvideosrc_create => pts %" GST_TIME_FORMAT " duration %"
@@ -1005,11 +1003,11 @@ gst_dshowvideosrc_getcaps_from_enum_mediatypes (GstDshowVideoSrc * src, IPin * p
 }
 
 static gboolean
-gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
-    UINT64 start, UINT64 stop)
+gst_dshowvideosrc_push_buffer (guint8 * buffer, guint size, gpointer src_object,
+    GstClockTime duration)
 {
   GstDshowVideoSrc *src = GST_DSHOWVIDEOSRC (src_object);
-  GstBuffer *buf;
+  GstBuffer *buf = NULL;
   IPin *pPin = NULL;
   HRESULT hres = S_FALSE;
   AM_MEDIA_TYPE *pMediaType = NULL;
@@ -1022,9 +1020,13 @@ gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
   buf = gst_buffer_new_and_alloc (size);
 
   GST_BUFFER_SIZE (buf) = size;
-  GST_BUFFER_TIMESTAMP (buf) = gst_clock_get_time (GST_ELEMENT (src)->clock);
-  GST_BUFFER_TIMESTAMP (buf) -= GST_ELEMENT (src)->base_time;
-  GST_BUFFER_DURATION (buf) = stop - start;
+
+  GstClock *clock = gst_element_get_clock (GST_ELEMENT (src));
+  GST_BUFFER_TIMESTAMP (buf) =
+    GST_CLOCK_DIFF (gst_element_get_base_time (GST_ELEMENT (src)), gst_clock_get_time (clock));
+  gst_object_unref (clock);
+
+  GST_BUFFER_DURATION (buf) = duration;
 
   if (src->is_rgb) {
     /* FOR RGB directshow decoder will return bottom-up BITMAP
@@ -1044,7 +1046,7 @@ gst_dshowvideosrc_push_buffer (byte * buffer, long size, gpointer src_object,
 
   GST_DEBUG ("push_buffer => pts %" GST_TIME_FORMAT "duration %"
       GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-      GST_TIME_ARGS (stop - start));
+      GST_TIME_ARGS (duration));
 
   /* the negotiate() method already set caps on the source pad */
   gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
