@@ -18,6 +18,20 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-shmsrc
+ *
+ * Receive data from the shared memory sink.
+ *
+ * <refsect2>
+ * <title>Example launch lines</title>
+ * |[
+ * gst-launch shmsrc socket-path=/tmp/blah ! \
+ * "video/x-raw-yuv, format=(fourcc)YUY2, color-matrix=(string)sdtv, \
+ * chroma-site=(string)mpeg2, width=(int)320, height=(int)240, framerate=(fraction)30/1" ! autovideosink
+ * ]| Render video from shm buffers.
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -58,7 +72,8 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-GST_BOILERPLATE (GstShmSrc, gst_shm_src, GstPushSrc, GST_TYPE_PUSH_SRC);
+#define gst_shm_src_parent_class parent_class
+G_DEFINE_TYPE (GstShmSrc, gst_shm_src, GST_TYPE_PUSH_SRC);
 
 static void gst_shm_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -79,21 +94,6 @@ static void gst_shm_pipe_dec (GstShmPipe * pipe);
 
 // static guint gst_shm_src_signals[LAST_SIGNAL] = { 0 };
 
-
-static void
-gst_shm_src_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class, &srctemplate);
-
-  gst_element_class_set_details_simple (element_class,
-      "Shared Memory Source",
-      "Source",
-      "Receive data from the sharem memory sink",
-      "Olivier Crete <olivier.crete@collabora.co.uk>");
-}
-
 static void
 gst_shm_src_class_init (GstShmSrcClass * klass)
 {
@@ -131,11 +131,20 @@ gst_shm_src_class_init (GstShmSrcClass * klass)
           "True if the element cannot produce data in PAUSED", FALSE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "Shared Memory Source",
+      "Source",
+      "Receive data from the shared memory sink",
+      "Olivier Crete <olivier.crete@collabora.co.uk>");
+
   GST_DEBUG_CATEGORY_INIT (shmsrc_debug, "shmsrc", 0, "Shared Memory Source");
 }
 
 static void
-gst_shm_src_init (GstShmSrc * self, GstShmSrcClass * g_class)
+gst_shm_src_init (GstShmSrc * self)
 {
   self->poll = gst_poll_new (TRUE);
   gst_poll_fd_init (&self->pollfd);
@@ -305,14 +314,14 @@ gst_shm_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   do {
     if (gst_poll_wait (self->poll, GST_CLOCK_TIME_NONE) < 0) {
       if (errno == EBUSY)
-        return GST_FLOW_WRONG_STATE;
+        return GST_FLOW_FLUSHING;
       GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
           ("Poll failed on fd: %s", strerror (errno)));
       return GST_FLOW_ERROR;
     }
 
     if (self->unlocked)
-      return GST_FLOW_WRONG_STATE;
+      return GST_FLOW_FLUSHING;
 
     if (gst_poll_fd_has_closed (self->poll, &self->pollfd)) {
       GST_ELEMENT_ERROR (self, RESOURCE, READ, ("Failed to read from shmsrc"),
@@ -348,11 +357,9 @@ gst_shm_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   gst_shm_pipe_inc (self->pipe);
 
   *outbuf = gst_buffer_new ();
-  GST_BUFFER_FLAG_SET (*outbuf, GST_BUFFER_FLAG_READONLY);
-  GST_BUFFER_DATA (*outbuf) = (guint8 *) buf;
-  GST_BUFFER_SIZE (*outbuf) = rv;
-  GST_BUFFER_MALLOCDATA (*outbuf) = (guint8 *) gsb;
-  GST_BUFFER_FREE_FUNC (*outbuf) = free_buffer;
+  gst_buffer_append_memory (*outbuf,
+      gst_memory_new_wrapped (GST_MEMORY_FLAG_READONLY,
+          buf, rv, 0, rv, gsb, free_buffer));
 
   return GST_FLOW_OK;
 }
