@@ -44,7 +44,7 @@
  * Includes
  */
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 /* save the snapshot images
@@ -171,15 +171,15 @@ static const gchar *test_names[TEST_CASES] = {
 static void print_result (void);
 static gboolean run_test (gpointer user_data);
 static gboolean setup_add_pad_probe (GstElement * elem, const gchar * pad_name,
-    GCallback handler, gpointer data);
+    GstPadProbeCallback handler, gpointer data);
 
 
 /*
  * Callbacks
  */
 
-static gboolean
-pad_has_buffer (GstPad * pad, GstBuffer * buf, gpointer user_data)
+static GstPadProbeReturn
+pad_has_buffer (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
 {
   gboolean *signal_sink = (gboolean *) user_data;
   gboolean print_and_restart = FALSE;
@@ -197,7 +197,7 @@ pad_has_buffer (GstPad * pad, GstBuffer * buf, gpointer user_data)
       case 8:                  // video recording start
         DIFF_TIME (t_final[num_pics_cont], t_initial, diff);
         result.avg = result.min = result.max = diff;
-        //g_signal_emit_by_name (camera_bin, "user-stop", 0);
+        //g_signal_emit_by_name (camera_bin, "capture-stop", 0);
         print_and_restart = TRUE;
         break;
       default:
@@ -209,7 +209,7 @@ pad_has_buffer (GstPad * pad, GstBuffer * buf, gpointer user_data)
     print_result ();
     g_idle_add ((GSourceFunc) run_test, NULL);
   }
-  return TRUE;
+  return GST_PAD_PROBE_OK;
 }
 
 static void
@@ -223,24 +223,22 @@ element_added (GstBin * bin, GstElement * element, gpointer user_data)
   }
 
   if (need_vmux_pad_probe) {
-    g_object_get (camera_bin, "videomux", &elem, NULL);
+    g_object_get (camera_bin, "video-muxer", &elem, NULL);
     if (elem) {
       need_vmux_pad_probe = FALSE;
       GST_INFO_OBJECT (elem, "got default video muxer");
-      if (setup_add_pad_probe (elem, "src", (GCallback) pad_has_buffer,
-              &signal_vid_sink)) {
+      if (setup_add_pad_probe (elem, "src", pad_has_buffer, &signal_vid_sink)) {
         /* enable test */
         target[8] = test_09_taget;
       }
     }
   }
   if (need_ienc_pad_probe) {
-    g_object_get (camera_bin, "imageenc", &elem, NULL);
+    g_object_get (camera_bin, "image-encoder", &elem, NULL);
     if (elem) {
       need_ienc_pad_probe = FALSE;
       GST_INFO_OBJECT (elem, "got default image encoder");
-      if (setup_add_pad_probe (elem, "src", (GCallback) pad_has_buffer,
-              &signal_img_enc)) {
+      if (setup_add_pad_probe (elem, "src", pad_has_buffer, &signal_img_enc)) {
         /* enable test */
         target[5] = test_06_taget;
       }
@@ -291,7 +289,7 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       // but this needs sync so that we have received "image-captured" message already
       if (have_img_captured) {
         have_img_captured = FALSE;
-        g_signal_emit_by_name (camera_bin, "user-start", NULL);
+        g_signal_emit_by_name (camera_bin, "capture-start", NULL);
       } else {
         have_img_done = TRUE;
       }
@@ -300,8 +298,6 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       GstClockTime max = 0;
       GstClockTime min = -1;
       GstClockTime total = 0;
-      GstClockTime first_shot = 0;
-      GstClockTime snd_shot = 0;
 
       num_pics_cont = 0;
       signal_cont = FALSE;
@@ -309,13 +305,11 @@ img_capture_done (GstElement * camera, GString * fname, gpointer user_data)
       DIFF_TIME (t_final[0], t_initial, diff);
       max < diff ? max = diff : max;
       min > diff ? min = diff : min;
-      first_shot = diff;
       total += diff;
 
       DIFF_TIME (t_final[1], t_final[0], diff);
       max < diff ? max = diff : max;
       min > diff ? min = diff : min;
-      snd_shot = diff;
       total += diff;
 
       for (i = 2; i < CONT_SHOTS; ++i) {
@@ -416,7 +410,7 @@ bus_callback (GstBus * bus, GstMessage * message, gpointer data)
               // we need to have this received before we can take next one
               if (have_img_done) {
                 have_img_done = FALSE;
-                g_signal_emit_by_name (camera_bin, "user-start", NULL);
+                g_signal_emit_by_name (camera_bin, "capture-start", NULL);
               } else {
                 have_img_captured = TRUE;
               }
@@ -484,7 +478,7 @@ cleanup_pipeline (void)
 
 static gboolean
 setup_add_pad_probe (GstElement * elem, const gchar * pad_name,
-    GCallback handler, gpointer data)
+    GstPadProbeCallback handler, gpointer data)
 {
   GstPad *pad = NULL;
 
@@ -493,7 +487,8 @@ setup_add_pad_probe (GstElement * elem, const gchar * pad_name,
     return FALSE;
   }
 
-  gst_pad_add_buffer_probe (pad, (GCallback) handler, data);
+  gst_pad_add_probe (pad, GST_PAD_PROBE_TYPE_BLOCK | GST_PAD_PROBE_TYPE_BUFFER,
+      handler, data, NULL);
   gst_object_unref (pad);
 
   return TRUE;
@@ -538,7 +533,8 @@ setup_pipeline (void)
     goto error;
   }
 
-  g_signal_connect (camera_bin, "img-done", (GCallback) img_capture_done, NULL);
+  g_signal_connect (camera_bin, "image-done", (GCallback) img_capture_done,
+      NULL);
 
   bus = gst_pipeline_get_bus (GST_PIPELINE (camera_bin));
   gst_bus_add_watch (bus, bus_callback, NULL);
@@ -547,29 +543,35 @@ setup_pipeline (void)
   GST_INFO_OBJECT (camera_bin, "camerabin created");
 
   /* configure used elements */
-  res &= setup_pipeline_element ("vfsink", "fakesink", &sink);
-  res &= setup_pipeline_element ("audiosrc", audiosrc_name, NULL);
-  res &= setup_pipeline_element ("videosrc", videosrc_name, NULL);
-  res &= setup_pipeline_element ("audioenc", audioenc_name, NULL);
-  res &= setup_pipeline_element ("videoenc", videoenc_name, NULL);
-  res &= setup_pipeline_element ("imageenc", imageenc_name, &ienc);
-  res &= setup_pipeline_element ("videomux", videomux_name, &vmux);
+  res &= setup_pipeline_element ("viewfinder-sink", "fakesink", &sink);
+  res &= setup_pipeline_element ("audio-source", audiosrc_name, NULL);
+  res &= setup_pipeline_element ("video-source", videosrc_name, NULL);
+  res &= setup_pipeline_element ("audio-encoder", audioenc_name, NULL);
+  res &= setup_pipeline_element ("video-encoder", videoenc_name, NULL);
+  res &= setup_pipeline_element ("image-encoder", imageenc_name, &ienc);
+  res &= setup_pipeline_element ("video-muxer", videomux_name, &vmux);
   if (!res) {
     goto error;
   }
 
   GST_INFO_OBJECT (camera_bin, "elements created");
 
+  if (GST_STATE_CHANGE_FAILURE ==
+      gst_element_set_state (camera_bin, GST_STATE_READY)) {
+    g_warning ("can't set camerabin to ready\n");
+    goto error;
+  }
+  GST_INFO_OBJECT (camera_bin, "camera ready");
+
   /* set properties */
   g_object_set (camera_bin, "filename", filename->str, NULL);
 
-  if (src_csp && strlen (src_csp) == 4) {
+  if (src_csp) {
     GstCaps *filter_caps;
 
     /* FIXME: why do we need to set this? */
-    filter_caps = gst_caps_new_simple ("video/x-raw-yuv",
-        "format", GST_TYPE_FOURCC,
-        GST_MAKE_FOURCC (src_csp[0], src_csp[1], src_csp[2], src_csp[3]), NULL);
+    filter_caps = gst_caps_new_simple ("video/x-raw",
+        "format", G_TYPE_STRING, src_csp, NULL);
     if (filter_caps) {
       g_object_set (camera_bin, "filter-caps", filter_caps, NULL);
       gst_caps_unref (filter_caps);
@@ -585,12 +587,11 @@ setup_pipeline (void)
 
   /* connect signal handlers */
   g_assert (sink);
-  if (!setup_add_pad_probe (sink, "sink", (GCallback) pad_has_buffer,
-          &signal_vf_sink)) {
+  if (!setup_add_pad_probe (sink, "sink", pad_has_buffer, &signal_vf_sink)) {
     goto error;
   }
   if (!vmux) {
-    g_object_get (camera_bin, "videomux", &vmux, NULL);
+    g_object_get (camera_bin, "video-muxer", &vmux, NULL);
     if (!vmux) {
       need_pad_probe = need_vmux_pad_probe = TRUE;
       /* only run the test if we later get the element */
@@ -599,13 +600,12 @@ setup_pipeline (void)
     }
   }
   if (vmux) {
-    if (!setup_add_pad_probe (vmux, "src", (GCallback) pad_has_buffer,
-            &signal_vid_sink)) {
+    if (!setup_add_pad_probe (vmux, "src", pad_has_buffer, &signal_vid_sink)) {
       goto error;
     }
   }
   if (!ienc) {
-    g_object_get (camera_bin, "imageenc", &ienc, NULL);
+    g_object_get (camera_bin, "image-encoder", &ienc, NULL);
     if (!ienc) {
       need_pad_probe = need_ienc_pad_probe = TRUE;
       /* only run the test if we later get the element */
@@ -614,8 +614,7 @@ setup_pipeline (void)
     }
   }
   if (ienc) {
-    if (!setup_add_pad_probe (ienc, "src", (GCallback) pad_has_buffer,
-            &signal_img_enc)) {
+    if (!setup_add_pad_probe (ienc, "src", pad_has_buffer, &signal_img_enc)) {
       goto error;
     }
   }
@@ -627,22 +626,15 @@ setup_pipeline (void)
 
   /* configure a resolution and framerate for video and viewfinder */
   if (image_width && image_height) {
-    g_signal_emit_by_name (camera_bin, "user-image-res", image_width,
+    g_signal_emit_by_name (camera_bin, "set-image-resolution", image_width,
         image_height, NULL);
   }
   /* configure a resolution and framerate for video and viewfinder */
   if (video_width && video_height && view_framerate_num && view_framerate_den) {
-    g_signal_emit_by_name (camera_bin, "user-res-fps", video_width,
+    g_signal_emit_by_name (camera_bin, "set-video-resolution-fps", video_width,
         video_height, view_framerate_num, view_framerate_den, NULL);
   }
 
-  if (GST_STATE_CHANGE_FAILURE ==
-      gst_element_set_state (camera_bin, GST_STATE_READY)) {
-    g_warning ("can't set camerabin to ready\n");
-    goto error;
-  }
-  GST_INFO_OBJECT (camera_bin, "camera ready");
-
   if (GST_STATE_CHANGE_FAILURE ==
       gst_element_set_state (camera_bin, GST_STATE_PLAYING)) {
     g_warning ("can't set camerabin to playing\n");
@@ -706,7 +698,7 @@ test_03 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
 
   /* the actual results are fetched in bus_callback::preview-image */
   result.times = 1;
@@ -723,7 +715,7 @@ test_04 (void)
   /* switch to image mode */
   g_object_set (camera_bin, "mode", 0, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
 
   /* the actual results are fetched in bus_callback::image-captured */
   result.times = 1;
@@ -743,7 +735,7 @@ test_05 (void)
   /* switch to image mode */
   g_object_set (camera_bin, "mode", 0, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
 
   /* the actual results are fetched in img_capture_done */
   result.times = CONT_SHOTS;
@@ -764,7 +756,7 @@ test_06 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
 
   /* the actual results are fetched in pad_has_buffer */
   result.times = 1;
@@ -786,7 +778,7 @@ test_07 (void)
   g_object_set (camera_bin, "mode", 0, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
   /* the actual results are fetched in img_capture_done */
   result.times = 1;
   return FALSE;
@@ -851,7 +843,7 @@ test_09 (void)
   g_object_set (camera_bin, "mode", 1, NULL);
   g_object_set (camera_bin, "filename", filename->str, NULL);
   GET_TIME (t_initial);
-  g_signal_emit_by_name (camera_bin, "user-start", 0);
+  g_signal_emit_by_name (camera_bin, "capture-start", 0);
 
   /* the actual results are fetched in pad_has_buffer */
   result.times = 1;
@@ -982,9 +974,6 @@ main (int argc, char *argv[])
   GOptionContext *ctx;
   GError *err = NULL;
 
-  if (!g_thread_supported ())
-    g_thread_init (NULL);
-
   ctx = g_option_context_new (NULL);
   g_option_context_add_main_entries (ctx, options, NULL);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
