@@ -20,6 +20,9 @@
  *
  */
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GValueArray
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
 
 #include <gst/gst.h>
 #include <string.h>
@@ -403,8 +406,8 @@ gst_element_print_properties (GstElement * element)
               pfraction->def_num, pfraction->def_den);
         }
 
-        else if (GST_IS_PARAM_SPEC_MINI_OBJECT (param)) {
-          g_print ("%-*.*s | MiniObject of type \"%s\"",
+        else if (G_IS_PARAM_SPEC_BOXED (param)) {
+          g_print ("%-*.*s | Boxed of type \"%s\"",
               c3w, c3w,
               g_type_name (param->value_type), g_type_name (param->value_type));
         }
